
const should            = require('should');

const Helper            = require('../test/helper');
const dbHelper          = require('../test/dbHelper');
const Logger            = require('../service/logger');
const Woveon            = require('../service/woveon');
const Service           = require('../service/service');
let   config            = require('../service/config');


// --------------------------------------------------------------------- 
// Make timestamps without timezone assume Z timezone, instead of local
let types = require('pg').types;
const TIMESTAMP_OID = 1114;
let parseFn = function(val) {
//  console.log(`parseFn date string ${val}`);
  return new Date(`${val}Z`);
};
types.setTypeParser(TIMESTAMP_OID, parseFn)
// --------------------------------------------------------------------- 


let teamchannel = {
  pluginId : process.env.WOV_plugin_id,
  teamId   : process.env.WOV_plugin_team_id,
  token    : null,
};


const logger   = new Logger('L',
  {showName: true, debug: true, level: 'verbose'});
const toWoveon = new Woveon(config, logger);
let pg         = null;

/**
 * Written like this to allow await/async calls.
 */
( async function() {

  logger.info('Starting...');
  pg = await dbHelper.getWoveonDBConnection();

  // ---------------------------------------------------------------------
  // grab first team that matches
  //   (this means we skip all the auth since we grab the channel token here)
  // ---------------------------------------------------------------------
  let teamq = await pg.query(`SELECT * FROM teamchannel WHERE teamid='${teamchannel.teamId}' and applicationid='${teamchannel.pluginId}' limit 1`);
  // console.log('teamq:', teamq.rows);
  teamchannel.token = teamq.rows[0].token;
  logger.info('teamchannel: ', teamchannel);


  // ---------------------------------------------------------------------
  // generate the post and comment
  // ---------------------------------------------------------------------
  let rawP = Woveon.genPost( teamchannel.token, 'Carl Larson',
    'https://scontent.xx.fbcdn.net/v/t1.0-1/c15.0.50.50/p50x50/10354686_10150004552801856_220367501106153455_n.jpg?oh=8a3549a50c99193e342e22c66b375815&oe=5AC4632F',
    Service.generateRandomString(), 'Facebook', 'Another post to',
    false, [], [], '2017-11-02T13:29:57.000Z');
  let rawC = Woveon.genComment(
    teamchannel.token, 'Brads Local Retail',
    'https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7',
    rawP.postId, Service.generateRandomString(), 'Blah again.',
    '', '', '2017-11-02T13:32:03.000Z');
  
  logger.info('Creating Post    : ', rawP);
  logger.info('Creating Comment : ', rawC);


  // ---------------------------------------------------------------------
  // send to Woveon
  // --------------------------------------------------------------------- 
  let resultP = await toWoveon.createPost(rawP)
    .catch( (err) => { console.log(err); console.log(rawP); throw new Error('Error'); });
  logger.info('Post result    : ', resultP);
  should.notEqual(resultP.data, null);

  let resultC = await toWoveon._createComment(rawC)
    .catch( (err) => { console.log(err); console.log(rawC); throw new Error('Error'); });
  logger.info('Comment result  : ', resultC);
  should.notEqual(resultC.data, null);


  // --------------------------------------------------------------------- 
  // Query the database to see if it went through
  // --------------------------------------------------------------------- 
  await dbHelper.comparePostToDB(pg, rawP, resultP.data.id);
  console.log('compare Post to DB: success');

  await dbHelper.compareCommentToDB(pg, rawC, resultC.data.id);
  console.log('compare Comment to DB: success');


  /*
  { token: 'BPEUE1D5LH9I1JNTX468',
  author: 'Brads Local Retail',
  authorUrl: 'https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7',
  postId: '142118089752457_148634255767507',
  commentId: '148634255767507_148638665767066',
  message: 'testing comment post 11',
  images: [],
  video: [],
  date: 2017-11-30T20:04:45.000Z },
  */























  await pg.end();
})()
.catch( (err)=> {console.log(err); if ( pg != null ) pg.end(); process.exit(1);});

