


# Rules usefulto microservices

# ---------------------------------------------------------------------
# ---------------------------------------------------------------------
all:
	@echo
	@echo "*** Build and Run"
	@echo "'make run-{microservice}' : start the microservice"
	@echo "'make devrun-{microservice}' : start the microservice with nodemon"
	@echo
	@echo "*** Utility"
	@echo "'make local-tests'    : run tests."
	@echo "'make local-tests TEST=test/X' : run test X in ./test directory (.js not required)"
	@echo "'make envs'           : display and test env vars are set"
	@echo "'make listen'         : runs the process that listens for events from the plugin... for dev/debug."
	@echo "'make send'           : sends a message to this service."
	@echo


# Check that these env vars are set. If usign Woveon Tools, it manages for you.
check_pass_and_secrets: .FORCE
ifeq ($(HASWOVTOOLS),0)

ifeq ($(WOV_we_apisecret),)
$(warning 'ERROR: WOV_we_apisecret not set, use: "export WOV_we_apisecret=`openssl rand -hex 40`"')
endif

ifeq (${WOV_we_datasecret},)
$(warning 'ERROR: WOV_we_datasecret not set, use:  "export WOV_we_datasecret=`openssl rand -hex 40`"')
endif

ifeq (${WOV_testuser_email},)
$(warning 'ERROR: WOV_testuser_email not set (testing account name), use:  "export WOV_testuser_email=tester@woveon.net"')
endif

ifeq (${WOV_testuser_password},)

$(warning 'ERROR: WOV_testuser_password not set, use:  "export WOV_testuser_password=XXX" ... jq -r ".testuser.password" `wov-cd -e secrets`/cw_secrets/ple.json')
endif

ifeq ($(WOV_we_apisecret),)
$(error )
endif
ifeq (${WOV_we_datasecret},)
$(error )
endif
ifeq (${WOV_testuser_email},)
$(error )
endif
ifeq (${WOV_testuser_password},)
$(error )
endif

else
	@echo

endif


# Display and check env vars
envs:
	@printf "\nEnvironment variables used:\n"
	@echo ${ENVS}
	@printf "\n...debugging WOV_LOGGER_ASPECTS: \n"
	@printf '   suggested: export WOV_LOGGER_ASPECTS="listener.route listener.incoming listener.result requester"\n'
	@printf '     current: export WOV_LOGGER_ASPECTS="${WOV_LOGGER_ASPECTS}"\n'
	@make check_pass_and_secrets

testserver :
	@echo envs_rules.mk
	cd node_modules/woveon-engine-p/testserver ; ${ENVS} node src/index

# Test when using bastion security
pre-local-test: pre-local-test-additions .FORCE
	@echo "- $(@) --------------------------------------------------------------"
	@echo " ... test mongodb connection is active"
	@mongo --eval "printjson(\"\")" > /dev/null 2>&1 || (echo "FAILED: run mongo in docker locally via 'docker run --name mongo -p 27017:27017 -d mongo:3.6-jessie', or restart an existing 'docker ps -a' to find them and 'docker restart [CONTAINERID]' to restart." ; exit 1)
	@echo " ... test testserver is running"
	@curl -q http://localhost:${TESTSERVERPORT}/api/v1/health > /dev/null 2>&1 || (echo "FAILED: 'make testserver', to start the testserver" ; exit 1)

tests : 
	@echo "- $(@) --------------------------------------------------------------"
	@echo "run 'make local-tests' for testing plugins locally."
	@echo ""

# Switched to bastion, so I can clean this up
local-tests : pre-local-test check_pass_and_secrets .FORCE
	${ENVS} mocha -b $$TEST

local-tests-nyc : pre-local-test check_pass_and_secrets .FORCE
	${ENVS} ./node_modules/nyc/bin/nyc.js --reporter=text --reporter=html mocha -b $$TEST

