
// 'use strict';
// let mongoose = require('mongoose');
const DB = require('../service/db');
let Schema = DB.Mongoose.Schema;
const Long       = DB.Mongoose.Types.Long;

// let mongodb  = require('mongodb');

/*
Channel Document values:
{
  "__v"       : 0,
  "_id"       :  // Mongo identifier
  "token"     : // channel token of the channel here and in Woveon API
  "remotelistenerID"  : // mongo identifier of the remotelistener currently working on this
  "pageId"    : // Facebook identifier for the page this is watching
  "pageImage" : // icon to use for the page
  "pageName"  : // name of page
  "pageToken" : // (secret) extended token to use to access the page
}

example entry for a Facebook Channel:
{
  "_id" : ObjectId("5a132e6422702a889de90102"),
  "token" : "UGY9I5GFUNXPIFEWSZQ8",
  "__v" : 0,
  "remotelistenerID" : "5a132e6422702a889de90101",
  "pageId" : "143382282958865",
  "pageImage" : "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/22851676_143382639625496_4485385941938763387_n.png?oh=734b9f0361cf62dba8f4fd1bddd65dcf&oe=5A918DAB",
  "pageName" : "Brads Retail",
  "pageToken" : "EAAZATpRQZAi3QBADbZBhsRrf3OoZBYqKJAnRrZB4sl6swZBdIP9xL50jnLrF4kdvJ5VZBUwHjhxvZCgxq1meZAi6nPITbwD9bspWT4vmClkJKidF950blGhoDIsUTGl7CBokbJbCfKqicOLVkXNntNfaxNis2znFm7HCMHA6U0b0XJYHtODulEbtpRYyLJVb5lFYZD"
}
*/

module.exports = (_db) => {
  let retval = null;
  let ChannelSchema = null;

  try {
    SipBlockSchema = new Schema({
      finished : {type : Boolean, default: false},
      lid      : {type : Long, default : null},
      pid      : {type : Long, default : null},
      l        : Long,
      p        : Long,
      f        : {type : Long, default : null},
    });
  } catch (e) {
  }

  try {
  ChannelSchema = new Schema({
    token            : {type : String, default : null}, // an identifying token, not secret

    // handle parent/sub relationships.
    parentchannel    : {type : String, default : null},   // token of parent. null if this is top most
    subchannels      : [{type : String, default : null}], // tokens of children

    // The RemoteListener microservice that is waiting to hear from the RemoteService.
    remotelistenerID : {type : String, default : null},
      // { f:, p:, l:, finished:, pid:, lid: }

    // isActive in plugins has different states, not just a boolean value as in API
    isActive         : {
      type    : String,
      enum    : [
        'initing',   // browser just asked to create the channel, so entry is in db; plugin has yet to respond
        'starting',  // plugin is now trying to start it up.
        'running',   // plugin has started it successfully
        'inactive',  // channel is valid but nothing listening to it. Watcher or draining RemoteListener has turned it off
        'error',     // something broke... not touching for now.
      ],
      default : 'initing',
    },
    errorMessage : String, // for when isActive is set to error, it sets this

    lastUpdate : String, // ??? is used?


    // plugin-specific, stored as a JSON object
    pluginData : {type : Schema.Types.Mixed, default : {}}, // Mixed

    /*
    pageId    : String,
    pageImage : String,
    pageName  : String,
    pageToken : String, // sensitive, so encrypt it!
    */

    // sipBlocks : { type : 'Mixed', default : []},
    sipBlocks : [SipBlockSchema],
    e_mon        : {type : Long, default : null}, // monitoring since time
    // e_mon        : {type : 'Mixed', default : null}, // monitoring since time
    // e_mon        : {type : mongodb.Long, default : null}, // monitoring since time
    // e_mon        : {type : mongoose.Schema.Types.Long, default : null}, // monitoring since time

    // Engine variables to define "sipping" of the Remote Service data
    // NOTE: These are timestamps in 64 bit long format of milliseconds since epoc.
    //       To get a date object, just "new Date(val)".
    /*
    e_first      : {type : mongoose.Schema.Types.Long, default : null},
    e_earliest   : {type : mongoose.Schema.Types.Long, default : null},
    e_latest     : {type : mongoose.Schema.Types.Long, default : null},
    e_earliestID : {type : String, default : null},
    e_latestID   : {type : String, default : null},
    */

    //  remotelistenerIP   : String,
    //  remotelistenerPort : String,
  }, {
    timestamps : true,    // createdAt, updatedAt
    minimize   : false,   // keep {} and other 'minimizable' data
  });
  } catch (err) {
    console.log(err);
    throw new Error('Failed to create a schema for Channel.');
  }

  // try to grab cached. note, since Mongoose caches them inernally, and not
  // per connection, I am doing this.
  try {
    retval = _db.connection.model('Channels');
  } catch (err) {retval = _db.connection.model('Channels', ChannelSchema);}

  return retval;
};
