

/**
NOTE: removed since they still need a return trip to Woveon... to Warehose the data.

 * Tracks messages originating from Woveon, on a channel, with the remote service identifier.
 *
 * Messages from Woveon have an entry placed here, so messages back from the remote service do not
 * go back to Woveon. Used to block create and delete.
 *
 * Messages types includes: post, comment, like
 *
 *   WoveonListener: (from Woveon)
 *     * create -> add entry here
 *     * update -> do nothing, as remote service might modify message so we need that return trip
 *         ex. image location, etc.
 *     * delete -> do nothing, since already have the post in the db to block remote return trip
 *
 *   RemoteListener: (from RS)
 *     * create -> check entry, reject if found
 *     * update -> no check needed, just forward result back... but check error codes just in case
 *     * delete -> check entry, reject if found
 */


const DB = require('../service/db');
const Schema = DB.Mongoose.Schema;


module.exports = (_db) => {
  let retval = null;
  let BlockMsgSchema = null;
  let BlockMsg2Schema = null;

  // Define Schema for use in multiple models
  // ---------------------------------------------------------------------
  try {
    BlockMsgSchema = new Schema({
      token    : {type: String, default: null}, // an identifying channel token, not secret
      remoteId : {type: String},                // remote service identifier for the msg (post, comment, like).
    }, {timestamps: true}); // createdAt, updatedAt;
  } catch (err) {
    console.log(err);
    throw new Error('Failed to create a schema for WovMsg.');
  }

  try {
    BlockMsg2Schema = new Schema({
      token    : {type: String, default: null}, // an identifying channel token, not secret
      remoteId : {type: String},                // remote service identifier for the msg (post, comment, like).
      secondId : {type: String},                // Facebook commentId for example
    }, {timestamps: true}); // createdAt, updatedAt;
  } catch (err) {
    console.log(err);
    throw new Error('Failed to create a schema for WovMsg.');
  }


  // Generate Models
  // ---------------------------------------------------------------------
  // * try to grab cached. note, since Mongoose caches them inernally, and not
  //   per connection, I am doing this so testing scripts can create/delete these models.
  try {
    retval = {
      Posts    : _db.connection.model('WovPosts'),
      Comments : _db.connection.model('WovComments'),
      Likes    : _db.connection.model('WovLikes'),
    };
  } catch (err) {
    retval = {
      Posts    : _db.connection.model('WovPosts', BlockMsgSchema),
      Comments : _db.connection.model('WovComments', BlockMsg2Schema),
      Likes    : _db.connection.model('WovLikes', BlockMsg2Schema),
    };
  }

  return retval;
};

