let mongoose = require('mongoose');
mongoose.Promise = Promise;

// Add in types 
require('mongoose-long')(mongoose);


// if ( global.wov_dbidgen == null) global.wov_dbidgen = 1;
// let getId = (function() { let id = 1; return () => { return id++; }; })();
/**
 * A simple wrapper for loading up the mongo database.
 */
module.exports = class DB {


  /**
   * Start it up.
   * @param {Logger} _logger
   */
  constructor(_logger) {

    // see https://mongoosejs.com/docs/deprecations.html
    mongoose.set('useFindAndModify', false);

    this.logger = _logger;
    this.id = `${Math.floor(Math.random()*1000)}-${new Date().getTime()}`; // wov_dbidgen++; //getId();
//    this.logger.info('New DB id ', this.id);
    if ( this.logger == null ) throw new Error('DB logger is null');
    this.connection = null;
    if ( mongoose.wovconnections == null ) mongoose.wovconnections = {};
  }


  /**
   * Returns if mongoose connection to MongoDB has been established.
   * @return {bool}
   */
  isConnected() {
    let retval = false;
    if ( this.connection && this.connection.readyState == 1 ) {retval = true;}
    return retval;
  }


  /**
   * Initialize the mongoose connection to the MongoDB.
   * @param {*} _url
   * @return {promise}
   */
  async init(_url) {
    this.logger.verbose(`...DB start connection to MongoDB: ${_url}`);

    // mongoose.Promise = global.Promise;

    // mongoose is screwy so not going to pretty up this code
    // console.log('---Mongoose not open: readystate:', this.connection.readyState);
    this.logger.verbose('---Mongoose connect started: ', _url);
    //
    // 1
    this.connection = mongoose.createConnection(_url, {useNewUrlParser: true});
    // 2
    // mongoose.connect(_url, {}); // {'useMongoClient': true});
    // 3
    // mongoose.connect(_url, {'useMongoClient': true});
    // this.connection = mongoose.connection;
    //
    this.logger.verbose('---Mongoose connect started: readystate:', this.connection.readyState);

    return new Promise( (r, j) => {
      let that = this;
      this.connection.on('error', (err) => {
        this.logger.error('---Mongoose error: do you have a mongodb running locally? ');
        this.logger.error('\n\n type: `make local-db-start`     to start mongodb\n');
        // this.logger.warn(err);
        j(err);
      });
      this.connection.on('open', () => {
        this.logger.verbose('---Mongoose open: readystate:', this.connection.readyState, `id(${this.id})`);
        mongoose.wovconnections[this.id] = _url;//true;
        r(true);
      });
      this.connection.on('disconnected', function() {
        let that2 = that;
//        console.log(`---Mongoose connections: ${JSON.stringify(mongoose.wovconnections)})`);
        // that2.logger.verbose(`---Mongoose disconnected: id(${that2.id})`);
      });
    });
  }

  /**
   * Closes the connection to the mongoose db. However, it leaves the mongoose db open for other connections.
   */
  async close() {
    this.logger.aspect('dbdisconnect', 'close db connection', this.id);
    delete mongoose.wovconnections[this.id];
    this.logger.aspect('dbdisconnect', '---beforeha promise');
    await new Promise( function(res, rej) {
      this.logger.aspect('dbdisconnect', '---before connection close');
      this.connection.close(function(e) {
        this.logger.aspect('dbdisconnect', 'closing connection: ', e);
        if ( e ) rej(e);
        else res(true);
      }.bind(this));
      this.logger.aspect('dbdisconnect', '---after connection close');
    }.bind(this));
    this.logger.aspect('dbdisconnect', '---after promise');
    /*
    if ( mongoose.wovconnections.length == 0 ) {
      console.log('disconnecting mongoose');
      await mongoose.disconnect();
      // console.log('discunnected, closing now');
    }
    */
  };


  /**
   * Closes this connection and attempts to disconnect mongoose. If mongoose has other connections, don't disconnect.
   */
  static async disconnect(_logger) {
    _logger.aspect('dbdisconnect', 'disconnect mongoose');

    _logger.aspect('dbdisconnect', '---before disconnect');
    await new Promise( function(res, rej) {
      _logger.aspect('dbdisconnect', '---before disconnect 2');
      module.exports.Mongoose.disconnect(function(e) {
        _logger.aspect('dbdisconnect', '  ---disconnected: ', e);
        if ( e ) rej(e);
        else res(true);
      }.bind(this));
      _logger.aspect('dbdisconnect', '---after connection close');
    }.bind(this));
    _logger.aspect('dbdisconnect', '---after promise');



    /*
    this.close();
    if ( mongoose.wovconnections.length == 0 ) {
      this.logger.log('disconnecting mongoose');
      await mongoose.disconnect();
      // console.log('discunnected, closing now');
    } else {
      this.logger.warn('attempted to disconnect db, but mongoose still has active db connections... only did db.close');
    }
    */
  }

};


// This is the only Mongoose! Otherwise Promises break down....
module.exports.Mongoose = mongoose;
require('mongoose-long')(module.exports.Mongoose);
