
// const autoBind      = require('auto-bind-inheritance');
// const CryptoJS      = require('crypto-js');
// const os            = require('os');
// const uuidv4        = require('uuid/v4');

const Logger        = require('woveon-logger');

const DB            = require('./db');
const Service       = require('woveon-service');
// const Listener      = Service.Listener;
// const Requester     = Service.Requester;
// const Woveon      = require('./reqWoveon');
// import sdk from 'woveon-sdk';

const Long          = DB.Mongoose.Types.Long;

// const PluginResSchemaBase = require('../resource/schemaBase');

module.exports = class WEService extends Service {


  // ---------------------------------------------------------------------
  // overrides
  // ---------------------------------------------------------------------
  //  The following dev functions should be overriden for your services.
  // ---------------------------------------------------------------------

  /**
   * Turns off database then calls super.
   */
  async onShutdown() {
    this.logger.aspect('onshutdown', `WEService onShutdown: ${this.name}`);
    if ( this.db ) {
      this.logger.aspect('onshutdown', `WEService ${this.name}: service db.close`);
      await this.db.close();
    }
    await super.onShutdown();
  }

  /**
   * The listener has been created but not started. This is where you
   * add routes and network calls, like database init.
  */
  async onInit() {

    try {
      // if ( this.config != null ) {
      let fulldburl = `mongodb://${WEService.WEConfig.get('WOV_we_db_url')}/${WEService.WEConfig.get('WOV_we_db_collection')}`;
      await this.db.init(fulldburl);
      // }
    }
    catch (err) {
      console.log(err);
      throw new Error('Failed to init datbase');
      process.exit(1);
    }
    await super.onInit();
    this.plResourceModels = {};
  };

  // ---------------------------------------------------------------------
  // /overrides
  // ---------------------------------------------------------------------


  /**
   * Create the service.
   * @param {object} _serviceArgs - additional options
   *                     : name - overwrite the _name
   *                     : logger - pass in a logger
   */
  constructor(_serviceArgs) {
    // console.log('passing WEService Args: ', _serviceArgs);
    super(_serviceArgs);

    this.logger.aspect('weservice', '---------------------------------------------------------------------');
    this.logger.aspect('weservice', '--------------------------------------------------------------------');
    this.logger.aspect('weservice', ' Woveon Plugin Engine');
    this.logger.aspect('weservice', `  :: ${this.name}`);
    this.logger.aspect('weservice', '---------------------------------------------------------------------');

    this.db     = new DB(this.logger);
  };

};

/**
 * Helper. Converts int to Long.
 * @param {int} _v - turns into a Long
 * @return {Long} -
 */
module.exports.l = function(_v) { return Long.fromInt(_v); };


/**
 * Helper. Converts Long to int.
 * @param {Long} _l - turns into an int
 * @return {int} -
 */
module.exports.i = function(_l) { return _l.toInt(); };

/**
 * Helper. Converts Long to highSEPlow. Meant for floating point where high is
 * seconds and low is milliseconds. Could be used in other ways.
 *   ex. module.exports.f( new Long(ms, sec) ) => 'sec.ms'
 *
 * @param {Long} _l - a Long
 * @param {string} _sep - a separator placed between long and high bits
 * @param {int} _lowbitspadding - number of places assumed in lowbits so '1' is actually '000000001'
 * @return {string} - highbitsSEPlowbits
 */
module.exports.f = function(_l, _sep = '.', _lowbitspadding=9) {
  // logger.info(`   f:  low bits ${_l.getLowBits()}`);
  // logger.info(`   f:  low bits padded ${_lowbitspadding}: ${module.exports.pad(_l.getLowBits(), _lowbitspadding)}`);
  let ms = module.exports.pad(_l.getLowBits(), _lowbitspadding); // case of < 100 ms needs padding, ex. 78ms -> '078'
  let s  = _l.getHighBits();
  return `${s}${_sep}${ms}`;
};

/**
 * Like f, but returns a float instead of a string.
 * @param {Long} _l - a Long
 * @param {string} _sep - a separator placed between long and high bits
 * @return {float} -
 */
module.exports.F = function(_l, _sep = '.') { return parseFloat(module.exports.f(_l, _sep)); };


/**
 * Pads a number with _p to _width characters.
 *   - from: https://stackoverflow.com/a/10073788/2664702
 *   ex. pad(78,3) -> '078'
 *
 * @param {number} n - number
 * @param {int} width - width of characters to pad ex. 3 => XXX
 * @param {char} _p - pad character, defaults to '0'
 * @return {string} - padded character
 */
module.exports.pad = function pad(n, width, _p = '0') { n = n + ''; return n.length >= width ? n : new Array(width - n.length + 1).join(_p) + n; };


/**
 * Generates ConfigMap variables for WovTools/Kubernetes.
 */
module.exports.echoK8SConfigMap = () => {
  new module.exports.WEConfig(new Logger('config', {debug : true}));
  console.log(Service.Config.genK8SConfigMap());
};


/**
 * Generates Secrets variables for WovTools/Kubernetes.
 */
module.exports.echoK8SSecrets= () => {
  new module.exports.WEConfig(new Logger('config', {debug : true}));
  console.log(Service.Config.genK8SSecrets());
};


/**
 * Base WoveonEngine config.
 */
module.exports.WEConfig      = class WEConfig extends Service.Config {

  /**
   * @param {Logger} _logger - woveon-logger object.
   * @param {array} _confs - additional environment variables
   * @param {array} _sconfs - additional secret environment variables
   * @param {object} _defaults - changes to default behavior of the Woveon-Service Config object.
   */
  constructor(_logger, _confs = [], _sconfs = [], _defaults = {} ) {
    super(_logger,
      _confs.concat([
        'WOV_STAGE', 'WOV_ME', 'WOV_PROJECT',
        'WOV_we_wl_port', 'WOV_we_rl_port', 'WOV_we_ov_port', 'WOV_we_outofdateseconds',
        'WOV_we_wl_url', 'WOV_we_rl_url', 'WOV_we_ov_url', 'WOV_we_ver',
        'WOV_we_wlexternal_url', 'WOV_we_rlexternal_url', 'WOV_cluster_dns',
        'WOV_api_ver', 'WOV_api_fullurl', 'WOV_api_port_ext',
        'WOV_we_db_url', 'WOV_we_db_collection',
        'WOV_www_app_fullurl',
        'WOV_plugin_id',
        'WOV_testuser_email',
      ]),
      _sconfs.concat(['WOV_we_apisecret', 'WOV_we_datasecret', 'WOV_testuser_password']),
      _defaults);
  }
};


module.exports.DB          = DB;
    // --- Database class for mongoose.

module.exports.ReqRes = require('./reqRes');
    // --- Requester build around the sending of Res objects.

module.exports.ReqWoveon   = require('./reqWoveon');
    // --- Requester build around the sending of Resource objects to Woveon.
    // --- A Requester class designed for talking to Woveon's API backend.

module.exports.ResLib      = require('./reslib');
    // --- Res object and machinery.

module.exports.ResTypes    = require('./restypes');
