

const qs        = require('query-string');
const Requester = require('woveon-service').Requester;

/**
 * reqRes - Requester for Res objects
 * This class sends Res object requests to a remote service.
 */
module.exports = class ReqRes extends Requester {

  /**
   * Constructor.
   * @param {Logger} _logger -
   * @param {URL} _baseurl - server url
   * @param {URL path} _resourceUrl - a path to append for any resource REST call
   */
  constructor(_logger, _baseurl, _resourceUrl = null) {
    if ( _logger != null && _logger.constructor != null && _logger.constructor.name != 'Logger' ) {
      throw new Error('Need to pass in a Logger object.');
    }
    super(_logger, _baseurl);
    this.resourceUrl = _resourceUrl;
    this.credentials = {};
  }


  /**
   * @param {Object} _credentials - Data to send along with any message to a server.
   */
  setCredentials(_credentials) {
    this.credentials = _credentials;
    this.logger.verbose('set credentials: ', this.credentials);
  }


  /**
   * Get a resource from Woveon.
   *
   * @param {object} _res -
   * @param {boolean} _rawresult - Return json or raw Response object if true.
   * @return {object} - Res or Response object
   */
  async getRes(_res, _rawresult = false) {
    this.logger.info('getRes uses erqWovID, which I have not implemented... is this needed? I am not pulling Res from Woveon API at this time I do not think...');
    // let result = _res.reqWovID();
    let data = {xid : _res.getXID(), token : _res.getToken()};
    data = Object.assign({}, data, this.credentials);
    return this.get(this.resourceUrl+'/'+_res.getType()+'?'+qs.stringify(data), null, _rawresult);
  }


  /**
   *
   * Sends a Res object to Woveon.
   * @param {object}  _res       - data passed to Woveon API. generated from a reqWovData call
   * @param {boolean} _rawresult - Return json or raw Response object if true.
   * @return {*} - WovReturn or Response object from fetch call
   */
  async postRes(_res, _rawresult = false) {
    let result = _res.reqWovData();
    let retval = await this.postResData(result, _rawresult);
    return retval;
  }


  /**
   *
   * Code that sends Res object data to Woveon API. Pulled out from postRes so incomplete Res object data can be sent.
   * @param {object}  _resdata   - data passed to Woveon API. generated from a reqWovData call
   * @param {boolean} _rawresult - Return json or raw Response object if true.
   * @return {*} - WovReturn or Response object from fetch call
   */
  async postResData(_resdata, _rawresult = false) {
    _resdata.data = Object.assign({}, _resdata.data, this.credentials);
    let retval = await this.post(this.resourceUrl+_resdata.url, null, _resdata.data, _rawresult);
    // this.logger.info('  postResData result: ', retval);
    return retval;
  }


  /**
   */
  async putRes(_res, _rawresult = false) {
    let result = _res.reqWovData();
    result.data = Object.assign({}, result.data, this.credentials);
    return this.put(this.resourceUrl+result.url, null, result.data, _rawresult);
  }

  async deleteRes(_res, _rawresult = false) {
    let data = {xid : _res.getXID(), token : _res.getToken()};
    data = Object.assign({}, data, this.credentials);
    return this.delete(this.resourceUrl+'/'+_res.getType(), null, data, _rawresult);
  }

};
