const qs       = require('query-string');
const ReqRes   = require('./reqRes');
const WEConfig = require('./index').WEConfig;

/**
 * reqWoveon - Woveon Requester Res
 * This class sends Res object requests to a remote service.
 */
module.exports = class ReqWoveon extends ReqRes {

  /**
   * Build the ReqRes
   * @param {object} _logger - logger object
   */
  constructor(_logger) {
    super(_logger, WEConfig.get('WOV_api_fullurl'), module.exports.apiResPath());
    let c = {
      applicationid : WEConfig.get('WOV_plugin_id'),
      appsecret     : WEConfig.sget('WOV_we_apisecret'),
    };
    this.setCredentials(c);
    if ( this.credentials.applicationid != WEConfig.get('WOV_plugin_id') ||
         this.credentials.applicationid == null ) {
      throw new Error('Bad credentials (applicationid): ', c);
      process.exit(1);
    }
    if ( this.credentials.appsecret != WEConfig.sget('WOV_we_apisecret') ||
         this.credentials.appsecret == null ||
         this.credentials.appsecret == '' ) {
      throw new Error('Bad credentials (appsecret): ', c);
      process.exit(1);
    }
  }

  /**
   * Called by WoveonListener when it has finsihed /channel_launch
   *
   * Tells Woveon the channel is active. Also hands it an icon to use, probably taken
   * from the user's account for the remote service.
   *
   * @param {*} _channeltoken
   * @param {*} _icon
   * @return {promise}
   */
  async activateChannel(_channeltoken, _icon) {
    this.logger.verbose('activate channel: ', this.credentials, '  w URL: ', module.exports.apiResPath());
    return this.put(module.exports.apiResPath(), null,
      Object.assign({}, this.credentials, {token: _channeltoken, pageImage: _icon}));
  }

  /**
   * TODO: optimize
   * @param {*} _ress -
   * @return {array} - results of all the pushes
   */
  async bulkUpload(_ress) {
    let retval = [];

    this.logger.info('bulkUpload');
    for (let i=0; i< _ress.length; i++ ) {
      let res = _ress[i];
      /*
      this.logger.info('  bulk upload: (res ', i, ') w/ applicationId : ', res.getXID(), res.getSubject());
      this.logger.info(`  sending object ${i}: `, res.reqWovData());
      this.logger.info('  called reqWovData of: ', res.reqWovData);
      this.logger.info('  called reqWovData res of: ', res);
      this.logger.info('  called reqWovData res of: ', res.getRemoteID);
      */
      let r = await this.postRes(res);
      this.logger.aspect('bulkUpload', '  upload result: ', r);
      retval.push(r);
      // NOTE: this would be a good place to place the r[i].id back on the resource
    }
    this.logger.info('bulkUpload end');

    return retval;
  }

};

module.exports.apiUrl     = function(_path='') { return `${_path}`;}
module.exports.apiVerPath  = function(_path='') { return `/api/${WEConfig.get('WOV_api_ver')}${_path}`;}
module.exports.apiResPath = function(_path='') { return `/api/${WEConfig.get('WOV_api_ver')}/application/protect${_path}`;}
