
// Imported Code
// const should            = require('should');
const Logger            = require('woveon-logger');

// Actual Code
const WEService           = require('../src/service');
const channelModelConnector        = require('../src/models/channelConnector');
const remoteListenerModelConnector = require('../src/models/remoteListenerConnector');
const _WovMsgsModel       = require('../src/models/wovMsgsConnector');

const expect              = require('chai').expect;

console.log('WEConfig: ', WEService.WEConfig.toString());


// Testing code
// const fixtures          = require('../fixtures/v1/index');

let data = {
  service             : null,
  channelModel        : null,
  remotelistenerModel : null,
  wov                 : null, // {Posts:, Comments:, Likes: }
};

  let mtag = require('path').basename(__filename).slice(0, -3);
  let logger = new Logger(mtag, {debug : true, showName : true});

  data.service = new WEService({name : 'Generic Service'});
  data.service.logger.options.debug = true;


  describe(`${mtag}: Model Testing`, function() {
    logger.h1().info(this.title);


    // setup the service
    before( (done) => {
      data.service.init()
        .then(() => {
          data.channelModel = channelModelConnector(data.service.db);
          data.remotelistenerModel = remoteListenerModelConnector(data.service.db);
          done();
        })
        .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });
    });

    // Clears out Channel and RemoteListener Models
    beforeEach( function(done) {
      if (data.service.db.isConnected()) {
//        data.service.logger.info('going to remove');
        data.channelModel.remove({})
          .catch((err) => {console.log(err); throw new Error('Failed clearing database.');})
          .then(() => {return data.remotelistenerModel.remove({});})
          .catch((err) => {console.log(err); throw new Error('Failed clearing database.');})
          .then(() => {done();})
          .catch((err) => {console.log(err); throw new Error('Failed clearing database.');});
      } else {done();}
    });


    it(`${mtag}: ensure models are empty`, function(done) {
      logger.h2().info(this.test.title);

      data.channelModel.find({})
        .catch((err) => {console.log(err); process.exit(1);})
        .then( (channels) => {
          // console.log(channels);
          expect(channels.length).to.be.equal(0);
          // channels.should.be.length(0);
          return data.remotelistenerModel.find({});
        })
        .catch((err) => {console.log(err); process.exit(1);})
        .then( (_rls) => {
          expect(_rls.length).to.be.equal(0);
          done();
        })
        .catch((err) => {console.log(err); process.exit(1);});
    });


    it(`${mtag}: ensure adding to models works`, function(done) {
      logger.h2().info(this.test.title);
      this.timeout(3000);

      logger.info('1');
      let t = WEService.generateRandomString();
      let channel = new data.channelModel({token : t});
//      logger.info('2');
      channel.save()
        .then(() => {
//          logger.info('3.1');
          data.channelModel
            .find({token : t})
            .then((result) => {
              // logger.info('3.2');
//              console.log('new result', result);
              expect(result.length).to.be.equal(1);
              expect(result[0].token).to.not.equal(null);
              expect(result[0].token).to.equal(t);
              done();
            })
            .catch( (err)=>{logger.error(err); throw new Error('failed');});
        });
//      logger.info('3');
    });


  });


  describe(`${mtag}: Wov Model Testing`, function() {
    logger.h1().info(this.title);

    before( async function() {
      data.wov = _WovMsgsModel(data.service.db); // create connection
      await data.wov.Posts.remove({}); // all
      await data.wov.Comments.remove({}); // all
      await data.wov.Likes.remove({}); // all
    });


    it(`${mtag}: check post`, async function() {
      logger.h2().info(this.test.title);
      let wdata = {p : null, c : null, l : null};


      // get data
      wdata.p = await data.wov.Posts.find();
      wdata.c = await data.wov.Comments.find();
      wdata.l = await data.wov.Likes.find();

      // compare to expected
      logger.info('wdata: ', JSON.stringify(wdata, null, '  '));
      expect(wdata.p.length).to.equal(0);
      expect(wdata.c.length).to.equal(0);
      expect(wdata.l.length).to.equal(0);

      // add data
      await new data.wov.Posts({token : 1, remoteId : 1}).save();

      // check for added data
      wdata.p = await data.wov.Posts.find();
      logger.info('wdata: ', JSON.stringify(wdata, null, '  '));
      expect(wdata.p.length).to.equal(1);

      // fetch data
      wdata.p = await data.wov.Posts.find({token : 1, remoteId : 1});
      logger.info('wdata: ', JSON.stringify(wdata, null, '  '));
      expect(wdata.p.length).to.equal(1);

      // fetch data but not found
      wdata.p = await data.wov.Posts.find({token : 2, remoteId : 1});
      logger.info('wdata: ', JSON.stringify(wdata, null, '  '));
      expect(wdata.p.length).to.equal(0);

      // add comment data, and query
      r = await new data.wov.Comments({token : 1, remoteId : 1, secondId : 1}).save();
      wdata.c = await data.wov.Comments.find({token : 1, remoteId : 1, secondId : 1});
      logger.info('wdata.c: ', JSON.stringify(wdata.c, null, '  '));
      expect(wdata.c.length).to.equal(1);

      // add like data, and query
      r = await new data.wov.Likes({token : 1, remoteId : 1, secondId : 1}).save();
      wdata.l = await data.wov.Likes.find({token : 1, remoteId : 1, secondId : 1});
      logger.info('wdata.l: ', JSON.stringify(wdata.l, null, '  '), wdata.l[0].token, typeof wdata.l[0]);
      wdata.l[0].save();
      expect(wdata.l.length).to.equal(1);

      // lean query
      wdata.l = await data.wov.Likes.find({token : 1, remoteId : 1, secondId : 1}).lean();
      logger.info('wdata.l: ', JSON.stringify(wdata.l, null, '  '), wdata.l[0].token, typeof wdata.l[0]);
      wdata.l[0].token=2;
      try {
        wdata.l[0].save();
        throw new Error('Should have exited');
      } catch (err) {};
      logger.info('wdata.l: ', JSON.stringify(wdata.l, null, '  '), wdata.l[0].token, typeof wdata.l[0]);
      expect(wdata.l.length).to.equal(1);


      // query all data again
      wdata.p = await data.wov.Posts.find();
      wdata.c = await data.wov.Comments.find();
      wdata.l = await data.wov.Likes.find();
      logger.info('wdata: ', JSON.stringify(wdata, null, '  '));
      expect(wdata.p.length).to.equal(1);
      expect(wdata.c.length).to.equal(1);
      expect(wdata.l.length).to.equal(1);
    });


    after( async function() {
      await data.wov.Posts.remove({}); // all
      await data.wov.Comments.remove({}); // all
      await data.wov.Likes.remove({}); // all
    });

  });


describe(`${mtag}: Shutdown`, function() {

  it(`${mtag}: close db and service`, function(done) {
    logger.h2().info(this.test.title);

    logger.info('1');
    data.service.doShutdown()
      .catch( (err)=>{data.service.logger.error(err); throw new Error('failed');})
      .then( () => {
        logger.info('3');
        done();
      })
      .catch( (err)=>{data.service.logger.error(err); throw new Error('failed');});
    logger.info('2');
  });

  it(`${mtag}: try db again, should fail`, function(done) {
    logger.h2().info(this.test.title);

    data.service.logger.info('trying here');
    expect(data.service.db.isConnected()).to.equal(false);
    done();
  });

});

