
// let wtf = require('wtfnode');

let h        = require('./msHelper')(__filename);
let ResLib   = require('../src/service').ResLib;
const uuidv4 = require('uuid/v4');
const MessageResProto = require('../src/service/restypes/message');

const PLT     = require('../pltest/src/pl');
const WEConfig = PLT.WoveonListener.WEConfig;

let logger = h.logger;


let resp2 = new (class Message2ResProto extends MessageResProto {
  /** Testing ResProto that changes getAuthor and adds a 'tt' to schema. */
  constructor() {
    super('message2');

    Object.assign(this.interface, {
      getAuthor() {
        if (this._wovdata.author)     { return '1' + this._wovdata.author; }
        else if (this._rsdata && this._rsdata.author) {
          return '1' + this._rsdata.author.first + ' ' + this._rsdata.author.last;
        }
        return null;
      },
    });
    Object.assign(this.plschema, {tt : Number});
  }
})();

const plp           = new (require('../pltest/src/pl/resMessage'))();
const MessageModel2 = new ResLib.ResModel(resp2, plp);

let wl = null;
let rl = null;
let towov = new (require('../src/service/reqWoveon'))(logger);

describe(`${h.mtag}: Test the Res object with test plugin`, async () => {

  before( async () => {
    // create the WL
    let logwl = new Logger('WL', {
      debug    : true,
      showName : true,
      level    : 'verbose'},
      {
        'listener'          : true,
        'listener.route'    : true,
        'listener.incoming' : true,
        'listener.result'   : true,
        'requester'         : true,
      });
    wl = new PLT.WoveonListener({logger : logwl});
    await wl.init();
    await wl.startup();
    rl = new PLT.RemoteListener({logger : logrl});
    await rl.init();
    logger.info('... created WoveonListener');
  });



  it(`${h.mtag}: ...simple woveon created resource`, async function() {
    logger.h3().info(this.test.title);

    let wovdata = {
      token   : 'A',
      author  : 'Adam Smith',
      subject : 'Hi from My Company',
      message : 'Yo',
    };
    let res = await wl.MessageModel.generateRes(wovdata);
    logger.verbose(' postres1: ', res._wovdata, res._rsdata);
    expect(res.getToken()).to.equal(wovdata.token);
    expect(res.getAuthor()).to.equal(wovdata.author);
    expect(res.getSubject()).to.equal(wovdata.subject);
  });


  it(`${h.mtag}: ...woveon created resource`, async function() {
    logger.h3().info(this.test.title);

    let wovdata = {
      token   : 'B',
      author  : 'Barny Smith',
      subject : 'Hello',
      message : 'Yo',
    };
    // let res = new Post.Res(wdata, {}, Post.mixin);
    // let res = new PostRes(wdata, {}, {});
    let res = await wl.MessageModel.generateRes(wovdata, {});
    logger.verbose(' postres2: ', res._wovdata, res._rsdata);
    expect(res.getToken()).to.equal(wovdata.token);
    expect(res.getAuthor()).to.equal(wovdata.author);
    expect(res.getSubject()).to.equal(wovdata.subject);
  });

  it(`${h.mtag}: ...plugin created resource`, async function() {
    logger.h3().info(this.test.title);

    let wovdata = null;
    let rsdata = {
      id              : 'AAAAA',
      subject         : 'Chas say\'n hi',
      staysInPluginDB : 'A',
      plauthor        : {first : 'Chas', last : 'Smith', aid : '121314'},
    };
    // let res = new PostRes(wdata, rsdata, {});
    let res = await wl.MessageModel.generateRes(wovdata, rsdata);
    res.displayStructure();
    expect(res.getToken()).to.equal(null);
    expect(res.getStaysInPluginDB()).to.equal('A');
    expect(res.getSubject()).to.equal(rsdata.subject);
  });

  it(`${h.mtag}: ...plugin created resource, null wdata`, async function() {
    logger.h3().info(this.test.title);

    let wovdata  = {};
    let rsdata = {
      id       : 'AAAAA',
      plauthor : {first : 'Chas', last : 'Smith', aid : '121314'},
      subject  : 'Chas say\'n hi',
    };
    // let res = new PostRes(wdata, rsdata, {});
    let res = await wl.MessageModel.generateRes(wovdata, rsdata);
    expect(res.getToken()).to.equal(null);
    // expect(res.getAuthor()).to.equal('Chas Smith');
    expect(res.getSubject()).to.equal(rsdata.subject);
    // expect(res.getAuthorFirst()).to.equal(rsdata.author.first);
  });

  it(`${h.mtag}: ...add an Engine for an imagined Remote Service`, async function() {
    logger.h3().info(this.test.title);

    let wovdata1  = {};
    let wovdata2  = {author : 'Chass Ssmith'};
    let rsdata = {
      id       : 'AAAAA',
      plauthor : {first : 'Chas', last : 'Smith', aid : '121314'},
      subject  : 'Chas say\'n hi',
    };
    // let res1 = new PostRes(wdata1, rsdata, {});
    let res1 = await wl.MessageModel.generateRes(wovdata1, rsdata);
    // logger.info('res1: ', res1);
    expect(res1.getToken()).to.equal(null);
    expect(res1.getAuthor()).to.deep.equal(rsdata.plauthor);
    expect(res1.getSubject()).to.equal(rsdata.subject);
    let res2 = await MessageModel2.generateRes(wovdata2, rsdata);
    res2.displayData();
    expect(res2.getToken()).to.equal(null); // null since getSubject does not exist on res2
    expect(res2.getAuthor()).to.equal('Chass Ssmith');
    expect(res2.getSubject()).to.equal(rsdata.subject);
    // logger.info('MessageModel2 schema: ', MessageModel2.resproto.schema);
    expect(MessageModel2.resproto.plschema.tt).to.exist;
  });

  it(`${h.mtag}: ...Wov and plugin created resource`, async function() {
    logger.h3().info(this.test.title);

    let wovdata  = {token : 'C', author : 'Chas A Smith', images : [], videos : [], date : new Date()};
    let rsdata = {
      plauthor : {first : 'Chas', last : 'Smith', aid : '121314'},
      subject  : 'Chas say\'n hi',
    };
    // let res = new PostRes(wdata, rsdata, {});
    let res = await wl.MessageModel.generateRes(wovdata, rsdata);
    // logger.info(res);
//    logger.info('res : ', res.getID());
    expect(res.getXID()).to.exist;
    expect(res.getToken()).to.equal(wovdata.token);
    expect(res.getAuthor()).to.equal('Chas A Smith');
    expect(res.getSubject()).to.equal(rsdata.subject);

    logger.verbose('  ... test request objects');
    let reqwovdata = res.reqWovData();
    let reqrsdata  = res.reqPLData();
    logger.info('  req wovdata :', reqwovdata);
    logger.info('  req rsdata  :', reqrsdata);

    expect(reqwovdata.url).to.equal('/posts');
    expect(reqwovdata.data.authorUrl).to.equal(null);
    expect(reqwovdata.data.message).to.equal(null);
    expect(reqwovdata.data.images.length).to.equal(0);
    expect(reqwovdata.data.videos.length).to.equal(0);
    expect(reqwovdata.data.recipient).to.equal(null);
    expect(reqwovdata.data.date).to.not.equal(null);
  });


  it(`${h.mtag}: ...Send resource to test server`, async function() {
    logger.h3().info(this.test.title);

    let result = res = null;
    let ts = await h.genTSRequester(`/api/${WEConfig.get('WOV_api_ver')}/application/protect`);

    // Channel creation
    let teamId = 1;
    let channel = await ts.post(`/api/${WEConfig.get('WOV_api_ver')}/user/protect/team/protect/channel`, null, {
      session  : h.PE.Service.generateRandomString(),
      teamId   : teamId,
      sourceId : 7,
      title    : '1aNewChannel',
    });
    logger.info('result of create channel: ', channel);
    expect(channel.success).to.equal(true);
    expect(channel.code).to.equal(200); // data is {}, so no need to check
    expect(channel.data.token).to.exist;
    expect(channel.data.url).to.exist;

    // create resource (note, a new message would only have token in wovdata)
    let wovdata  = {token : channel.data.token, xid : uuidv4()};
    let rsdata = {
      plauthor : {first : 'Chas', last : 'Smith', aid : '121314'},
      subject  : 'Chas say\'n hi',
      message  : 'Message body',
    };

    // send resource
    // res = new PostRes(wdata, rsdata, {});
    res = await MessageModel2.generateRes(wovdata, rsdata);
    res.applyPostProcessing(await res.tryToCompleteWithRes());
    logger.info('sending resource: ', res);

    try {

      logger.info('Get: should fail, since message does not yet exist');
      result = await towov.getRes(res);
      // result = await ts.getRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(false);
      expect(result.code).to.equal(200);

      logger.info('POST: create the resource');
      result = await towov.postRes(res);
      // result = await ts.postRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);
      expect(res.getXID()).to.equal(result.data.xid);
      expect(res.getMessage()).to.equal(result.data.message);
      expect(res.getSubject()).to.equal(result.data.subject);
      expect(res.getAuthor()).to.deep.equal(result.data.author);
      expect(result.data.rsraw.author).to.deep.equal(res._rsdata.author);

      logger.info('GET: should succeed');
      result = await towov.getRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);
      expect(res.getXID()).to.equal(result.data.xid);
      expect(res.getMessage()).to.equal(result.data.message);
      expect(res.getSubject()).to.equal(result.data.subject);
      // logger.info('  res.GetAuthor      : ', JSON.stringify(res.getAuthor(), null, '  '));
      // logger.info('  result.data.author : ', JSON.stringify(result.data.author, null, '  '));
      expect(res.getAuthor()).to.deep.equal(result.data.author);
      expect(result.data.rsraw.author).to.deep.equal(res._rsdata.author);

      logger.info('PUT: modify first name to "Chass"');
      res._rsdata.plauthor.first = 'Chass';
      result = await towov.putRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);
      expect(res.getXID()).to.equal(result.data.xid);
      expect(res.getMessage()).to.equal(result.data.message);
      expect(res.getSubject()).to.equal(result.data.subject);
      expect(res.getAuthor()).to.deep.equal(result.data.author);
      expect(result.data.rsraw.author).to.deep.equal(res._rsdata.author);

      logger.info('GET: show modified');
      result = await towov.getRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);
      expect(res.getXID()).to.equal(result.data.xid);
      expect(res.getMessage()).to.equal(result.data.message);
      expect(res.getSubject()).to.equal(result.data.subject);
      expect(res.getAuthor()).to.deep.equal(result.data.author);
      expect(result.data.rsraw.author).to.deep.equal(res._rsdata.author);

      logger.info('DELETE: delete res');
      result = await towov.deleteRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);

      logger.info('GET: fails because deleted');
      result = await towov.getRes(res);
      logger.info('result: ', result);
      expect(result.success).to.equal(false);
      expect(result.code).to.equal(200);

    }
    catch (err) {
      throw err;
      logger.throwError('You probably need to turn on the testserver. "cd ../testserver ; make run"');
    }

  });


  it.only(`${h.mtag}: ... genResFromPLData`, async function() {
    logger.h3().info(this.test.title);

    let wovdata  = {
    };
    let rsdata = {
      created_time         : 2,
      pltestRemoteId       : 2,
      pltestRemoteParentId : 1,
      plauthor             : 'Chas Smith',
      staysInPluginDB      : '1',
    };
    let res1    = rl.MessageModel.generateRes(wovdata, rsdata);
    res1.applyPostProcessing( await res1.tryToCompleteWithRes() );
    // res1.displayData('res1 data');
    let pldata1 = res1.reqPLData().data;
    logger.info('pldata1: ', pldata1);

    let res2    = rl.MessageModel.generateRes(pldata1, pldata1, {errorOnWovdataSchema : false});
    // res2.applyPostProcessing( await res2.tryToCompleteWithRes() );
    // await res2.tryToCompleteWithRes();
    res2.displayData('res2 data');
    expect(res2.getXID()).to.not.be.null;
    expect(res2.getXIDTop()).to.be.undefined;
    expect(res2.getXIDParent()).to.be.undefined;
    expect(res2.getResInc()).to.not.be.null;
    expect(res2.getRemoteID()).to.equal(rsdata.pltestRemoteId);
    expect(res2.getRemoteParentID()).to.equal(rsdata.pltestRemoteParentId);
    expect(res2.getAuthor()).to.equal(rsdata.plauthor);
    expect(res2.getStaysInPluginDB()).to.equal(rsdata.staysInPluginDB);
    res2.displayData('res2 data');
    expect(res2.isComplete()).to.be.false; // never finds parent

    let pldata2 = res2.reqPLData().data;
    // res2.displayData('pldata1: ', pldata1);
    // res2.displayData('pldata2: ', pldata2);
    expect(pldata1.wovdata).to.deep.equal(pldata2.wovdata);
    expect(pldata1.rsdata).to.deep.equal(pldata2.rsdata);
  });


  it(`${h.mtag}: ...array util funcs`, async function() {
    logger.h3().info(this.test.title);

    let res1 = await rl.MessageModel.generateRes(null, {id : 'res1'});
    let res2 = await rl.MessageModel.generateRes(null, {id : 'res2'});
    let res3 = await rl.MessageModel.generateRes(null, {id : 'res3'});

    let a = [];

    // null
    expect(res1.findPosInArray(null)).to.equal(-1);

    // not in
    expect(res1.findPosInArray(a)).to.equal(-1);

    // insert sorted
    a = res2.insertIntoSortedArray(a);
    expect(a.length).to.equal(1);

    // find in non-empty
    expect(res1.findPosInArray(a)).to.equal(-1);

    // find self
    expect(res2.findPosInArray(a)).to.equal(0);

    // insert sorted at end
    a = res3.insertIntoSortedArray(a);
    expect(a.length).to.equal(2);

    // find self at end
    expect(res3.findPosInArray(a)).to.equal(1);

    // find self at front
    expect(res2.findPosInArray(a)).to.equal(0);

    // insert sorted at front
    a = res1.insertIntoSortedArray(a);
    expect(a.length).to.equal(3);

    // find self at front
    expect(res1.findPosInArray(a)).to.equal(0);

    // find self in middle
    expect(res2.findPosInArray(a)).to.equal(1);

    // find self at end
    expect(res3.findPosInArray(a)).to.equal(2);

    // insert self at front (existing)
    a = res1.insertIntoSortedArray(a);
    expect(a.length).to.equal(3);

    // insert self at mid (existing)
    a = res2.insertIntoSortedArray(a);
    expect(a.length).to.equal(3);

    // insert self at end (existing)
    a = res3.insertIntoSortedArray(a);
    expect(a.length).to.equal(3);

  });

  it.only(`${h.mtag}: ...schemas of resource`, async function() {
    logger.h3().info(this.test.title);

    let res1 = await rl.MessageModel.generateRes(null, {id : 'res1'});
    logger.info('plschema : ', Object.keys(res1.getPLSchema()));  // Object.keys(res1._resmodel.plschema));
    logger.info('wovschema: ', Object.keys(res1.getWovSchema())); // Object.keys(res1._resmodel.wovschema));

    expect(res1.getPLSchema() ).to.deep.equal(res1._resmodel.plschema);
    expect(res1.getWovSchema()).to.deep.equal(res1._resmodel.wovschema);

    expect(Object.keys(res1.getWovSchema()) ).to.have.members([
      'xid', 'xidTop', 'xidParent', 'resInc', 'remoteId', 'remoteParentId', 'isstub', 'token', // identifying data
      'subject', 'author', 'message',
    ]);
    expect(Object.keys(res1.getPLSchema()) ).to.have.all.members([
      'xid', 'xidTop', 'xidParent', 'resInc', 'remoteId', 'remoteParentId', 'isstub', 'plauthor', 'staysInPluginDB']);
  });

  after( async function() {
    await wl.doShutdown();
    // logger.info('wtf dump: after'); wtf.dump();
  });

});
