
const Logger   = require('woveon-logger');
const {expect} = require('chai');

const h                  = require('./msHelper')(__filename);
const ResLib             = require('../src/service/reslib.js');

let mtag = h.mtag;
let logger= new Logger(mtag, {debug : true, showName : true, level : 'verbose'});


// NOTE: much of these tests are ignored since 3c_rl tests all this, with the proper steps found in handleResForWoveon
describe(`${mtag}: Res test`, async () => {
  let TestModel   = null;
  let rl          = null;

  let rsdata1 = {subject : 't1', pltestRemoteId : 'rid1', created_time : 1, pltestRemoteParentId : null};
  let rsdata2 = {subject : 't2', pltestRemoteId : 'rid2', created_time : 2, pltestRemoteParentId : 'rid1'};
  let rsdata3 = {subject : 't3', pltestRemoteId : 'rid3', created_time : 3, pltestRemoteParentId : 'rid2'};

  before( async ()=>{
    rl = await h.genRL(false, true, true, );
    TestModel = rl.MessageModel;
  });

  beforeEach( async () => {
    logger.info('... cleaning out mongo');
    await TestModel.models.deleteMany({});
    await rl.IncompleteRes.deleteMany({});
    logger.info('  ... cleaning out mongo done');
  });

  /*
  it(`${mtag}: create res`, async function() {
    logger.h3().info(this.test.title);

    let wovdata = {
      token   : 'XXTOKENXX',
      author  : 'Barny Smith',
      subject : 'Subject Hello',
      message : 'Message Yo',
      resInc  : ResLib.Long.fromInt(54),
    };
    let rsdata = {
      author               : {first : 'Chas', last : 'Smith', aid : '121314'},
      subject              : 'Chas say\'n hi',
      from                 : {text : '<ccretus@woveon.net>'},
      to                   : {text : '<cw-tester@woveon.net>'},
      pltestRemoteId       : 1,
      pltestRemoteParentId : null,
    };

    // two separate hierarchies: post and plugin/post (ex. email)
    let post = TestModel.generateRes(wovdata, rsdata);

    logger.info('EmailPost Res: ', post.getIDs());
    expect(post._wovdata).to.exist;
    expect(post._rsdata).to.exist;
    expect(post._wovdata).to.deep.equal(wovdata);
    expect(post._rsdata).to.deep.equal(rsdata);


    let r2 = post.reqWovData();
    logger.info('reqWovData: ', r2);

    let r3 = post.reqPLData();
    logger.info('reqPLData : ', r3);

  });*/


  it(`${mtag}: generateRes no try to complete`, async function() {
    logger.h3().info(this.test.title);

    let res1 = await TestModel.generateRes(null, rsdata1);
    let res2 = await TestModel.generateRes(null, rsdata2);
    let res3 = await TestModel.generateRes(null, rsdata3);

    // not complete because has not set wovdata
    expect(res1.isComplete()).to.equal(false);
    expect(res2.isComplete()).to.equal(false);
    expect(res3.isComplete()).to.equal(false);

  });


  it(`${mtag}: generateRes with missing parents`, async function() {
    logger.h3().info(this.test.title);

    let res1 = await TestModel.generateRes(null, rsdata1);
    let res2 = await TestModel.generateRes(null, rsdata2);
    let res3 = await TestModel.generateRes(null, rsdata3);

    res1.applyPostProcessing( await res1.tryToCompleteWithRes() );
    res2.applyPostProcessing( await res2.tryToCompleteWithRes() );
    res3.applyPostProcessing( await res3.tryToCompleteWithRes() );

    // not complete because has not set wovdata
    // res1.displayData('res1');
    expect(res1.isComplete()).to.equal(false);
    res1.getResInc(); // generates it. normally called in RL.handleResForWoveon
    expect(res1.isComplete()).to.equal(true);
    expect(res2.isComplete()).to.equal(false);
    expect(res3.isComplete()).to.equal(false);
  });


  it(`${mtag}: generateRes for top msg`, async function() {
    logger.h3().info(this.test.title);

    logger.info('generate and save');
    let res1 = await TestModel.generateRes(null, rsdata1);
    let res2 = await TestModel.generateRes(null, rsdata2);
    let res3 = await TestModel.generateRes(null, rsdata3);
    res1.applyPostProcessing( await res1.tryToCompleteWithRes() );
    /*
    res2.applyPostProcessing( await res2.tryToCompleteWithRes() );
    res3.applyPostProcessing( await res3.tryToCompleteWithRes() );
    */

    logger.info('TestModel:', await TestModel.models.find());
    await Promise.all([res1.updateMongo(), res2.updateMongo(), res3.updateMongo()]);
    // await Promise.all([res1.updateMongo(), res2.updateMongo(), res3.updateMongo()]);

    expect(res1.isComplete()).to.equal(true);
    expect(res2.isComplete()).to.equal(false);
    expect(res3.isComplete()).to.equal(false);

    logger.info(`PruneIncompleteAndSave call should return res1 since it is complete`);
    expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res1])).length).to.equal(1);
    logger.info('TestModel:', await TestModel.models.find());
    logger.info('  ... success');

  });

  /*
  it(`${mtag}: generateRes for missing parent and have them eventually complete`, async function() {
    logger.h3().info(this.test.title);

    for (let i=0; i<1; i++) {
      logger.h2().info(`... sync test: ${logger.ord(i)} pass`);

      logger.aspect('test4', 'generate and save 1');
      let res1 = await TestModel.generateRes(null, rsdata1);
      res1.applyPostProcessing( await res1.tryToCompleteWithRes() );
      logger.aspect('test4', 'generate and save 2');
      let res2 = await TestModel.generateRes(null, rsdata2);
      res2.applyPostProcessing( await res2.tryToCompleteWithRes() );
      logger.aspect('test4', 'generate and save 3');
      let res3 = await TestModel.generateRes(null, rsdata3);
      res3.applyPostProcessing( await res3.tryToCompleteWithRes() );
      logger.aspect('test4', 'generate and save 4');
      logger.info('TestModel:', await TestModel.models.find());
      await Promise.all([res1.updateMongo(), res2.updateMongo(), res3.updateMongo()]);
      logger.aspect('test4', 'generate and save 5');

      logger.aspect('test4', 'prune res2 which is incomplete');
      expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res2])).length).to.equal(0);

      logger.aspect('test4', 'res1 is complete so should be returned');
      expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res1])).length).to.equal(1);

      logger.aspect('test4', 'since res1 is complete, complete res2 with res1');
      expect((await rl.handleResForWoveon_CompleteIncompleteWith([res1], 'aaa')).length).to.equal(1);

      expect((await rl.IncompleteRes.find()).length).to.equal(0, 'Should have no incomplete res now');
    }
  });
  */

  /*
  it(`${mtag}: generateRes for missing parent and have them eventually complete`, async function() {
    logger.h3().info(this.test.title);

    // generate and save
    logger.aspect('test4', 'generate and save 1');
    let res1 = await TestModel.generateRes(null, rsdata1).tryToCompleteRL();
    logger.aspect('test4', 'generate and save 2');
    let res2 = await TestModel.generateRes(null, rsdata2).tryToCompleteRL();
    logger.aspect('test4', 'generate and save 3');
    let res3 = await TestModel.generateRes(null, rsdata3).tryToCompleteRL();
    logger.aspect('test4', 'generate and save 4');
    logger.info('TestModel:', await TestModel.models.find());
    await Promise.all([res1.updateMongo(), res2.updateMongo(), res3.updateMongo()]);

    logger.aspect('test4', 'prune res3 and res2 which are incomplete (though res3 parent should be set now)');
    expect((await rl.incompleteRes_PruneIncompleteAndSave([res3])).length).to.equal(0);
    expect((await rl.incompleteRes_PruneIncompleteAndSave([res2])).length).to.equal(0);

    logger.aspect('test4', 'try to complete with res3 but nothing');
    expect((await rl.incompleteRes_CompleteIncompleteWith([res3], 'aaa')).length).to.equal(0);

    logger.aspect('test4', 'try to complete with res2 but nothing completes');
    expect((await rl.incompleteRes_CompleteIncompleteWith([res2], 'aaa')).length).to.equal(0);

    logger.aspect('test4', 'however, res3 should have partial completion');
    logger.info('parent or something should be set: ', res3.getIDs());
    // here need test

    // res1 is complete so should be returned
    expect((await rl.incompleteRes_PruneIncompleteAndSave([res1])).length).to.equal(1);

    // since res1 is complete, complete res2 and res3
    expect((await rl.incompleteRes_CompleteIncompleteWith([res1], 'aaa')).length).to.equal(2);

    expect((await rl.IncompleteRes.find()).length).to.equal(0, 'Should have no incomplete res now');
  });
  */


  /*
  it(`${mtag}: sync issues`, async function() {
    logger.h3().info(this.test.title);
    this.timeout(10000);

    for (let i=0; i<50; i++) {
      logger.h2().info(`... sync test: ${logger.ord(i)} pass`);
      let res1 = await TestModel.generateRes(null, rsdata1);
      res1.applyPostProcessing( await res1.tryToCompleteWithRes() );
      let res2 = await TestModel.generateRes(null, rsdata2);
      res2.applyPostProcessing( await res2.tryToCompleteWithRes() );
      let res3 = await TestModel.generateRes(null, rsdata3);
      res3.applyPostProcessing( await res3.tryToCompleteWithRes() );
      logger.info('TestModel:', await TestModel.models.find());
      await Promise.all([res1.updateMongo(), res2.updateMongo(), res3.updateMongo()]);
      expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res3])).length).to.equal(0);
      expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res2])).length).to.equal(0);
      expect((await rl.handleResForWoveon_PruneIncompleteAndSave([res1])).length).to.equal(1);
      expect((await rl.handleResForWoveon_CompleteIncompleteWith([res1], 'aaa')).length).to.equal(2);
    }
  });
  */

  after( async function() {
    logger.h1().info(`${mtag} shutting down`);
    await rl.doShutdown();
  });

});
