
const h                 = require('./msHelper')(__filename);
const expect            = require('chai').expect;
const WE                = require('../src');
const PLT               = require('../pltest/src/pl');
const Logger            = require('woveon-logger');


// https://stackoverflow.com/a/43260158/2664702
/** all permutations of an array
 * @param {array} _xs - elements to permute
 * @return {array} - array of arrays of permutations
 */
function perm(_xs) {
  let retval = [];

  for (let i = 0; i < _xs.length; i = i + 1) {
    let rest = perm(_xs.slice(0, i).concat(_xs.slice(i + 1)));

    if (!rest.length) { retval.push([_xs[i]]); }
    else {
      for (let j = 0; j < rest.length; j = j + 1) {
        retval.push([_xs[i]].concat(rest[j]));
      }
    }
  }
  return retval;
}

// let C = null;
const logger = new Logger(h.mtag, {debug : true, showName : true, level : 'info'});

describe(`${h.mtag}: Test the data conversion and PostProcessing of RL`, async () => {

  before(async () => {
    // if ( !WEConfig.isInited() ) { logger.info(`INITING LOGGER "${h.mtag}"`); new WE.Service.WEConfig(logger); }
    await WE.Service.WEConfig.blockForInit(WE.Service.WEConfig, logger, true);

    rl = new PLT.RemoteListener({logger : logger, sipsize : 10});
    rl.isrunning = false;
    await rl.init();
  });

  beforeEach(async () => {
    logger.info('beforeEach');
    await rl.MessageModel.models.deleteMany({}); // clear database
    await rl.IncompleteRes.deleteMany({});       // clear database
  });

  it(`${h.mtag}: forward simple message`, async function() {
    logger.h3().info(this.test.title);

    let msgdata = [{
      plauthor             : 'Adam Smith',
      staysInPluginDB      : 'a',
      subject              : 'Hi from My Company',
      message              : 'Yo',
      pltestRemoteId       : 1,
      pltestRemoteParentId : null,
      created_time         : 1,
    }];
    let wovres = rl.convertChannelDataToRes('ctoken', msgdata);
    wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );

    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}`) );
    let result = await rl.handleResForWoveon('ctoken', wovres);
    wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);
    expect(wovres[0].isComplete()).to.be.true;

    // see if model in PL
    let r = await wovres[0]._resmodel.models.findOne({remoteId : wovres[0].getRemoteID()}).lean();
    logger.info('r in PL: ', r);

  });

  it(`${h.mtag}: forward parent and child message, in order`, async function() {
    logger.h3().info(this.test.title);

    let msgdata = {
      parent : [{
        created_time         : 1,
        pltestRemoteId       : 1,
        pltestRemoteParentId : null,
        plauthor             : 'Adam Smith',
        staysInPluginDB      : 'a',
        subject              : 'Hi from My Company',
        message              : 'Yo',
      }],
      child : [{
        created_time         : 2,
        pltestRemoteId       : 2,
        pltestRemoteParentId : 1,
        plauthor             : 'A Aron',
        staysInPluginDB      : 'junk',
        subject              : 'Hello Adam.',
        message              : 'Re: Yo',
      }],
    };

    let wovres = null;
    let result = null;

    // forward parent
    wovres = rl.convertChannelDataToRes('ctoken', msgdata.parent);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    result = await rl.handleResForWoveon('ctoken', wovres);
    wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);
    expect(wovres[0].isComplete()).to.be.true;
    // logger.info(await wovres[0]._resmodel.models.findOne({remoteId : wovres[0].getRemoteID()}).lean());
    expect(await wovres[0]._resmodel.models.findOne({remoteId : wovres[0].getRemoteID()}).lean()).to.not.be.null;

    // forward child
    wovres = rl.convertChannelDataToRes('ctoken', msgdata.child);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    result = await rl.handleResForWoveon('ctoken', wovres);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);
    expect(wovres[0].isComplete()).to.be.true;
    expect(await wovres[0]._resmodel.models.findOne({remoteId : wovres[0].getRemoteID()}).lean()).to.not.be.null;

  });


  it(`${h.mtag}: forward parent and child message, out of order`, async function() {
    logger.h3().info(this.test.title);

    let msgdata = {
      parent : [{
        created_time         : 1,
        pltestRemoteId       : 1,
        pltestRemoteParentId : null,
        plauthor             : 'Adam Smith',
        staysInPluginDB      : 'a',
        subject              : 'Hi from My Company',
        message              : 'Yo',
      }],
      child : [{
        created_time         : 2,
        pltestRemoteId       : 2,
        pltestRemoteParentId : 1,
        plauthor             : 'A Aron',
        staysInPluginDB      : 'junk',
        subject              : 'Hello Adam.',
        message              : 'Re: Yo',
      }],
    };

    let wovres = null;
    let result = null;

    // forward child for is incomplete
    wovres = rl.convertChannelDataToRes('ctoken', msgdata.child);
    wovres[0].displayData('child pre handle');
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    result = await rl.handleResForWoveon('ctoken', wovres);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);
    wovres[0].displayData('child');
    expect(wovres[0].isComplete()).to.be.false;

    // child is not in PL
    expect(await wovres[0]._resmodel.models.findOne({remoteId : msgdata.child[0].pltestRemoteId}).lean()).to.be.null;
    // child is in Incomplete
    expect(await rl.IncompleteRes.findOne({'_wovdata.remoteId' : msgdata.child[0].pltestRemoteId}).lean()).to.not.be.null;

    // forward parent, to complete child and forward this
    wovres = rl.convertChannelDataToRes('ctoken', msgdata.parent);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    result = await rl.handleResForWoveon('ctoken', wovres);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);
    expect(wovres[0].isComplete()).to.be.true;

    logger.info('fetch parent: ', await wovres[0]._resmodel.models.findOne({remoteId : msgdata.parent[0].pltestRemoteId}).lean());
    expect(await wovres[0]._resmodel.models.findOne({remoteId : msgdata.parent[0].pltestRemoteId}).lean()).to.not.be.null;
    expect(await wovres[0]._resmodel.models.findOne({remoteId : msgdata.child[0].pltestRemoteId}).lean()).to.not.be.null;

    // check message model and incomplete
    expect( (await rl.MessageModel.models.find()).length ).to.equal(2);
    expect( (await rl.IncompleteRes.find()).length ).to.equal(0);
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.parent[0].pltestRemoteId})).isComplete() ).to.be.true;
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.child[0].pltestRemoteId})).isComplete() ).to.be.true;
  });


  it('forward at same time in order', async function() {
    logger.h3().info(this.test.title);

    let msgdata = {
      parent : [{
        created_time         : 1,
        pltestRemoteId       : 1,
        pltestRemoteParentId : null,
        plauthor             : 'Adam Smith',
        staysInPluginDB      : 'a',
        subject              : 'Hi from My Company',
        message              : 'Yo',
      }],
      child : [{
        created_time         : 2,
        pltestRemoteId       : 2,
        pltestRemoteParentId : 1,
        plauthor             : 'A Aron',
        staysInPluginDB      : 'junk',
        subject              : 'Hello Adam.',
        message              : 'Re: Yo',
      }],
    };

    let wovres = rl.convertChannelDataToRes('ctoken', [msgdata.parent[0], msgdata.child[0]]);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    let result = await rl.handleResForWoveon('ctoken', wovres);
    wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);

    // check message model and incomplete
    expect( (await rl.MessageModel.models.find()).length ).to.equal(2);
    expect( (await rl.IncompleteRes.find()).length ).to.equal(0);
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.parent[0].pltestRemoteId})).isComplete() ).to.be.true;
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.child[0].pltestRemoteId})).isComplete() ).to.be.true;

    // NOTE: wovres[0] is complete, but wovres[1] is still incomplete, because it was rebuilt into another Res and completed
    // expect(wovres[0].isComplete()).to.be.true;
    // expect(wovres[1].isComplete()).to.be.true;
  });


  it('forward at same time out of order', async function() {
    logger.h3().info(this.test.title);

    let msgdata = {
      parent : [{
        created_time         : 1,
        pltestRemoteId       : 1,
        pltestRemoteParentId : null,
        plauthor             : 'Adam Smith',
        staysInPluginDB      : 'a',
        subject              : 'Hi from My Company',
        message              : 'Yo',
      }],
      child : [{
        created_time         : 2,
        pltestRemoteId       : 2,
        pltestRemoteParentId : 1,
        plauthor             : 'A Aron',
        staysInPluginDB      : 'junk',
        subject              : 'Hello Adam.',
        message              : 'Re: Yo',
      }],
    };

    let wovres = rl.convertChannelDataToRes('ctoken', [msgdata.child[0], msgdata.parent[0]]);
    // wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: preforward`) );
    let result = await rl.handleResForWoveon('ctoken', wovres);
    wovres.forEach( (r, i) => r.displayData(`${logger.ord(i)}: postforward`) );
    expect(result).to.equal(0);

    // nothing in message model or incomplete
    expect( (await rl.MessageModel.models.find()).length ).to.equal(2);
    expect( (await rl.IncompleteRes.find()).length ).to.equal(0);

    // pull and make Res, that should be complete
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.parent[0].pltestRemoteId})).isComplete() ).to.be.true;
    expect( (await rl.MessageModel.generateResFromQuery({remoteId : msgdata.child[0].pltestRemoteId})).isComplete() ).to.be.true;
  });


  // make sure it can complete in all orders
  it('exhaustivly do res in all permutations for hierarhcy', async function() {
    this.timeout(115000);
    let n = 4;
    let dati = {
      created_time         : undefined,
      pltestRemoteId       : undefined,
      pltestRemoteParentId : undefined,
      plauthor             : 'Adam Smith',
      staysInPluginDB      : 'a',
      subject              : 'Hi from My Company',
      message              : 'Yo',
    };
    let data = [];
    for (let i=0; i<n; i++) {
      data[i] = JSON.parse(JSON.stringify(dati));
      data[i].created_time   = i;
      data[i].pltestRemoteId = i;
      data[i].pltestRemoteParentId = (i==0?null:data[i-1].pltestRemoteId);
    }
    let perms = perm(data);
    for (let i=0; i<perms.length; i++) {
      let p = perms[i];
      // logger.info('p: ', p);
      let k = [];
      for (let j=0; j<p.length; j++) { k.push(p[j].pltestRemoteId); }
      logger.info(`perm ${i}: ${k}`); // logger.info('p: ', p);

      let wovres = rl.convertChannelDataToRes('ctoken', p);
      let result = await rl.handleResForWoveon('ctoken', wovres);
      expect(result).to.equal(0);

      // check message model and incomplete
      // logger.h3().info('message modesl data: ', ); await rl.MessageModel.models.find();
      expect( (await rl.MessageModel.models.find()).length ).to.equal(p.length);
      expect( (await rl.IncompleteRes.find()).length ).to.equal(0);

      // check each in message model is complete
      for (let j=0; j<p.length; j++) {
        expect( (await rl.MessageModel.generateResFromQuery({remoteId : p[j].pltestRemoteId})).isComplete() ).to.be.true;
      }

      // clear models
      await rl.MessageModel.models.deleteMany({}); // clear database
      await rl.IncompleteRes.deleteMany({});       // clear database
    }


    // for (let i=0; i<perms.length; i++) { logger.info(`perm ${i}: `, perms[i]); }

  });


  after(async () => {
    logger.h3().info(`${h.mtag} - shutting down`);
    await rl.doShutdown();
  });
});
