

// const h       = require('./msHelper')(__filename);
const WE      = require('../src');
const PLT     = require('../pltest/src/pl');
const Logger  = require('woveon-logger');
const uparse  = require('url-parse');
const expect  = require('chai').expect;
// const DB      = WE.Service.DB;

let mtag = require('path').basename(__filename).slice(0, -3);

describe(`${mtag}: Test interaction between microservers`, async () => {
  let ov = rl = wl = ts = null;

  let logger= new Logger(mtag, {debug : true, showName : true, level : 'verbose'});
  let logov = new Logger('OV', {debug : true, showName : true, level : 'verbose'});
  let logrl = new Logger('RL', {debug : true, showName : true, level : 'verbose'});
  let logwl = new Logger('WL', {debug : true, showName : true, level : 'verbose'}, {'listener' : true, 'listener.route' : true});
  let logts = new Logger('TS', {debug : true, showName : true, level : 'verbose'});

  let req           = new WE.Service.Requester(logger);
  let ses = null;
  let WEConfig = null;

  // setup the service
  before(async () => {
    WEConfig = WE.Service.WEConfig;
    if ( ! WEConfig.isInited() ) { logger.info(`INITING LOGGER "${mtag}"`); new WEConfig(logger); }
    else { logger.info(`already INITING`); }
    await WEConfig.blockForInit();
    WEConfig.displayMe();

    // c_ov.service.port = c_ov.plugin.ov_port;
    ov = new PLT.Overseer({logger : logov});
    ov.isrunning = null; // turn off loop

    // c_rl.service.port = c_rl.plugin.rl_port;
    rl = new PLT.RemoteListener({logger : logrl, sipsize : 10});
    rl.isrunning = null; // turn off loop

    // c_wl.service.port = c_wl.plugin.wl_port;
    wl = new PLT.WoveonListener({logger : logwl});

    ts = new WE.Service.Requester(logts, WEConfig.get('WOV_api_fullurl')); // http://localhost:3010

    await ov.init();
    await ov.startup();
    logger.info('... created Overseer');

    logger.info(`${mtag}: get rid of bad remote listeners at start`);
    await ov.RemoteListenerModel.deleteMany({});
    await ov.ChannelModel.deleteMany({});

    await rl.init();
    await rl.startup();
    logger.info('... created RemoteListener');

    await wl.init();
    await wl.startup();
    logger.info('... created WoveonListener');
  });


  describe(`${mtag}: start channels and auth it with test server, and make sure it is running`, async () => {

    // Create a session
    it(`${mtag}: create session`, async function() {
      logger.h3().info(this.test.title);

      expect(WEConfig.get('WOV_testuser_email')).to.not.equal(undefined);
      expect(WEConfig.sget('WOV_testuser_password')).to.not.equal(undefined);

      let ret = await ts.get(`/api/v1/user/auth?mail=${WEConfig.get('WOV_testuser_email')}&password=${WEConfig.sget('WOV_testuser_password')}`);
      logger.info('ret: ', ret);

      expect(ret.code).to.equal(200);
      expect(ret.success).to.equal(true);
      expect(ret.data.id).to.equal(2);
      expect(ret.data.mail).to.equal(WEConfig.get('WOV_testuser_email'));
      expect(ret.data.xid).to.exist;
      expect(ret.data.session).to.exist;
      expect(ret.data.name).to.exist;

      ses = ret.data;

    });

    it(`${mtag}: start channel, make sure can connect to running testserver`, async () => {
      logger.info(`${mtag}: start channel, make sure can connect to running testserver`);


      // TODO: Testserver needs to implement this route!
      let ret = await ts.post('/api/v1/user/protect/team/protect/channel', null,
        {
          session  : ses.session,
          teamId   : 1,
          sourceId : 7,
          title    : 'NewChannel',
        }); // get('/auth?teamId=1');
      logger.info('ret : ', ret);
      expect(ret.success).to.equal(true); // is testserver on?

      // should have 1 running RL, 1 running channel
      let rls = await ov.RemoteListenerModel.find({});
      expect(rls.length).to.equal(1);
      let channels = await ov.ChannelModel.find({});
      expect(channels.length).to.equal(1);

    });


    it(`${mtag}: start channel and auth`, async () => {
      logger.h1().info(`${mtag}: start channel and auth`);
      let htmlrequester = new WE.Service.Requester(logger, null, 'text/html');
      let d_chanstart = null, oat = null, state = null, channeltoken = null;

      // Tell API we want to create channel
      let d_auth = await ts.post('/api/v1/user/protect/team/protect/channel', null,
        {
          session  : ses.session,
          teamId   : 1,
          sourceId : 7,
          title    : 'NewChannel2',
        }); // get('/auth?teamId=1');
      // API responds with url and token for starting the channel
      logger.info('d_auth: ', d_auth);
      expect(d_auth.success).to.equal(true); // is testserver on?
      expect(d_auth.code).to.equal(200); // is testserver on?
      expect(d_auth.data.url).to.exist;
      expect(d_auth.data.token).to.exist;

      // Call the /channel_start url that was returned
      //  NOTE: this uses pltest's pseudo Oauth, which redirects to testserver as a RomoteService
      //        and this creates an Oauth token and returns to /channel_launch.
      logger.info('... calling WL./channel_start ', d_auth.data.url); // WL/v1/channel_start?t=XXX
      try {

        d_chanstart = await htmlrequester.post(d_auth.data.url);
        logger.info('d_chanstart: ', d_auth.data.url, ': ', d_chanstart);
        expect(d_chanstart.code).to.equal(200);
        expect(d_chanstart.success).to.equal(true);
        expect(d_chanstart.meta.url).to.exist;

        logger.verbose(' channel start would redirect to remote server and do any auth... here, we are looking at the URL that it was redirected to, and parsing out the tokens and such to make sure all is ok.');
        logger.verbose(' here... parse the d_chanstart.meta.url value to get oauthtoken and state (which is state and channel token)');
        let csurl = uparse(d_chanstart.meta.url, true);
        logger.info('asdf ', csurl);
        expect(csurl.query.t).to.equal(d_auth.data.token);
        expect(csurl.query.state).to.equal(d_auth.data.token); // b/c using channel token as state for test
        // expect(csurl.query.oauthtoken).to.exist;

        // oat          = csurl.query.oauthtoken;
        channeltoken = csurl.query.t;
        state        = csurl.query.state;
      }
      catch (err) { logger.rethrowError(err); }

      // get oauth for channel
      oat = (await req.get(`${WEConfig.get('WOV_api_fullurl')}/u/channeloauth?token=${channeltoken}`)).data;
      logger.info('oat: ', oat);

      // since testserver calls validate
      /*
      let d_chanlaunch;
      try {
        logger.info('call the url: http://localhost:50105/v1/channel_validatelaunch?oauthtoken=${oat}&state=${state}&token=${channeltoken}, as this will check that hte launch works');
        d_chanlaunch = await htmlrequester.post(`http://localhost:50105/v1/channel_validatelaunch?oauthtoken=${oat}&state=${state}&token=${channeltoken}`);


        logger.info('d_chanlaunch: ', d_chanlaunch);
        expect(d_chanlaunch.code).to.equal(200);
        expect(d_chanlaunch.success).to.equal(true);
      }
      catch (err) { logger.rethrowError(err); }
      */

      logger.info('then check that database has channel and such');

      // should have 1 running RL, 2 running channels (becaus once channel created in last call)
      let rls = await ov.RemoteListenerModel.find({});
      expect(rls.length).to.equal(1);
      let channels = await ov.ChannelModel.find({});
      logger.info('channels: ', channels);
      expect(channels.length).to.equal(2);
      expect(channels[1].token).to.equal(channeltoken);
      expect(channels[1].pluginData.oauthtoken).to.equal(oat);
      expect(channels[1].pluginData.state).to.equal(state);

    });


    it(`${mtag}: start channel, fail channel_start, then pass, fail channel_launch, then pass`, async () => {
      logger.h1().info(`${mtag}: start channel and auth`);
      // let req           = new WE.Service.Requester(logger);
      let htmlrequester = new WE.Service.Requester(logger, null, 'text/html');
      let d_chanstart = null, oat = null, state = null, channeltoken = null;
      let d_chanlaunch;

      // Tell API we want to create channel
      let d_auth = await ts.post('/api/v1/user/protect/team/protect/channel', null,
        {
          session  : ses.session,
          teamId   : 1,
          sourceId : 7,
          title    : 'NewChannel2',
        }); // get('/auth?teamId=1');
      // API responds with url and token for starting the channel
      logger.info('d_auth: ', d_auth);
      expect(d_auth.success).to.equal(true); // is testserver on?
      expect(d_auth.code).to.equal(200); // is testserver on?
      expect(d_auth.data.url).to.exist;
      expect(d_auth.data.token).to.exist;

      // Call the /channel_start url that was returned
      //  NOTE: this uses pltest's pseudo Oauth, which redirects to testserver as a RomoteService
      //        and this creates an Oauth token and returns to /channel_launch.
      logger.info('... calling plugin to WL./channel_start ', d_auth.data.url); // WL/v1/channel_start?t=XXX
      try {

        logger.info('...fail because of additional chars to token');
        d_chanstart = await htmlrequester.post(d_auth.data.url+'000');
        logger.info('d_chanstart (fail): ', d_chanstart);
        expect(d_chanstart.code).to.equal(200); // fails, but code is not a failure
        expect(d_chanstart.success).to.equal(true);
        expect(d_chanstart.meta.url).to.equal('http://localhost:50105/static/channel_start_error.html');
      }
      catch (err) {
        logger.error('NOTE: if you get this error, make sure "static" dir exists in your current directory, going to html, so if you are in the "test" directory, then it will fail. For WoveonEngine testing, make sure "static" directory is softlinked to "pltest/static"');
        logger.rethrowError(err);
      }

      try {
        logger.info('...success now');
        d_chanstart = await htmlrequester.post(d_auth.data.url);
        logger.info('d_chanstart (success): ', d_chanstart);
        expect(d_chanstart.code).to.equal(200);
        expect(d_chanstart.success).to.equal(true);
        expect(d_chanstart.meta.url).to.exist;
        expect(d_chanstart.meta.url).to.not.equal('http://localhost:50105/static/channel_start_error.html');

        let csurl = uparse(d_chanstart.meta.url, true);
        expect(csurl.query.t).to.equal(d_auth.data.token);
        expect(csurl.query.state).to.equal(d_auth.data.token); // b/c using channel token as state for test
//        expect(csurl.query.oauthtoken).to.exist;

        // oat          = csurl.query.oauthtoken;
        channeltoken = csurl.query.t;
        state        = csurl.query.state;
      }
      catch (err) { logger.rethrowError(err); }

      oat = (await req.get(`${WEConfig.get('WOV_api_fullurl')}/u/channeloauth?token=${channeltoken}`)).data;
      logger.info('oat: ', oat);

      try {
        logger.info('validatelaunch: catch fail for: oathtoken(oat)');
        // d_chanlaunch = await htmlrequester.post(`http://localhost:50105/v1/channel_validatelaunch?oauthtoken=a${oat}&state=${state}&token=${channeltoken}`);
        d_chanlaunch = await req.post(`http://localhost:50105/v1/channel_validatelaunch?oauthtoken=a${oat}&state=${state}&token=${channeltoken}`);
        logger.info('d_chanlaunch: ', d_chanlaunch);
        expect(d_chanlaunch.code).to.equal(200);
        expect(d_chanlaunch.success).to.equal(false);
        expect(d_chanlaunch.meta.url).to.exist;
        expect(d_chanlaunch.msg).to.equal('unable to start hook into remote service');
      }
      catch (err) { logger.rethrowError(err); }

      try {
        logger.info('validatelaunch: catch fail for: t');
        d_chanlaunch = await req.post(`http://localhost:50105/v1/channel_validatelaunch?oauthtoken=${oat}&state=s${state}&token=${channeltoken}`);
        logger.info('d_chanlaunch: ', d_chanlaunch);
        expect(d_chanlaunch.code).to.equal(200);
        expect(d_chanlaunch.success).to.equal(false);
        expect(d_chanlaunch.meta.url).to.exist;
        expect(d_chanlaunch.msg).to.equal('Remote service returning back state variable');
      }
      catch (err) { logger.rethrowError(err); }

      // NOTE: state only used for channel creation

      try {
        logger.info('call the url: http://localhost:50105/v1/channel_validatelaunch?oauthtoken=${oat}&state=${state}&token=${channeltoken}, as this will check that the launch works');
        d_chanlaunch = await req.post(`http://localhost:50105/v1/channel_validatelaunch?oauthtoken=${oat}&state=${state}&token=${channeltoken}`);
        logger.info('d_chanlaunch: ', d_chanlaunch);
        expect(d_chanlaunch.code).to.equal(200);
        expect(d_chanlaunch.success).to.equal(true);
        expect(d_chanlaunch.data).to.exist;
        expect(d_chanlaunch.data.url).to.exist; // would be used next
        expect(d_chanlaunch.meta.url).to.exist;
        expect(d_chanlaunch.meta.url).to.not.equal('http://localhost:50105/static/channel_validate_error.html');
      }
      catch (err) { logger.rethrowError(err); }

      logger.info('then check that database has channel and such');

      // should have 1 running RL, 2 running channels
      let rls = await ov.RemoteListenerModel.find({});
      expect(rls.length).to.equal(1);
      let channels = await ov.ChannelModel.find({});
      logger.info('channels: ', channels);
      expect(channels.length).to.equal(3);
      expect(channels[2].token).to.equal(channeltoken);
      expect(channels[2].pluginData.oauthtoken).to.equal(oat);
      expect(channels[2].pluginData.state).to.equal(state);
    });

  });

  after(async () => {
    // logger.info('wtf dump: pre at end'); wtf.dump();
    await rl.doShutdown();
    // logger.info('wtf 1');
    await ov.doShutdown();
    // logger.info('wtf 2');
    await wl.doShutdown();
    // logger.info('wtf dump: after at end'); wtf.dump(); await DB.disconnect(logger); logger.info('wtf dump: after disconnect of DB'); wtf.dump();

  });

});
