
/*
const PE      = require('../');
const PLT     = require('../../pltest/src/pl');
const Logger  = require('woveon-logger');
const URL     = require('url-parse');
const expect  = require('chai').expect;

let   c_ov    = PLT.Config;
let   c_rl    = JSON.parse(JSON.stringify(c_ov)); // duplicate config
let   c_wl    = JSON.parse(JSON.stringify(c_ov)); // duplicate config

//let mtag = require('path').basename(__filename).slice(0,-3);
*/

let h = require('./msHelper')(__filename);
let ov = rl = wl = ts = null;

// new h.PE.Service.WEConfig(new Logger('config', {debug : true}));


let logger= h.logger;

describe(`${h.mtag}: Test interaction between microservers`, () => {

  // setup the service
  before(async () => {

    //      here... get all configs and make sure they are local because testserver calls WoveonListener with returns a non-local URL

    ts = await h.genTSRequester();
    ov = await h.genOV(null, true, true);
    logger.verbose('... created Overseer');

    logger.verbose(`${h.mtag}: get rid of bad remote listeners at start`);
    await ov.RemoteListenerModel.deleteMany({});
    await ov.ChannelModel.deleteMany({});

    rl = await h.genRL(null, true, true);
    logger.verbose('... created RemoteListener');

    wl = await h.genWL(true, true);
    logger.verbose('... created WoveonListener');
  });


  describe(`${h.mtag}: test interactions of microservices`, async () => {

    it(`${h.mtag}: Overseer::deleteNeverInitedChannels`, async function() {
      logger.h1().info(this.title);

      let result = await wl.channelAuth();
      logger.info('auth    result :', result);
      expect(result.success).to.equal(true);
      let authedchannel = await ov.ChannelModel.find({token : result.data.token});
      logger.info('authed channel :', authedchannel);
      expect(authedchannel).to.be.have.lengthOf(1);

      logger.verbose('Now run ov.deleteNeverInitedChannels of default time, and this should be kept');
      await ov.deleteNeverInitedChannels();
      authedchannel = await ov.ChannelModel.find({token : result.data.token});
      logger.verbose('authed channel now :', authedchannel);
      expect(authedchannel).to.be.have.lengthOf(1);

      logger.verbose('Now run ov.deleteNeverInitedChannels of zero time, and this should be removed');
      await ov.deleteNeverInitedChannels(0);
      authedchannel = await ov.ChannelModel.find({token : result.data.token});
      logger.info('authed channel now :', authedchannel);
      expect(authedchannel).to.be.have.lengthOf(0);

    });

    it(`${h.mtag}: Overseer::checkRemoteListeners to kill out of date RL`, async function() {
      logger.h1().info(this.title);

      let rls = null;
      let authedchannel = null;

      // create a channel (auth) and mark as running
      let result = await wl.channelAuth();
      logger.info('auth    result :', result);
      expect(result.success).to.equal(true);
      authedchannel = await ov.ChannelModel.find({token : result.data.token});
      expect(authedchannel).to.be.lengthOf(1);
      authedchannel[0].isActive = 'running';
      authedchannel[0].remotelistenerID = rl.remotelistenerEntry._id;
      authedchannel[0].markModified('isActive');
      authedchannel[0].markModified('remotelistenerID');
      logger.info('saving back channel:', authedchannel[0]);
      await authedchannel[0].save();

      // verify
      authedchannel = await ov.ChannelModel.find({token : result.data.token});
      expect(authedchannel).to.be.lengthOf(1);
      expect(authedchannel[0].isActive).to.equal('running');
      expect(authedchannel[0].remotelistenerID).to.equal(''+rl.remotelistenerEntry._id);


      // ensure only 1 RL
      rls = await ov.RemoteListenerModel.find({});
      expect(rls).to.be.have.lengthOf(1);

      // ov should kill out of date RLs (beacause 0 time)
      await ov.checkRemoteListeners(0);

      // ensure 0 RLs, as it shut itself down
      rls = await ov.RemoteListenerModel.find({});
      expect(rls).to.have.lengthOf(0);

      // channel should be in state of 'inactive'
      authedchannel = await ov.ChannelModel.find({token : result.data.token});
      logger.verbose('authed channel :', authedchannel);
      expect(authedchannel).to.have.lengthOf(1);
      expect(authedchannel[0].isActive).to.equal('inactive');

    });


    it(`${h.mtag}: Overseer::recoverChannelsWithNoFunctioningRemoteListener w/ no RL`, async function() {
      logger.h1().info(this.title);

//      logger.set('level', 'verbose');

      logger.verbose('... get previous channel');
      channel = await ov.ChannelModel.find();
      expect(channel).to.have.lengthOf(1);
      expect(channel[0].isActive).to.equal('inactive');

      logger.verbose('... previous channel: ', channel);
      channel[0].isActive = 'running';
      channel[0].remotelistenerID = '0';
      channel[0].markModified('isActive');
      channel[0].markModified('remotelistenerID');
      await channel[0].save();
      logger.verbose('  ... previous channel now marked as running: ', channel);

      logger.verbose('... recover the channel');
      await ov.recoverChannelsWithNoFunctioningRemoteListener();
      channel = await ov.ChannelModel.find();
      expect(channel).to.have.lengthOf(1);
      logger.verbose('  ... channel now not running: ', channel);

      logger.set('level', 'info');
    });


    it(`${h.mtag}: Overseer::recoverChannelsWithNoFunctioningRemoteListener w/ RL`, async function() {
      logger.h1().info(this.title);
      logger.set('level', 'verbose');

      // do shutdown, but since RL killed above, there are no RLs in the database
      expect((await ov.RemoteListenerModel.find({})).length).to.equal(0);

      // shutdown and handle error
      try {
        await rl.doShutdown();
      }
      catch (e) { logger.throwError(e); }

      // create a new RL
      rl = await h.genRL(null, true, true);
      logger.verbose('... created RemoteListener');


      logger.verbose('... get previous channel');
      channel = await ov.ChannelModel.find();
      expect(channel).to.have.lengthOf(1);
      expect(channel[0].isActive).to.equal('inactive');

      logger.verbose('... previous channel: ', channel);
      channel[0].isActive = 'running';
      channel[0].remotelistenerID = rl.remotelistenerEntry._id;
      channel[0].markModified('isActive');
      channel[0].markModified('remotelistenerID');
      await channel[0].save();
      logger.verbose('  ... previous channel now marked as running, with active RL: ', channel);

      logger.verbose('... recover channels, but none to recover');
      await ov.recoverChannelsWithNoFunctioningRemoteListener();
      channel = await ov.ChannelModel.find();
      expect(channel).to.have.lengthOf(1);
      expect(channel[0].isActive).to.equal('running');
      expect(channel[0].remotelistenerID).to.equal(''+rl.remotelistenerEntry._id);
      logger.verbose('  ... channel still running: ', channel);

      logger.set('level', 'info');
    });


  });

  after(async () => {
    await rl.doShutdown();
    await ov.doShutdown();
    await wl.doShutdown();
  });


});
