
let bcrypt = require('bcryptjs');
let expect = require('chai').expect;
const ngrok = require('ngrok');

module.exports = function(_filename, _options = {ovloglvl : 'verbose', rlloglvl : 'verbose', wlloglvl : 'verbose'}) {

  this.mtag    = require('path').basename(_filename).slice(0, -3);

  this.PE      = require('../src/');
  this.Logger  = require('woveon-logger');
//  this.URL     = require('url-parse');
  this.expect  = require('chai').expect;
  this.logger  = new this.Logger(this.mtag, {debug : true, showName : true, level : 'info', color : 'inverse'});

  this.WEConfig = this.PE.Service.WEConfig;

  this.logov = new this.Logger('OV', {debug : true, showName : true, level : _options.ovloglvl});
  this.logrl = new this.Logger('RL', {debug : true, showName : true, level : _options.rlloglvl});
  this.logwl = new this.Logger('WL', {debug : true, showName : true, level : _options.wlloglvl});
  this.logts = new this.Logger('TS', {debug : true, showName : true, level : 'info'}, {requester : true});

  // open ngrok urls so we can disconnect them later
  this._ngroks = [];


  /**
   * Create a RemoteListener
   * @param {boolean} _isrunning - whether to generate a running RL
   * @param {boolean} _doinit - wheter to call init on RL
   * @param {boolean} _dostartup - wheter to call startup on RL
   * @param {object} _useexisting - Pass in your own remote listener here, and it will tune it for testing
   * @return {object} - a RemoteListener
   */
  this.genRL = async (_isrunning, _doinit, _dostartup, _useexisting = null) => {
    let retval = _useexisting;
    if (retval ==null ) {
      retval = new (require('../pltest/src/pl')).RemoteListener({logger : this.logrl});
    }

    retval.isrunning = (_isrunning === true) || null;
    if ( _doinit )    await retval.init();
    if ( _dostartup ) await retval.startup();
    return retval;
  };


  this.genOV = async (_isrunning, _doinit, _dostartup, _useexisting) => {
    let retval = _useexisting;
    if ( retval == null ) {
      retval = new (require('../pltest/src/pl')).Overseer({logger : this.logov});
    }

    retval.isrunning = (_isrunning === true) || null;
    if ( _doinit )    {
      await retval.init();
    }
    if ( _dostartup ) {
      await retval.startup();
    }
    return retval;
  };


  /**
   * @param {object} _useexisting - Pass in your own WoveonListener here, and it will tune it for testing
   * @return {object} - a WoveonListener
   */
  this.genWL = async (_doinit, _dostartup, _useexisting = null) => {
    let retval = _useexisting || new (require('../pltest/src/pl')).WoveonListener({logger : this.logwl});

    if ( _doinit )    await retval.init();
    if ( _dostartup ) await retval.startup();
    return retval;
  };

  // This requests to the 'testserver', in /woveon-engine-p/testserver, started by cd-ing there and 'make'
  this.genTSRequester = async (_resourceUrl = null, _baseurl=null) => {
    if ( _baseurl == null ) _baseurl = this.PE.Service.WEConfig.get('WOV_api_fullurl');
    return new this.PE.Service.ReqRes(this.logts, _baseurl, _resourceUrl);
  };


  /**
   * This encyrpts a password using bcrypt (salts it as well).
   */
  this.cryptPassword = async function(_password) {
    let SALT_WORK_FACTOR= 10;
    return new Promise((resolve, reject) => bcrypt.genSalt(SALT_WORK_FACTOR, (error, salt) => {
      console.log('cryptPassword: salt: ', salt, error);
      if (error) {return reject(error);}
      return bcrypt.hash(_password, salt, (err, hash) => {
        console.log('cryptPassword: hash: ', hash, err);
        if (err) {return reject(err);}
        return resolve(hash);
      });
    }));
  };


  /**
   * create session for a known user
   *   TS /api/v1/user/auth GET : mail, password
   */
  this.createSession = async function(_ts, _email, _password) {
    logger.aspect('session', '... create session');
    let ses = await _ts.get(`/user/auth?mail=${_email}&password=${_password}`);
    logger.aspect('session', '  ... ses: ', ses);
    expect(ses.code).to.equal(200);
    expect(ses.success).to.equal(true);
    expect((typeof ses.data.id == 'string')? parseInt(ses.data.id, 10):ses.data.id).to.equal(2);
    expect(ses.data.mail).to.equal(_email);
    expect(ses.data.xid).to.exist;
    expect(ses.data.session).to.exist;
    expect(ses.data.name).to.exist;
    return ses;
  };


  /**
   * INCOMPLETE
   *
   * launch a channel
   *
   * NOTE: Can return two URLs
   *   - remote service (which will redirect)
   *   - static web page to collect user data
   *
   * @param {object} _ses - session object created with createSession
   */
  this.channelAuth = async function(_ses, _title, _teamid) {
    let retval = null;
    logger.info('  ... auth channel');

    let result = await ts.post('/user/protect/team/protect/channel', null, {
      teamId   : _teamid,
      sourceId : PLConfig.get('WOV_plugin_id'),
      title    : _title,
      session  : ses.data.session,
    });
    logger.info('result: ', result);
    if ( result.code != 200 ) retval = WovReturn.retError('failed contacting api to auth channel. Is testserver on?');
    else retval = WovReturn.retSuccess(result.data);

    return retval;
  };


  /**
   * INCOMPLETE
   *
   * Call this from the URL returned from channelAuth. This checks the loop from X to remote service and
   * back to its return URL registered with the plugin and that remote service.
   * NOTE: not all plugins will do this, since they will go to a local webpage to collect data to validate
   *
   * @param {string} _remoteserviceURL - URL returned from channelAuth
   */
  this.channelStartForRedirectPlugins = async function(_remoteserviceURL) {
    let retval = null;
    let rq     = new PE.Service.Requester(logger);

    logger.info('Contact /channel_start to begin, which redirects to RemoteService and back to /channel_launch');
    logger.info('  channel_start with post url : ', _remoteserviceURL);

    // get raw result ( a redirect) // {'Content-Type' : 'text/html'}; // , {t : dAuth.data.token});
    let result = await rq.get(_remoteserviceURL, null, true);
    try {
      // logger.info('result: ', result);
      expect(result.constructor.name).to.equal('Response');
      expect(result.ok).to.equal(true);
      expect(result.status).to.equal(200);
      expect(result.statusText).to.equal('OK');
      expect(result.headers.get('content-type').startsWith('text/html')).to.be.true;
    }
    catch (err) { retval = WovReturn.retError(err); };

    if ( retval == null ) { retval = WovReturn.retSucces(await result.text()); }
    return retval;
  };


  /**
   * INCOMPLETE
   *
   * Calls the plugin to validate with this data.
   * @param {url} _authURL - from the initial auth call. we pull the host name and such from this.
   * @param {object} _channelData - data/credentials used to validate this channel.
   */
  this.channelValidate = async function(_authURL, _channelData) {
    let retval = null;
    let rq     = new PE.Service.Requester(logger);
    logger.info('... channelValidate');

    // use authURL
    let u    = url.parse(_authURL, true);
    let newu = `${u.protocol}//${u.host}/${PLConfig.get('WOV_api_ver')}/channel_validatelaunch`;
    let result = await rq.post(newu, null, _channelData);

    // above post returns ultimately from /channel_launch
    try {
      logger.info('result: ', result);
      expect(result.success).to.equal(true);
      expect(result.code).to.equal(200);
      expect(result.data).to.exist;
    }
    catch (err) { retval = WovReturn.retError(err); }

    if ( retval == null ) { retval = WovReturn.retSucces(true); }
    return retval;

  };


  /**
   * Checks _envvar in config if it is NGROK. If so, replace with an ngrok url going to localhost:_port.
   * @param {string} _envvar - Name of the env variable to check.
   * @param {int} _port - port to tell ngrok to route to if ngrok invoked.
   */
  this.ngrokConfig = async function(_envvar, _port) {
    // this.logger.info(`ngrokConfig: ${_envvar} ${_port} : ${this.WEConfig.get(_envvar)}`);
    if ( this.WEConfig.get(_envvar) == 'NGROK' ) {
      let url = await ngrok.connect(_port);
      this._ngroks.push(url);
      this.logger.info(`ngrokConfig: replacing ${_envvar} with ${url} on port ${_port}.`);
      this.WEConfig.set(_envvar, url);
    }
  };


  /**
   * Call when done to disconnect.
   */
  this.ngrokDisconnect = async function() {
    let allproms = [];
    while ( this._ngroks.length != 0 ) {
      let url = this._ngroks.pop();
      allproms.push(ngrok.disconnect(url));
    }

    // await in parallel
    await Promise.all(allproms);

  };


  return this;
};
