

/**
 * This class simulates something like Facebook or an Email server, a RemoteService
 * to Woveon, that has data we are going to suck in.
 */

const autoBind         = require('auto-bind-inheritance');
const DownloaderEngine = require('./downloaderEngine');
const RemoteDataSipper = require('../src/microservices/remoteDataSipper');

const l = require('../src/service').l;

module.exports = class SimulatedRemoteService { // extends RemoteDataSipper {

  constructor(_curtime, _logger, _config) {
    // super('SimulatedRemoteService', _config, { logger: _logger });
    //

    this.logger = _logger;
    this.engine = null;
    this.posts = [];

    this.resetSim(_curtime, l(0));

    autoBind(this);
  }

  /**
   * @param {object} _engine - the engine that this uses
   */
  linkToEngine( _engine) { this.engine = _engine; this.engine.rs = this; }

  /**
   * Reset time, not data.
   * @param {int} _curtime
   * @param {int} _monitor
   */
  resetSim(_curtime, _monitor) {
    if ( typeof _curtime != 'object' ) { this.logger.throwError('passing non Long to resetSim: _curtime ', _curtime); }
    if ( typeof _monitor != 'object' ) { this.logger.throwError('passing non Long to resetSim: _monitor ', _monitor); }

    this._curtime = l(_curtime);
    if ( this.engine ) this.engine.resetSim(_monitor);
    RemoteDataSipper.orderedStringReset();
  }

  /**
   * Go to next time of update, sending update to engine.
   * @param {Long} _newtime - New time of the simulation.
   */
  async setToTime(_newtime) {
    let bounds = [l(-1), l(-1)];
    let i = 0;
    this.logger.info(`setToTime : ${this._curtime} -> ${_newtime}`);

    this.logger.with('add_to_stacklvl', 1).aspect(`setToTime`, `setToTime : ${this._curtime} -> ${_newtime}`);

    // find start, save index
    while (i < this.posts.length && this.posts[i][0].lessThanOrEqual(this._curtime)) { i++; }
    bounds[0] = l(i);

    // go to this _newtime, save index
    this.logger.info('  posts: ', this.posts[i][0], _newtime);
    while (i < this.posts.length && this.posts[i][0].lessThanOrEqual(_newtime)) { i++; }
    bounds[1] = l(i);

    this.logger.aspect(`setToTime`, ` - bounds: ${bounds}`);

    // send all updates to engine
    for (i=0; l(i).lessThan(bounds[1]); i++) {
      this.logger.info('  ... going to add msg index: ', i);
      let p = this.posts[i];
      await this.engine._addMsg(p[0], p[1]);
    }
    this._curtime = _newtime;
  }


  /**
   * Adds data.
   * @param {Long} _time -
   * @param {object} _data -
   */
  setPost(_time, _data) {
    this.logger.with('add_to_stacklvl', 1).info(`setPost ${_time} : ${_data}`);
    this.posts.push([_time, _data]);
    this.posts.sort(DownloaderEngine._compareEntries);
  }


  /**
   * @param {String} _msg -
   */
  displayMe(_msg) {
    if ( _msg == null ) _msg = '';
    else _msg = `'${_msg}' `;
    this.logger.with('add_to_stacklvl', 1).info(`SimulatedRemoteService: ${_msg}length:${this.posts.length} curtime:${this._curtime}`);
    for (let i=0; i<this.posts.length; i++) {
      let e = this.posts[i];
      this.logger.info(`${i}: '${JSON.stringify(e)}'`);
    }
  }

  /**
   */
  compareToEngine() {
    if ( this.posts.length != this.engine.posts.length ) {
      this.logger.error(`Error in ${this.constructor.name}.compareToEngine`);
      this.displayMe();
      this.engine.displayMe();
      throw new Error('Sim and Engine not even same length.');
    }
    else {
      for (let i=0; i< this.posts.length; i++ ) {
        if ( (!this.posts[i][0].equals(this.engine.posts[i][0]) ) ||
             (this.posts[i][1] != this.engine.posts[i][1] ) ) {
          this.logger.error(`Error in ${this.constructor.name}.compareToEngine`);
          this.displayMe();
          this.engine.displayMe();
          throw new Error(`Sim and Engine differ at index ${i}`);
        }

      }

    }

  }
};
