

// Imported Code
const should            = require('should');
const URL               = require('url-parse');
const crypto            = require('crypto');
// const nock              = require('nock');

// Service Code
// let   testconfig        = require('../testconfig')
// const Woveon            = require('../service/woveon');
const Logger            = require('woveon-logger');

let   config            = require('../service/config');
const Helper            = require('./helper');
const Woveon            = require('../service/woveon');
const Requester         = require('../service/requester');
const Service           = require('../service/service');
const DB                = require('../service/db');
const ChannelModel      = require('../models/channelConnector');

let service      = null;
let channelModel = null;
let channeltoken = null;
let channel      = null;
let apiUrl       = new URL(config.api.url);
let session      = null; // loaded from API
let db           = null;

arrayCompare = (a1,a2) => {
  return a1.length==a2.length && a1.every((v,i)=> v === a2[i])
};

// This should be imported from the secrets file
let testconfig = Helper.testconfig;


  let mtag = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger(mtag,
    { showName: false, debug: true, level: 'info' },
    { 'start': { color: 'blue' }, 'listener' : true, 'woveon' : true, 'requester': true });
  //let sdk    = new Woveon(config, logger);

  /*
  try {
    config.service.port=0;
    config.service.address='127.0.0.1';
    Service.validateConfig(config, logger);
  } catch (err) {
    logger.error(err);
    logger.info('config:', config);
    process.exit(1);
  }
  */

describe(`${mtag}: Woveon SDK Tests`, () => {

  describe(`${mtag}: A. Local plugin test calls`, () => {


    // 1. start service to listen for API
    // 2. auth a channel in API for this plugin
    // 3. create a channel in plugin
    // 4. activate the channel with API

    it(`${mtag}: A.1- start service to listen for API`, async () => {
      logger.info(`${mtag}: A.1- start service to listen for API`);
      // create a testing channel

      // encrypt password
      logger.info('/start 1.1');
      testconfig.user.epassword = await Helper.cryptPassword(testconfig.user.password);

      //        process.exit(1);
      // The following used 'config.plugin.port' but maybe its rl_port instead of wl_port
      config.service.port = config.plugin.wl_port; // config.plugin.port; 
      console.log('plugin  : ', config.plugin);
      console.log('service : ', config.service);

      //        config.service.address= config.plugin.address; //'127.0.0.1';
      service = new Service('TestService', config, { 'logger': logger});
      db = new DB(service.logger);
      await db.init(service.config.plugin.dburl);
      service.toWoveon = new Woveon(service.config, service.logger, db);
      service.requester = new Requester(service.logger);


      await service.init()
        .then(async () => {

          // for testing
          await service.listener.onGet('/ping', async () => { logger.info('pong'); return 'pong'; }, __filename);

          // API calls to start channel creation
          await service.listener.onGet('/auth', async () => {

            // create the channel placeholder and return {url:,token:} to the API
            logger.info('/auth handling 1');
            channeltoken = Service.generateRandomString();
            channelModel = ChannelModel(service.db);
            channel = new channelModel({ token: channeltoken });
            logger.info('create channel: ', channel);
            return await channel.save()
              .catch((err) => { logger.error(err); process.exit(1); })
              .then(() => {
                let retval = {
                  url: `http://${service.config.plugin.url}:${config.service.port}/remote_auth?t=${channeltoken}`,
                  token: channeltoken,
                };
                return retval;
              })
              .catch((err) => { logger.error(err); process.exit(1); })
              .then((newchannel) => {

                console.log('new channel ', newchannel);
                return newchannel;

                // delete the created channel
                /*
                  return channelModel.remove({ token: channeltoken })
                    .catch((err) => { logger.error(err); process.exit(1); })
                    .then((_result) => {
                      console.log('_result of remote : ', _result.result);
                      _result.result.ok.should.equal(1);
                      _result.result.n.should.equal(1);
                    });
                    */

              })
              .catch((err) => { logger.error(err); process.exit(1); });
          }, __filename);

          await service.listener.onGet('/remote_auth', () => {
            console.log('TODO');
            process.exit(1);
          }, __filename);

          await service.startup();

          logger.info('...starting to listen?');
        })
        .catch((err) => { logger.error(err); process.exit(1); });
    });


    it(`${mtag}: A.2. test service /ping`, (done) => {
      logger.info(`${mtag}: A.2. test service /ping`);

      let url = `http://${config.plugin.url}:${config.service.port}/ping`;
      logger.info(`GET ${url}`);
      service.requester.get(url)
        .catch((err) => { logger.error(err); process.exit(1); })
        .then((result) => {
          logger.info('/ping: ', result);
          result.success.should.be.true();
          result.data.should.be.equal('pong');
          done();
        })
        .catch((err) => { logger.error(err); process.exit(1); });
    });


    it(`${mtag}: A.3. test own call to /auth`, (done) => {
      logger.info(`${mtag}: A.3. test own call to /auth`);

      let url = `http://${config.plugin.url}:${config.service.port}/auth`;
      // let url = `http://127.0.0.1:50105/auth`;
      logger.info(`GET ${url}`);
      service.requester.get(url)
        .catch((err) => {logger.error(err); process.exit(1);})
        .then((result) => {
          logger.info('/auth: ', result);
          result.success.should.be.true();
          done();
        })
        .catch((err) => { logger.error(err); process.exit(1); });
    });

  });




  describe(`${mtag}: B. API plugin coordination test calls`, () => {


    it(`${mtag}: B.1 - create a session for the test user`, (done) => {
      logger.info(`${mtag}: B.1 - create a session for the test user`);

      try {

        Helper.initAPISession()
          .then((_session) => { session = _session; done(); })
          .catch((err) => { logger.error(err); process.exit(1); });

        /*
        // console.log(`hashed password : ${testconfig.user.epassword}`);
        let url = `${apiUrl.protocol}//${apiUrl.hostname}:${apiUrl.port}`+
          `/api/v1/user/auth?mail=${testconfig.user.email}&password=${testconfig.user.password}`;
        logger.info('begin create channel in API: ', url);

        new Promise(async () => {
          await service.requester.get(url)
            .catch((err) => {logger.error(err); process.exit(1);})
            .then((result) => {
              logger.info('Created Session with API: ', result);
              result.success.should.be.true();
              session = result.data;
              done();
            })
            .catch((err) => {logger.error(err); process.exit(1);});
        });
        */
      } catch (err) { console.log(err); process.exit(1); }
    });


    it(`${mtag}: B.2 - create a channel in API for this plugin (API contacts plugin /auth to getToken)`, (done) => {
      logger.info(`${mtag}: B.2 - create a channel in API for this plugin (API contacts plugin /auth to getToken)`);

      logger.info('using session: ', session);
      try {
        Helper.emulateBrowserChannelAuth(session)
          .then((_result) => {
            logger.info(' - channel auth returned ', _result);
            _result.success.should.be.true();
            should.notEqual(_result.data, null);
            let url = new URL(_result.data,true);
            channel = {token : url.query.t, url: 'stuff'};
            //channel = {token : _result.data[0].token, url: _result.data[0].pageImage};
            done();
          })
          .catch((err) => { logger.error(err); process.exit(1); });
      } catch (err) { console.log(err); process.exit(1); }
    });

    /*
      it('4- create channel in plugin', (done) => {
        channelModel = ChannelModel(service.db);
        channel = new channelModel({ token: channeltoken });
        logger.info('using channel: ', channel);
        channel.save()
          .then(() => { done(); })
          .catch( (err)=>{logger.error(err); process.exit(1); });
      });
      */


    it(`${mtag}: B.3 - activateChannel in API`, async () => {
      logger.info(`${mtag}: B.3 - activateChannel in API`, channel);

      let icon = 'asdf';

      try {
        let result = await service.toWoveon.activateChannel(channel.token, icon);
        logger.info('  - B.3 - ', JSON.stringify(result));
        result.success.should.be.true();
        result.data.token.should.be.equal(channel.token);
        result.data.pageImage.should.be.equal(icon);
      } catch (err) { logger.error(err); throw new Error('failed');; };

    });

  });


  describe(`${mtag}: C. - Test calls to API with SDK`, () => {
    logger.info('C');

    let p = null;
    let pu = null;
    let p_created = null;
    let p_updated = null;

    it(`${mtag}: C.1 - createPost`, (done) => {
      logger.info(`${mtag}: C.1 - createPost`);

      p = Woveon.genPost( channel.token, 'AAAA', '',
        `${new Date().getTime()}_${crypto.randomBytes(4).toString('hex')}`,
        'A Subject', 'Test message', false);

      try {
        logger.info('createPost:', JSON.stringify(p));

        service.toWoveon.createPublicPost(p.token, p.author,
          p.authorUrl, p.postId, p.subject, p.message)
          .catch((err) => { logger.error(err); process.exit(1); })
          .then((result) => {
            logger.info('  - C.1 - result:', JSON.stringify(result));
            result.success.should.be.true();
            result.data.postId.should.be.equal(p.postId);
            result.data.author.should.be.equal(p.author);
            result.data.subject.should.be.equal(p.subject);
            result.data.message.should.be.equal(p.message);
            result.data.id.should.not.be.equal(null);
            result.data.recipient.should.be.equal(testconfig.team.title);
            p_created = result.data;
            done();
          })
          .catch((err) => { logger.error(err); throw new Error('failed'); });
      } catch (err) { logger.error(err); throw new Error('failed'); }
    });


    it(`${mtag}: C.2 - updatePost`, (done) => {
      logger.info(`${mtag}: C.2 - updatePost`);

      try {
        pu = {
          token   : p.token,
          id      : p_created.id,
          subject : 'A New Subject',
          message : 'A new message.',
          images  : ['img1.png', 'img2.png'],
          videos  : ['vid1.mp4', 'vid2.mp4'],
        };

        logger.info('updatePost:', JSON.stringify(pu), ' : ', p_created.id);
        service.toWoveon.updatePost(pu)
          .catch((err) => { logger.error(err); process.exit(1); })
          .then((result) => {

            logger.info('update result:', result);

            // success check
            result.success.should.be.true();

            // changed
            result.data.subject.should.be.equal(pu.subject);
            result.data.message.should.be.equal(pu.message);
            should.equal(arrayCompare(result.data.images, pu.images), true);
            should.equal(arrayCompare(result.data.videos, pu.videos), true);

            // not changed
            result.data.id.should.be.equal(p_created.id);
            result.data.date.should.be.equal(p_created.date);
            try {
              result.data.lastActivity.should.not.be.equal(p_created.lastActivity);
            } catch (err) {
              logger.warn("ERROR ON API: server should update lastActivity with update. Bummer. Continuing.");
            }

            p_updated = result.data;
            done();
          })
          .catch((err) => { logger.error(err); throw new Error('failed'); });
      } catch (err) { logger.error(err); throw new Error('failed'); }

    });


    it(`${mtag}: C.3 - deletePost`, (done) => {
      logger.info(`${mtag}: C.3 - deletePost`);

      try {
        let dp = {token: p.token, id: p_created.id};
        logger.info('deletePost:', JSON.stringify(dp));
        service.toWoveon.deletePost(dp)
          .catch((err) => { logger.error(err); process.exit(1); })
          .then((result) => {
            logger.info('result:', result);
            result.success.should.be.true();
            done();
          })
          .catch((err) => { logger.error(err); throw new Error('failed'); });
      } catch (err) { logger.error(err); throw new Error('failed'); }
    });

  });

  describe(`${mtag}: D. - Clean up`, () => {

    it(`${mtag}: D.1 - clean up models`, async () => {
      logger.info(`${mtag}: D.1 - clean up models`);

      await Helper.modelsClear();
    });

    it(`${mtag}: D.2 - close server`, (done) => {
      logger.info(`${mtag}: D.2 - close server`);

      service.shutdown()
        .then(() => { done(); })
        .catch((err) => { logger.error(err); throw new Error('failed'); });
    });

  });

});
