
// Imported Code
const should            = require('should');
const nock              = require('nock');

// Actual Code
const config            = require('../service/config');
const Requester         = require('../service/requester');
const RemoteListenerService     = require('../microservices/remoteListenerService');
const Logger            = require('woveon-logger');

const ChannelModel      = require('../models/channelModel');
const RemoteListenerModel       = require('../models/remoteListenerModel');

let service           = null;
let channelModel      = null;
let remotelistenerModel       = null;

module.exports = () => {

  let mtag = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger(mtag,{debug: true, showName: true});

  describe(`${mtag}: Call local msRemoteListener funcitonality`, () => {


    // Clears out Channel and RemoteListener Models
    /*
    beforeEach((done) => {
      ChannelModel.remove({})
        .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); })
        .then( () => { return RemoteListenerModel.remove({}); })
        .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); })
        .then( () => { done(); })
        .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); });
    });
    */



    describe(`${mtag}: simple tests`, () => {

      // setup the service
      before((done) => {
        config.service.port = 0;
        service = new RemoteListenerService(config);
        service.logger.options.debug = true;
        service.init()
          .then( () => {
            channelModel = ChannelModel(service.db);
            remotelistenerModel = RemoteListenerModel(service.db);
            service.startup(); 
          })
          .then(() => {done();})
          .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });
      });


      it(`${mtag}: call workrrController.health`, (done) => {
        logger.info(`${mtag}: call workrrController.health`);
        service.health()
          .then((healthresult) => {
            // console.log('healthresult', healthresult);
            healthresult.should.be.true();
            done();
          })
          .catch((err) => { service.logger.error(err); throw new Error('failed'); });
      });


      after( (done) => {
        // service.shutdown().then( ()=> { done(); })
        //  .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });
        if ( service.db.isConnected()) {
          channelModel.remove({})
            .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); })
            .then(() => { return remotelistenerModel.remove({}); })
            .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); })
            .then(() => { done(); })
            .catch((err) => { console.log(err); throw new Error('Failed clearing database.'); });
        } else {
          done();
        }
      });


    });


    describe(`${mtag}: path tests`, () => {

      // setup the service
      before((done) => {
        config.service.port = 0;
        service = new RemoteListenerService(config);
        service.requester = new Requester(service.logger);
        service.init()
          .then( () => { service.startup(); })
          .then(() => { done(); })
          .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });
      });

      it(`${mtag}: emulate call to remotelistenerController.health`, (done) => {

        logger.info(`${mtag}: emulate call to remotelistenerController.health`);

        let testurl = `${service.config.plugin.url.slice(0, -1)}:${service.config.service.port}`;
        // console.log('testing url ', testurl);
        let ns = nock(testurl)
          .get('/health')
          .reply(200, {success: true, data: true});

        // console.log('Nock checking: ', ns);
        service.requester.get(`${testurl}/health`)
          .catch((err) => {
            console.log(err);
            throw new Error('unexpected error');
          })
          .then((healthresult) => {
            //console.log('healthresult', healthresult);
            Object.keys(healthresult).length.should.be.equal(2);
            healthresult.success.should.be.true();
            healthresult.data.should.be.true();

            done();
          })
          .catch((err) => { console.log(err); throw new Error('failed'); });
      });


      it(`${mtag}: emulate call to remotelistenerController.start_channel`, (done) => {
        logger.info(`${mtag}: emulate call to remotelistenerController.start_channel`);

        let t = RemoteListenerService.generateRandomString();
        let testurl = `${service.config.plugin.url.slice(0, -1)}:${service.config.service.port}`;
        console.log('testing url ', testurl);
        nock(testurl)
          .post(`/start_channel`)
          .reply(200, {success: true, data: true});

        // create a channel to start
        let channel = new channelModel({token: t});
        channel.save()
          .then(() => {
            service.requester.post(`${testurl}/start_channel`, null, { token: t })
              .catch((err) => { console.log(err); throw new Error('unexpected error'); })
              .then((result) => {
                console.log('startChannel:', result);
                Object.keys(result).length.should.be.equal(2);
                result.success.should.be.true();
                result.data.should.be.true();
                done();
              })
              .catch((err) => { console.log(err); throw new Error('unexpected error'); });
          });
      });


      it(`${mtag}: call remotelistenerController.stop_channel... TODO.`);

      it(`${mtag}: call remotelistenerController.postStartup ... TODO`);

    });

  });

};
