

const should            = require('should');
const URL               = require('url-parse');

const Helper            = require('./helper');

const config            = require('../service/config');
const Logger            = require('woveon-logger');

const WoveonListenerService = require('../microservices/woveonListenerService');
const RemoteListenerService     = require('../microservices/remoteListenerService');


module.exports = () => {

  let mtag = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger('Logger',
    { showName: false, debug: true, level: 'info' },
    { 'start': { color: 'blue' }, 'listener': true, 'woveon': true, 'requester': true });
  let remotelistenerLoad = {maxchannels: 1};


  // WoveonListener
  let gconfig = JSON.parse(JSON.stringify(config));
  gconfig.service.port = config.plugin.port;
  let woveonlistener = null;

  // RemoteListener
  let wconfig = JSON.parse(JSON.stringify(config));
  wconfig.service.port = 0;
  let remotelistener = null;

  let session = null;
  let channels = [];


  describe(`${mtag}: WoveonListener params testing`, () => {
  

    // Clear database
    before( async () => { await Helper.modelsClear(); console.log('cleared models'); });


    it(`${mtag}: setup the services`, (done) => {
      logger.info(`${mtag}: setup the services`);

      woveonlistener = new WoveonListenerService(gconfig, {
        remotelistenerLoad : remotelistenerLoad,
        logger     : new Logger('G', {showName: true, debug: true},
          { listener: false, requester: false }), });

      remotelistener = new RemoteListenerService(wconfig, { 
        logger: new Logger('W', { showName: true, debug: true, level: 'verbose' },
            { listener: true, requester: false }) });

      remotelistener.init()
        .catch((err) => { console.log(err); throw new Error('failed remotelistener.init'); })
        .then(() => { return remotelistener.startup(); })
        .catch((err) => { console.log(err); throw new Error('failed remotelistener.startup'); })
        .then(() => { return woveonlistener.init(); })
        .catch((err) => { console.log(err); throw new Error('failed woveonlistener.init'); })
        .then(() => { return woveonlistener.startup(); })
        .catch((err) => { console.log(err); throw new Error('failed woveonlistener.startup'); })
        .then(() => { done(); })
        .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });
    });


    it(`${mtag}: init session`, (done) => {
      logger.info(`${mtag}: init session`);
      Helper.initAPISession(woveonlistener)
        .then((_session) => {session = _session; done();})
        .catch((err) => {woveonlistener.logger(err); throw new Error('Failed test.');});
    });


    it(`${mtag}: Spawn Channel, should be 1 in model.`, async function() {
      logger.h3().info(this.test.title);
      let newchanneltoken = null;

      await Helper.createNewChannel(woveonlistener, session)
        .then( (_newchanneltoken) => {
          console.log('newchannel token : ', _newchanneltoken);

          should.notEqual(_newchanneltoken, null);
          channels.push(_newchanneltoken);
          newchanneltoken=_newchanneltoken;
          return woveonlistener.ChannelModel.find();
        })
        .then( (chs) => {
          woveonlistener.logger.info('chs:', chs);
          chs.length.should.be.equal(1);
          return woveonlistener.RemoteListenerModel.find({})
                    .then( (_remotelisteners) => {
                      console.log('worekrs : ', _remotelisteners);
                      _remotelisteners.length.should.be.equal(1); // one remotelistener with one channel
                      _remotelisteners[0].numchannels.should.be.equal(1);
                    });
        })
        .catch((err) => {console.log(err); throw new Error('failed woveonlistener test');})

        /*
          return woveonlistener.launch({token: newchanneltoken})
            .catch((err) => { console.log(err); throw new Error('failed mswoveonlistener test'); })
            .then((result) => {
 //             woveonlistener.logger.info('woveonlistener launch result: ', result);
              return result;
            });
        })
        .catch((err) => {console.log(err); throw new Error('failed woveonlistener test');})
        .then( (_lresult) => {
//          console.log('launch result:', _lresult);
          _lresult.should.be.true();

          return woveonlistener.RemoteListenerModel.find({})
                    .then( (_remotelisteners) => {
                      console.log('worekrs : ', _remotelisteners);
                      _remotelisteners.length.should.be.equal(1); // one remotelistener with one channel
                      _remotelisteners[0].numchannels.should.be.equal(1);
                    });
        })
        .then( ()=>{done();})
        */
        .catch((err) => {console.log(err); throw new Error('failed woveonlistener test');});
    });


    it(`${mtag}: spawn second remotelistener... fail because only 1 per remotelistener service`, async function() {
      //this.timeout(0);
      console.log('\n*******************************************************');
      console.log(`${mtag}: spawn second remotelistener... fail because only 1 per remotelistener service`);

      let remotelisteners = await woveonlistener.RemoteListenerModel.find({});
      logger.info('remotelistenermodls ast start : ', remotelisteners);
      let newchanneltoken1;

      await Helper.createNewChannel(woveonlistener, session, null,null, null)
        .then( async (_newchanneltoken) => {
          console.log('created new channel: ', _newchanneltoken);
          should.notEqual(_newchanneltoken, null);
          newchanneltoken1 = _newchanneltoken;

          let remotelisteners = await woveonlistener.RemoteListenerModel.find({});
          try {
            remotelisteners.length.should.be.equal(2);
          } catch (err) { logger.warn('Caught error. When the spawning of a remotelistener is created in WoveonListener, this will not occur'); }

          let channels = await woveonlistener.ChannelModel.find({});
          //console.log('channels:', channels);

          channels.length.should.be.equal(2);
          channels[1].token.should.equal(_newchanneltoken);
          if ( channels[1]['remotelistenerID'] != null )
            throw new Error('Channel should not have a remotelistenerid, since remotelistener overloaded.');

          return channels;
        })
        .then( (chs) => {

          //retry launch
          return woveonlistener.launch({token: newchanneltoken1})
            .catch((err, _res) => {
              woveonlistener.logger.info('Caught launch error since can\'t find available remotelistener.');
              return false;
            });
        })
        .then( (_lresult) => {
          console.log('launch result:', _lresult);
          _lresult.should.be.false();
        })
        .catch((err) => {console.log(err); throw new Error('failed woveonlistener test');});
    });


    it('raise work capacity and wait for it to grab the remaining channel.');

    it('launch 2nd remotelistener and then launch 3 more channels. Should fail on 3rd.');

    it('shutdown a channel. wait for remotelistener to find unhandled channel.');

    it('shutdown two more channels of 2nd remotelistener. Shutdown the remotelistener.');

    it('shutdown 2nd channel of 1st remotelistener. add a new remotelistener. should be fine.');


    after( async () => {
      logger.h1().info('after hit');
      await remotelistener.shutdown();
      await woveonlistener.shutdown();
      await Helper.modelsClear();
    });

  });

};
