
// Imported Code
const should            = require('should');
const nock              = require('nock');
const URL               = require('url-parse');
const qs                = require('querystringify');

// Actual Code
const config            = require('../service/config');
const Logger            = require('woveon-logger');
const RemoteListenerService     = require('../microservices/remoteListenerService');
const WoveonListenerService = require('../microservices/woveonListenerService');
const Helper            = require('./helper');

let mswoveonlistener        = null;
let msremotelistener            = null;
let session             = null;


// This should be imported from the secrets file
let testconfig = Helper.testconfig;
let logger = new Logger('Logger',
  { showName: false, debug: true, level: 'info' },
  { 'start': { color: 'blue' }, 'listener': true, 'woveon': true, 'requester': true });


module.exports = () => {

  let mtag = require('path').basename(__filename).slice(0,-3);

  describe(`${mtag}: Test between MicroServices`, () => {

    before(async () => { await Helper.modelsClear(); });

    describe(`${mtag}: all good testing, micorserives up`, () => {
      logger.info(`${mtag}: all good testing, micorserives up`);
      let newchanneltoken = null;

      // setup the service
      before((done) => {
        let wconfig = JSON.parse(JSON.stringify(config));
        let gconfig = JSON.parse(JSON.stringify(config));
        wconfig.service.port = 0;
        gconfig.service.port = config.plugin.port;
        console.log('options');
        msremotelistener     = new RemoteListenerService(wconfig,
          {logger : new Logger('W',{showName: true, debug: true}, { listener:true, requester: true})});
        mswoveonlistener = new WoveonListenerService(gconfig,
          {logger : new Logger('G', {showName: true, debug: true}, { listener : true, requester: true})});
        msremotelistener.init()
          .catch((err)=>{console.log(err); throw new Error('failed msremotelistener.init');})
          .then( () => { return msremotelistener.startup(); })
          .catch((err)=>{console.log(err); throw new Error('failed msremotelistener.startup');})
          .then( () => { return mswoveonlistener.init(); })
          .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener.init');})
          .then( () => { return mswoveonlistener.startup(); })
          .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener.startup');})
          .then(() => {done();})
          .catch((err) => { console.log(err); throw new Error('Failed setting up testing');});
      });


      it(`${mtag}: init session`, (done) => {
        logger.info(`${mtag}: init session`);

        Helper.initAPISession(mswoveonlistener)
          .then( (_session) => { session = _session; done(); })
          .catch((err) => {mswoveonlistener.logger(err); throw new Error('Failed test.');});
      });


      it(`${mtag}: simple health test`, (done) => {
        logger.info(`${mtag}: simple health test`);

        console.log('session : ', session);
        console.log('one test');
        msremotelistener.health()
          .then((healthresult) => {
            // console.log('healthresult', healthresult);
            healthresult.should.be.true();
            done();
          })
          .catch((err) => { msremotelistener.logger.error(err); throw new Error('failed'); });
      });


      it(`${mtag}: Browser Channel Auth & Browser Remote Auth`, (done) => {
        logger.info(`${mtag}: Browser Channel Auth & Browser Remote Auth`);

        Helper.emulateBrowserChannelAuth(session)
          .then( (_result) => {
            logger.info('emulate result: ', _result);
            _result.success.should.be.true();
            should.notEqual(_result.data, null);

            let remoteurl = new URL(_result.data);
            console.log('url parsed: ', remoteurl.toString());
            newchanneltoken = qs.parse(remoteurl.query).t;
            should.notEqual(newchanneltoken,null);
            mswoveonlistener.logger.info('new channel token: ', newchanneltoken);


            // Browser Remote Auth
            // mswoveonlistener.toRemoteListeners.post(`${remoteurl.address}:${remoteurl.post}/start_channel`, 
            //                             null, {token: newchanneltoken})
            mswoveonlistener.toRemoteListeners.get(_result.data)
              .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener.toRemoteListeners');})
              .then((healthresult) => {
                mswoveonlistener.logger.info('returned startChannel:', healthresult);
                Object.keys(healthresult).length.should.be.equal(2);
                healthresult.success.should.be.true();
                done();
                healthresult.data.should.be.true();
              })
              .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener test');});
          })
          .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener test');});
      });


      /*
      it('full /auth run', (done) => {

        mswoveonlistener
          .auth({appSecretToken: mswoveonlistener.config.plugin.secret})
          .then( (result) => {

            // check result
            mswoveonlistener.logger.info('mswoveonlistener result:', result);
            Object.keys(result).length.should.be.equal(2);
            result.url.should.not.be.null();
            result.token.should.not.be.null();
            newchanneltoken = result.token;

            // check database entry
            mswoveonlistener.logger.info('find channel');
            return mswoveonlistener.channelModel.find({token: newchanneltoken});
          })
          .catch((err) => { mswoveonlistener.logger.error(err); console.log(err); throw new Error('failed'); })
          .then((result2) => {
//            console.log('result 2: ', result2);
            mswoveonlistener.logger.info('result 2: ', result2);
 //           console.log('result 2: ', result2);
            result2.should.be.length(1);
            result2[0].token.should.not.be.null;
            result2[0].token.should.be.equal(newchanneltoken);
            done();
          })
          .catch((err) => { mswoveonlistener.logger.error(err); process.exit(1); })
      });
      */


      it(`${mtag}: call /launch from a browser or user action (or remote service callback possibly)`, 
        (done) => {
          logger.info(`${mtag}: call /launch from a browser or user action (or remote service callback possibly)`);

        // call to API
        /*
        nock('http://local-api.woveon.com:4020')
          .post('/api/v1/application/protect/')
          .reply(200, { success: true, 
            data: {
              "teamid"        : testconfig.team.id,
              "applicationid" : testconfig.app.id,
              "title"         : testconfig.team.title,
              "pageImage"     : "asdf", 
              "token"         : "GMFSQ4MC6EO4JAMXX7M3",
              "date"          : "2017-11-10T20:33:03.824Z",
              "active"        : true,
              "id"            : 1,
            }});
            */

        mswoveonlistener
          .launch({ token: newchanneltoken })
          .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener test');})
          .then((result) => {
            mswoveonlistener.logger.info('woveonlistener result: ', result);
            done();
          })
          .catch((err)=>{console.log(err); throw new Error('failed mswoveonlistener test');});
      });

      it('call remotelistener.start_channel but fail and go to next');

      after( (done) => {
        // service.shutdown().then( ()=> { done(); })
        //  .catch((err) => { console.log(err); throw new Error('Failed setting up testing'); });

        try {

        msremotelistener.shutdown()
          .catch( (err) => {console.log(err); throw new Error('failed');})
          .then( (res) => {console.log('msremotelistener shutdown', res); return mswoveonlistener.shutdown();})
          .then( (res) => {console.log('mswoveonlistener shutdown', res); return Helper.modelsClear();})
          .then( () => {done();});
        } catch (err) {
          console.log(err);
          throw Error('Failed');
        }
      });

    });

  });
};