

const should            = require('should');

const Logger            = require('woveon-logger');

// const Helper            = require('./helper');
const dbHelper          = require('./dbHelper');
const Woveon            = require('../service/woveon');
const Service           = require('../service/service');
const DB                = require('../service/db');
let config              = require('../service/config');


module.exports = () => {
  let teamchannel = {
    pluginId : process.env.WOV_plugin_id,
    teamId   : process.env.WOV_plugin_team_id,
    token    : null,
  };
  let pg       = null;
  let mtag     = require('path').basename(__filename).slice(0, -3);
  let logger   = new Logger(mtag, {showName: false, debug: true, level: 'info'});
  let db       = new DB(logger);
  let toWoveon = null;// new Woveon(config, logger);

  let teamq = null;

  describe(`${mtag}: A - Confirm calls to woveon get to database.`, function() {
    logger.h1().info(this.title);

    before(async function() {
      await db.init(config.plugin.dburl);
      toWoveon = new Woveon(config, logger, db);
      console.log('...setting pg');
      pg = await dbHelper.getWoveonDBConnection();
      console.log('...connected to pg');

      // ---------------------------------------------------------------------
      // grab first team that matches
      //   (this means we skip all the auth since we grab the channel token here)
      // ---------------------------------------------------------------------
      teamq = await pg.query(`SELECT * FROM teamchannel `+
                                 `WHERE teamid='${teamchannel.teamId}' and `+
                                 `applicationid='${teamchannel.pluginId}' limit 1`);
      // console.log('teamq:', teamq.rows);
      teamchannel.token = teamq.rows[0].token;
      logger.info('teamchannel: ', teamchannel);

    });

    it(`${mtag}: A.1 Make a post and confirm it goes into database.`, async function() {
      logger.h2().info(this.test.title);

      // ---------------------------------------------------------------------
      // generate the post and comment
      // ---------------------------------------------------------------------
      let rawP = Woveon.genPost(teamchannel.token, 'Carl Larson',
        'https://scontent.xx.fbcdn.net/v/t1.0-1/c15.0.50.50/p50x50/10354686_10150004552801856_220367501106153455_n.jpg?oh=8a3549a50c99193e342e22c66b375815&oe=5AC4632F',
        Service.generateRandomString(), 'Facebook', 'Another post to',
        false, [], [], '2017-11-02T13:29:57.000Z');
      let rawC = Woveon.genComment(
        teamchannel.token, 'Brads Local Retail',
        'https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7',
        rawP.postId, Service.generateRandomString(), 'Blah again.',
        '', '', '2017-11-02T13:32:03.000Z');

      logger.info('Creating Post    : ', rawP);
      logger.info('Creating Comment : ', rawC);


      // ---------------------------------------------------------------------
      // send to Woveon
      // ---------------------------------------------------------------------
      let resultP = await toWoveon.createPost(rawP)
        .catch((err) => {console.log(err); console.log(rawP); throw new Error('Error');});
      logger.info('Post result    : ', resultP);
      should.notEqual(resultP.data, null);

      let resultC = await toWoveon._createComment(rawC)
        .catch((err) => {console.log(err); console.log(rawC); throw new Error('Error');});
      logger.info('Comment result  : ', resultC);
      should.notEqual(resultC.data, null);


      // ---------------------------------------------------------------------
      // Query the database to see if it went through
      // ---------------------------------------------------------------------
      await dbHelper.comparePostToDB(pg, rawP, resultP.data.id);
      console.log('compare Post to DB: success');

      await dbHelper.compareCommentToDB(pg, rawC, resultC.data.id);
      console.log('compare Comment to DB: success');

    });


    it(`${mtag}: A.2. Post another, taken from the stream`, async function() {
      logger.h2().info(this.test.title);

      let rndPid = Service.generateRandomString();
      let rndCid = Service.generateRandomString();

      let bulk = {
        posts: [
          {
            token     : teamchannel.token, // 'BPEUE1D5LH9I1JNTX468',
            author    : 'Brads Local Retail',
            authorUrl : 'https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7',
            subject   : 'Facebook',
            isDirect  : false,
            postId    : rndPid,
            message   : 'Look\'n good.',
            images    : ['https://scontent.xx.fbcdn.net/v/t1.0-9/24130022_148634205767512_6312941359864564182_n.png?oh=b019a84234643fcfbca1048bf5684a53&oe=5A8D840E'],
            videos    : [],
            date      : '2017-11-30T19:36:26.000Z',
          },
        ],
        comments: [
          {
            token     : teamchannel.token, // 'BPEUE1D5LH9I1JNTX468',
            author    : 'Brads Local Retail',
            authorUrl : 'https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7',
            postId    : rndPid,
            commentId : rndCid,
            message   : 'testing comment post 11',
            image     : '',
            video     : '',
            date      : '2017-11-30T20:04:45.000Z',
          },
        ],
        likes: [],
      };


      // ---------------------------------------------------------------------
      // send to Woveon
      // ---------------------------------------------------------------------
      let bresult = await toWoveon.bulkUpload(bulk)
        .catch((err) => {console.log(err); console.log(bulk); throw new Error('Error');});
      logger.info('Bulk upload review...   success? ', ((toWoveon.bulkUploadReview(bulk, bresult)==true)? 'false':'true'));
      logger.info('Bulk upload done...');


      // ---------------------------------------------------------------------
      // Query the database to see if it went through
      // ---------------------------------------------------------------------
      logger.info('bresult: ', bresult.posts[0]);
      await dbHelper.comparePostToDB(pg, bulk.posts[0], bresult.posts[0].data.id);
      console.log('compare Post to DB: success');

      await dbHelper.compareCommentToDB(pg, bulk.comments[0], bresult.comments[0].data.id);
      console.log('compare Comment to DB: success');
    });


  });


  after( async function() {
      await pg.end();
  });
};

