

const should            = require('should');
const nock              = require('nock');
const urlParse          = require('url-parse');
const opn               = require('opn');

const Logger            = require('woveon-logger');

const config            = require('../service/config');
const FBPlugin          = require('../plugins/facebook');
const Helper            = require('./helper');


module.exports = () => {

  let mtag = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger(mtag,{debug: true, showName: true});
  let wconfig = JSON.parse(JSON.stringify(config));
  let gconfig = JSON.parse(JSON.stringify(config));
  wconfig.service.port = process.env.WOV_ple_rl_port;
  gconfig.service.port = config.plugin.port;
  let fb = {
    woveonlistener: new FBPlugin.WoveonListener(gconfig,
        {logger: new Logger('fbG',
        {showName: true, debug: true, level: 'verbose'},
        {listener: true, requester: true})}),
    remotelistener: new FBPlugin.RemoteListener(wconfig,
        {logger: new Logger('fbW',
        {showName: true, debug: true, level: 'verbose'},
        {listener: true, requester: true})}),
  };
  let session    = null;
  let newpostid  = 32;
  let channel    = null; // a MongoDB Document

  describe(`${mtag}: Facebook plugin testing`, () => {

    /*
    let ttt_init = fb.remotelistener.init.bind(fb.remotelistener);
    console.log('remotelistener     object init function ', ttt_init.name, fb.remotelistener.prototype);
    console.log('remotelistener     object init function ', fb.remotelistener.init.name, fb.remotelistener.prototype);
    console.log('woveonlistener object auth function ', fb.woveonlistener.auth.name, fb.woveonlistener.prototype);
    console.log('woveonlistener object init function ', fb.woveonlistener.init.name, fb.woveonlistener.prototype);
    */

    // Create testing microservices for Facebook plugin
    before((done) => {
      Helper.modelsClear().then(() => {return fb.remotelistener.init();})
        .catch((err)=>{console.log(err); throw new Error('failed fb.remotelistener.init');})
        .then( () => {return fb.remotelistener.startup();})
        .catch((err)=>{console.log(err); throw new Error('failed fb.remotelistener.startup');})
        .then( () => {return fb.woveonlistener.init();})
        .catch((err)=>{console.log(err); throw new Error('failed fb.woveonlistener.init');})
        .then( () => {return fb.woveonlistener.startup();})
        .catch((err)=>{console.log(err); throw new Error('failed fb.woveonlistener.startup');})
        .then(() => {done();})
        .catch((err) => {console.log(err); throw new Error('Failed setting up testing');});
    });

    describe(`${mtag}: A. Do unit tests to API (using nock) to test outgoing calls.`, () => {

      it(`${mtag}: init session and fb Remote Auth`, function(done) {
        logger.h3().info(this.test.title);

        console.log('timeout changed');
        this.timeout(0); // permanent, until remotelistener restarts
        Helper.initAPISession()
          .catch((err) => { console.log(err); throw new Error('failed Helper.initAPISession'); })
          .then((_session) => {
            session = _session;
            logger.info('session: ', JSON.stringify(session));
            return Helper.emulateBrowserChannelAuth(session);
          })
          .then((_foundchannel)=> { 
            logger.info('found channel 1: ', _foundchannel);
            _foundchannel.data = urlParse(_foundchannel.data, true);
            logger.info('found channel 2: ', _foundchannel);
            channel = new fb.remotelistener.channelModel({token: _foundchannel.data.query.t});
            logger.info('found channel 3: ', channel);
            channel.token.should.equal(_foundchannel.data.query.t);
            return channel.save();
           })
          .catch((err) => { console.log(err); throw new Error('failed channel save'); })
          .then(()=>{done();});
          /*
          .then((_remoteServerURL) => {
            logger.info('remoteServerURL: ', _remoteServerURL.data);
            let url = urlParse(_remoteServerURL.data, true);
            logger.info('url parsed: (need t)', url);
            newchannel = url.query.t; // grab token
            logger.info('newchannel: ', newchannel);
  
            fb.woveonlistener.onPostLaunch = ()=>{
              console.log('gatekeerps launch'); 
              setTimeout( ()=>{done();},1000); // add another delay
            };
  
            // open a browser window to authenticate with Facebook
            opn(_remoteServerURL.data);
          });
          */
      });


      it(`${mtag}: create post`, async function() {
        logger.h3().info(this.test.title);

        logger.info(`${mtag}: create post`);
        let apiEvent = {type: 'createPost', token: channel.token, message: 'a message'};

        nock(`https://graph.facebook.com`).filteringPath(() => {return '/';}).post('/').reply(200, {id: newpostid});

        let r = await fb.woveonlistener.onAPIEvent(apiEvent)
                  .catch((err) => {console.log(err); throw new Error('failed');});

        console.log(`onAPIEvent (${JSON.stringify(apiEvent, null, '  ')}) `+
                    `returned: ${JSON.stringify(r, null, '  ')}.`);
        r.postId.should.be.equal(newpostid);

        console.log('create post returned ', r);
      });


      it(`${mtag}: update post`, async function() {
        logger.h3().info(this.test.title);

        let apiEvent = {
          type    : 'updatePost',
          token   : channel.token,
          postId  : newpostid,
          message : 'a message'};

        logger.info('updatePost with ', apiEvent);
        nock(`https://graph.facebook.com`).filteringPath(() => {return '/';}).post('/')
          .reply(200, {id: newpostid});

        let r = await fb.woveonlistener.onAPIEvent(apiEvent)
                  .catch((err) => {console.log(err); throw new Error('failed');});

        console.log(`onAPIEvent (${JSON.stringify(apiEvent)}) returned: ${JSON.stringify(r)}.`);
        should.notEqual(r, null); // unclear what it souhld be

        console.log('update post returned ', r);
      });


      it(`${mtag}: create comment`);
      it(`${mtag}: update comment`);
      it(`${mtag}: like post`);
      it(`${mtag}: unlike post`);
      it(`${mtag}: like comment`);
      it(`${mtag}: unlike comment`);
      it(`${mtag}: delete comment`);
      it(`${mtag}: delete post`);
    });


    describe(`${mtag}: B. Test incoming calls from simulated Webhook calls`,()=> {
      logger.h2().info(this.title);

      let testdesc = null;

      // A Woveon User has posted on a Facebook page
      // - facebook post on Brads Retail by Brad:
      it(`${mtag}: Woveon User Posted to own Facebook page, with Facebook`, async function() {
        logger.h3().info(this.test.title);

        logger.info(`${mtag}: Woveon User Posted to own Facebook page`);
        let entry = {
          'entry': [{
            'changes': [{
              'field': 'feed',
              'value': {
                'item': 'status', 'sender_name': 'Brads Retail',
                'sender_id': '143382282958865',
                'post_id': '143382282958865_148326885797738',
                'verb': 'add', 'published': 1, 'created_time': 1511189650,
                'message': 'Hi Obi.'
              }
            }],
            'id': '143382282958865', 'time': 1511189652
          }], 'object': 'page'
        };

        // don't handle, since chnanles don't exist. just convert simulated and compare results
        let retval = await FBPlugin.convertToW.convertWebhook(fb.remotelistener, entry, true);
        // let retval = await fb.remotelistener.handleWebhookCall(entry, true);
        logger.info('retval:', retval);

        // token is simulated since not real channel, skip check on it
        retval.posts[0].author.should.equal( entry.entry[0].changes[0].value.sender_name );
        should.equal(retval.posts[0].authorUrl, null);
        retval.posts[0].subject.should.equal( 'Facebook' );
        retval.posts[0].isDirect.should.equal( false );
        retval.posts[0].postId.should.equal( entry.entry[0].changes[0].value.post_id);
        retval.posts[0].message.should.equal( entry.entry[0].changes[0].value.message);
        retval.posts[0].images.length.should.equal( 0 );
        retval.posts[0].videos.length.should.equal( 0 );
        new Date(retval.posts[0].date).getTime().should.be.above( new Date('2017-10-01T19:36:26+0000').getTime() );
        retval.posts[0].verb.should.equal( 'add' );
      });


      // The Woveon user replied to Carl's notification to the Woveon User's page on Facebook
      // I.e. Carl posted on the Woveon user's page, user is responding to them
      it(`${mtag}: Woveon user replies to a customer's post on user's page.`, async function() {
        logger.h3().info(this.test.title);

        let entry = {
          'entry': [{
              'changes': [{
                  'field' : 'feed',
                  'value' : {
                    'verb'         : 'add',
                    'item'         : 'comment',
                    'sender_name'  : 'Brads Retail',
                    'comment_id'   : '143649766265450_148327049131055',
                    'sender_id'    : '143382282958865',
                    'post_id'      : '143382282958865_143649766265450',
                    'parent_id'    : '143382282958865_143649766265450',
                    'created_time' : 1511189713,
                    'message'      : 'reply to carl in notifications.',
                  }}],
            'id'   : '143382282958865',
            'time' : 1511189715,
          }],
          'object': 'page',
        };
        // let retval = await fb.remotelistener.handleWebhookCall(entry, true);
        let retval = await FBPlugin.convertToW.convertWebhook(fb.remotelistener, entry, true);
        logger.info('retval:', retval);

        // token is simulated since not real channel, skip check on it
        retval.comments[0].author.should.equal( entry.entry[0].changes[0].value.sender_name );
        should.equal(retval.comments[0].authorUrl, null);
        retval.comments[0].postId.should.equal( entry.entry[0].changes[0].value.post_id);
        retval.comments[0].commentId.should.equal( entry.entry[0].changes[0].value.comment_id);
        retval.comments[0].message.should.equal( entry.entry[0].changes[0].value.message);
        retval.comments[0].image.should.equal( '' );
        retval.comments[0].video.should.equal( '' );
        new Date(retval.comments[0].date).getTime().should.be.above( new Date('2017-10-01T19:36:26+0000').getTime() );
        retval.comments[0].verb.should.equal( 'add' );
      });


      it(`${mtag}: create comment`);
      it(`${mtag}: update comment`);
      it(`${mtag}: like post`);
      it(`${mtag}: unlike post`);
      it(`${mtag}: like comment`);
      it(`${mtag}: unlike comment`);
      it(`${mtag}: delete comment`);
      it(`${mtag}: delete post`);

    });





    after( async () => {
      logger.info('\n\n************************************************************');
      logger.info(`...shutdown ${mtag}`);
      await fb.remotelistener.shutdown();
      await fb.woveonlistener.shutdown();
      await Helper.modelsClear();
      nock.restore();
      logger.info(`...shutdown ${mtag} ...done`);
      logger.info('************************************************************\n\n');
    });

  });

};
