// Imported Code
const should            = require('should');

// My Actual Code
const Logger            = require('woveon-logger');

const config            = require('../service/config');
const Helper            = require('./helper');
const FBPlugin          = require('../plugins/facebook');


module.exports = (_simulated = true) => {

  let bradslocalchannel = '142118089752457';
  let mtag = require('path').basename(__filename).slice(0, -3);
  let logger   = new Logger('t', {debug: true, showName: true});
  let wconfig = JSON.parse(JSON.stringify(config));
  let gconfig = JSON.parse(JSON.stringify(config));
  wconfig.service.port = process.env.WOV_ple_rl_port;
  gconfig.service.port = config.plugin.port;
  let session = null;
  let newchannel = null;
  let fb = {
    woveonlistener: new FBPlugin.WoveonListener(gconfig,
      {logger: new Logger('fbG',
      {showName: true, debug: true, level: 'info'},
      {listener: false, requester: true})}),
    remotelistener: new FBPlugin.RemoteListener(wconfig,
        {logger: new Logger('fbW',
        {showName: true, debug: true, level: 'verbose'},
        {listener: false, requester: false, convertToW: true})}),
  };

  // block calls to DB to find channel, and use a faked channel

  describe(`${mtag}: Check webhook conversions`, function() {
    logger.h1().info(this.title);

    // setup the service
    before( async () => {
      if ( _simulated) {
      } else {
        await Helper.modelsClear();
        await fb.remotelistener.init();
        await fb.remotelistener.startup();
        await fb.woveonlistener.init();
        await fb.woveonlistener.startup();

        await Helper.initAPISession()
          .catch((err) => {console.log(err); throw new Error('failed Helper.initAPISession');})
          .then((_session) => {
            session = _session;
            logger.info('session: ', JSON.stringify(session));
            return Helper.emulateBrowserChannelAuth(session);
          })
          .then((_remoteServerURL) => {
            // logger.info('remoteServerURL: ', _remoteServerURL.data);
            let url = urlParse(_remoteServerURL.data, true);
            // logger.info('url parsed: (need t)', url);
            newchannel = url.query.t; // grab token
            logger.info('newchannel: ', newchannel);

            fb.woveonlistener.onPostLaunch = () => {
              console.log('gatekeerps launch');
              setTimeout(() => {done();}, 1000); // add another delay
            };

            // open a browser window to authenticate with Facebook
            opn(_remoteServerURL.data);
          });
      }
    });

    describe(`${mtag}: A. Posts`, function() {
      logger.h1().info(this.title);

      it(`${mtag}: A.1. simple Webhook Test of a simulated message`, async function() {
        logger.h2().info(this.test.title);

        let bulk = await FBPlugin.convertToW.convertWebhook(fb.remotelistener, msgSimple, _simulated);
        logger.info('bulk:', JSON.stringify(bulk, null, '  '));
        bulk.posts.length.should.equal(1);
        bulk.posts[0].verb.should.equal('add');
        bulk.posts[0].token.should.equal('test_token');
        bulk.posts[0].author.should.equal('Test Page');
        should.equal(bulk.posts[0].authorUrl, null);
        bulk.posts[0].subject.should.equal('Facebook');
        bulk.posts[0].isDirect.should.equal(false);
        bulk.posts[0].postId.should.equal('44444444_444444444');
        bulk.posts[0].message.should.equal('Example post content.');
        bulk.posts[0].images.length.should.equal(0);
        bulk.posts[0].videos.length.should.equal(0);
        bulk.posts[0].date.should.equal(1512748460000);
      });


      it(`${mtag}: A.2. simple Webhook Test of new post`, async function() {
        logger.h2().info(this.test.title);

        let bulk = await FBPlugin.convertToW.convertWebhook(fb.remotelistener, newPost, _simulated);
        logger.info('bulk:', JSON.stringify(bulk, null, '  '));
        bulk.posts.length.should.equal(1);
        bulk.posts[0].verb.should.equal('add');
        bulk.posts[0].token.should.equal(bradslocalchannel);
        bulk.posts[0].author.should.equal(newPost.entry[0].changes[0].value.sender_name);
        should.equal(bulk.posts[0].authorUrl, null);
        bulk.posts[0].subject.should.equal('Facebook');
        bulk.posts[0].isDirect.should.equal(false);
        bulk.posts[0].postId.should.equal(newPost.entry[0].changes[0].value.post_id);
        bulk.posts[0].message.should.equal(newPost.entry[0].changes[0].value.message);
        bulk.posts[0].images.length.should.equal(0);
        bulk.posts[0].videos.length.should.equal(0);
        bulk.posts[0].date.should.equal(
          newPost.entry[0].changes[0].value.created_time *1000 );

        if (!_simulated) {
          let r = fb.remotelistener.toWoveon.bulkUpload(bulk);
          if (fb.remotelistener.toWoveon.bulkUploadReview(bulk, r)) {
            throw new Error('Failed bulk upload');
          }
        }
      });
    });

    describe(`${mtag}: B. Comments`, function() {
      logger.h1().info(this.title);

      it(`${mtag}: B.1. simple Webhook Test of new comment`, async function() {
        logger.h2().info(this.test.title);

        let bulk = await FBPlugin.convertToW.convertWebhook(fb.remotelistener, newComment, _simulated);
        logger.info('bulk:', JSON.stringify(bulk, null, '  '));
        bulk.posts.length.should.equal(0);
        bulk.comments.length.should.equal(1);
        bulk.comments[0].verb.should.equal('add');
        bulk.comments[0].token.should.equal(bradslocalchannel);
        bulk.comments[0].author.should.equal(newComment.entry[0].changes[0].value.sender_name);
        should.equal(bulk.comments[0].authorUrl, null);
        bulk.comments[0].postId.should.equal(newComment.entry[0].changes[0].value.post_id);
        bulk.comments[0].commentId.should.equal(newComment.entry[0].changes[0].value.comment_id);
        bulk.comments[0].message.should.equal(newComment.entry[0].changes[0].value.message);
        bulk.comments[0].image.should.equal(newComment.entry[0].changes[0].value.photo);
        bulk.comments[0].video.should.equal('');
        bulk.comments[0].date.should.equal(
          newComment.entry[0].changes[0].value.created_time *1000 );

        if (!_simulated) {
          let r = fb.remotelistener.toWoveon.bulkUpload(bulk);
          if (fb.remotelistener.toWoveon.bulkUploadReview(bulk, r)) {
            throw new Error('Failed bulk upload');
          }
        }
      });

      it(`${mtag}: B.2. test comment edit`);
    });


    describe(`${mtag}: C. Like/Reaction`, function() {
      logger.h1().info(this.title);

      it(`${mtag}: C.1. test new like`);
      it(`${mtag}: C.2. test new reaction`);
      it(`${mtag}: C.3. test unlike`);
      it(`${mtag}: C.4. test unreaction`);

    });


    after(`${mtag}: shutdown RemoteListener`, async function() {
      if ( ! _simulated) {
        await fb.remotelistener.shutdown();
        await fb.woveonlistener.shutdown();
      }
    });
  });

};


// Taken from the webhook page : https://developers.facebook.com/apps/1780818332191604/webhooks/
let msgSimple = {
  'entry': [
    {
      'changes': [
        {
          'field' : 'feed',
          'value' : {
            'post_id'      : '44444444_444444444',
            'sender_name'  : 'Test Page',
            'sender_id'    : '1067280970047460',
            'item'         : 'status',
            'verb'         : 'add',
            'published'    : 1,
            'created_time' : 1512748460,
            'message'      : 'Example post content.',
          },
        },
      ],
      'id'   : '0',
      'time' : 1512748460,
    },
  ],
  'object': 'page',
};


// New Post
let newPost =
{
  'entry': [
    {
      'changes': [
        {
          'field' : 'feed',
          'value' : {
            'item'         : 'status',
            'sender_name'  : 'Brads Local Retail',
            'sender_id'    : '142118089752457',
            'post_id'      : '142118089752457_150597538904512',
            'verb'         : 'add',
            'published'    : 1,
            'created_time' : 1512756536,
            'message'      : 'New post.',
          },
        },
      ],
      'id'   : '142118089752457',
      'time' : 1512756537,
    },
  ],
  'object': 'page',
};


// New comment with emoji and image
let newComment = {
  'entry': [
    {
      'changes': [
        {
          'field' : 'feed',
          'value' : {
            'item'         : 'comment',
            'sender_name'  : 'Brads Local Retail',
            'photo'        : 'https://scontent-lga3-1.xx.fbcdn.net/v/t1.0-9/24852089_150597705571162_9120720514263257495_n.jpg?oh=7550c640d207865b3723c5b1c7d320de&oe=5AC162E7',
            'comment_id'   : '150597538904512_150597735571159',
            'sender_id'    : '142118089752457',
            'post_id'      : '142118089752457_150597538904512',
            'verb'         : 'add',
            'parent_id'    : '142118089752457_150597538904512',
            'created_time' : 1512756613,
            'message'      : 'A comment to that new post. with emoji and pic. 😎',
          },
        },
      ],
      'id'   : '142118089752457',
      'time' : 1512756614,
    },
  ],
  'object': 'page',
};


/*
// like of above post
{
  "entry": [
    {
      "changes": [
        {
          "field": "feed",
          "value": {
            "parent_id": "142118089752457_150597538904512",
            "sender_name": "Brads Local Retail",
            "sender_id": "142118089752457",
            "post_id": "142118089752457_150597538904512",
            "verb": "add",
            "item": "like",
            "created_time": 1512756650
          }
        }
      ],
      "id": "142118089752457",
      "time": 1512756650
    }
  ],
  "object": "page"
}


// A like of above comment
{
  "entry": [
    {
      "changes": [
        {
          "field": "feed",
          "value": {
            "parent_id": "142118089752457_150597538904512",
            "sender_name": "Brads Local Retail",
            "comment_id": "150597538904512_150597735571159",
            "sender_id": "142118089752457",
            "post_id": "142118089752457_150597538904512",
            "verb": "add",
            "item": "like",
            "created_time": 1512756678
          }
        }
      ],
      "id": "142118089752457",
      "time": 1512756679
    }
  ],
  "object": "page"
}


// follow-on reaction (all likes have reactions it seems)
{
  "entry": [
    {
      "changes": [
        {
          "field": "feed",
          "value": {
            "reaction_type": "like",
            "parent_id": "142118089752457_150597538904512",
            "sender_name": "Brads Local Retail",
            "comment_id": "150597538904512_150597735571159",
            "sender_id": "142118089752457",
            "post_id": "142118089752457_150597538904512",
            "verb": "add",
            "item": "reaction",
            "created_time": 1512756678
          }
        }
      ],
      "id": "142118089752457",
      "time": 1512756679
    }
  ],
  "object": "page"
}


// A Like on ACTUAL Dave's deals
{
  "entry": [
    {
      "changes": [
        {
          "field": "feed",
          "value": {
            "reaction_type": "like",
            "parent_id": "299217480212331_1150611931739544",
            "sender_name": "Madaline Brayshaw",
            "sender_id": "10210781312063666",
            "post_id": "299217480212331_1150611931739544",
            "verb": "add",
            "item": "reaction",
            "created_time": 1512713076
          }
        }
      ],
      "id": "299217480212331",
      "time": 1512713078
    }
  ],
  "object": "page"
}


// A Reaction
{
  "entry": [
    {
      "changes": [
        {
          "field": "feed",
          "value": {
            "reaction_type": "haha",
            "parent_id": "299217480212331_1150611931739544",
            "sender_name": "Madaline Brayshaw",
            "sender_id": "10210781312063666",
            "post_id": "299217480212331_1150611931739544",
            "verb": "edit",
            "item": "reaction",
            "created_time": 1512713083
          }
        }
      ],
      "id": "299217480212331",
      "time": 1512713084
    }
  ],
  "object": "page"
}
*/
