
const should            = require('should');
const nock              = require('nock');
const urlParse          = require('url-parse');
const opn               = require('opn');
const diff              = require('deep-diff').diff;

const config            = require('../service/config');
const Logger            = require('woveon-logger');

const FBPlugin          = require('../plugins/facebook');

const Helper            = require('./helper');


module.exports = () => {

  let mtag = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger(mtag,{debug: true, showName: true});
  let wconfig = JSON.parse(JSON.stringify(config));
  let gconfig = JSON.parse(JSON.stringify(config));
  wconfig.service.port = process.env.WOV_ple_rl_port;
  gconfig.service.port = config.plugin.port;
  let fb = {
    wovonelistener: new FBPlugin.WoveonListener(gconfig,
        {logger: new Logger('fbG',
        {showName: true, debug: true, level: 'verbose'},
        {listener: true, requester: true})}),
    remotelistener: new FBPlugin.RemoteListener(wconfig,
        {logger: new Logger('fbW',
        {showName: true, debug: true, level: 'verbose'},
        {listener: false, requester: false, convertToW: true})}),
  };
  let session    = null;
  let channel    = null; // a MongoDB Document
  let postarray = null;

  describe(`${mtag}: Facebook plugin data conversion testing`, () => {

    before( async function() {
      this.timeout(0);
      await Helper.modelsClear();
      await fb.remotelistener.init();
      await fb.remotelistener.startup();
      fb.remotelistener.isrunning = false; // turn off runLoop
      await fb.wovonelistener.init();
      await fb.wovonelistener.startup();
      session = await Helper.initAPISession();
      logger.info('before 1');
      let rsurl = await Helper.emulateBrowserChannelAuth(session)
                    .catch( (err)=> { console.log(err); throw err; } );
      logger.info('before 2');
      let url = urlParse(rsurl.data, true);
      logger.info('before 3');
      channelToken = url.query.t; // grab token
      logger.info('before 4', rsurl);

      // open a browser window to authenticate with Facebook
      opn(rsurl.data);
      logger.info('before 5');

      // wait for wovonelistener's onPostLaunch, called when FB is authed in browser
      await new Promise( (r,j) => {
        console.log('reached promise here');
        fb.wovonelistener.onPostLaunch = ()=>{ 
          channel = fb.remotelistener.myChannels[channelToken];
          setTimeout( ()=>{r(true);},1000); };
      });
    });

    describe(`${mtag}: A. pull Facebook posted data and make sure format didn't change.`, function() {
      logger.h2().info(this.title);
    });

    describe(`${mtag}: A. convert Facebook -> Woveon data`, function() {
      logger.h2().info(this.title);

      it(`${mtag}: A.1. pull Facebook post and make sure it didn't change from what we expect.`, async function() {
        logger.h3().info(this.test.title);

        logger.info('channel: ', channel);
        // get a known post (save FB protocol version)
        let post = await fb.remotelistener.toFacebook.get(
          `/142118089752457_148634255767507/`+
          fb.remotelistener._getFeedFields(1)+
          `&access_token=${channel.pluginData.pageToken}`);
        postarray = [post];

        // check for any post changes
        if ( JSON.stringify(postarray) != JSON.stringify(staticPost) ) {
          let diffs = diff(postarray, staticPost);
          logger.h1().error('Expected Post  :', JSON.stringify(staticPost, null, '  '));
          logger.error(     'Retrieved Post :', JSON.stringify(postarray, null, '  '));

          // ignore certain diffs, like paging
          for (let i=diffs.length-1; i>=0; i--) {
            let d = diffs[i];
            if ( d.path.includes('paging') ) {
              logger.info('ignore paging differences...', d);
              diffs.splice(i, 1);
            }
          }

          if ( diffs.length != 0 ) {
            logger.error(diffs);
            throw new Error('Retrieved post not same as expected');
          }
        }

      });

      it(`${mtag}: A.2. Test conversion of message.`, async function() {
        logger.h3().info(this.test.title);

        // logger.info('postarray : ',JSON.stringify(postarray,null,"  "));
        let w_obj = await FBPlugin.convertToW.convertPosts(fb.remotelistener, channel, postarray);
        logger.info('w_obj    : ', JSON.stringify(w_obj, null, '  '));

        // Post tests
        w_obj.posts.length.should.equal(1);
        w_obj.posts[0].token.should.equal(channel.token);
        w_obj.posts[0].author.should.equal(staticPost[0].from.name);
        w_obj.posts[0].authorUrl.should.equal(staticPost[0].from.picture.data.url);
        w_obj.posts[0].subject.should.equal('Facebook');
        w_obj.posts[0].isDirect.should.equal(false);
        w_obj.posts[0].postId.should.equal(staticPost[0].id);
        w_obj.posts[0].message.should.equal(staticPost[0].message);
        w_obj.posts[0].images.length.should.equal(1);
        w_obj.posts[0].images[0].should.equal(staticPost[0].full_picture);
        w_obj.posts[0].videos.length.should.equal(0);
        new Date(w_obj.posts[0].date).getTime().should.be.above( new Date('2017-10-01T19:36:26+0000').getTime() );
        w_obj.posts[0].date.should.equal(new Date(staticPost[0].created_time).getTime());

        // Comment test
        logger.info('  check converted comment 11 : ', w_obj.comments[11]);
        logger.info('  against expected           : ', staticPost[0].comments.data[11]);
        w_obj.comments.length.should.equal(12);
        w_obj.comments[11].token.should.equal(channel.token);
        w_obj.comments[11].author.should.equal(staticPost[0].comments.data[11].from.name);
        w_obj.comments[11].authorUrl.should.equal(staticPost[0].comments.data[11].from.picture.data.url);
        w_obj.comments[11].postId.should.equal(staticPost[0].id);
        w_obj.comments[11].commentId.should.equal(staticPost[0].comments.data[11].id);
        w_obj.comments[11].message.should.equal(staticPost[0].comments.data[11].message);
        w_obj.comments[11].image.should.equal(staticPost[0].comments.data[11].attachment.media.image.src);
        w_obj.comments[11].video.length.should.equal(0);
        /*
        console.log('comment created time:', staticPost[0].comments.data[11].created_time);
        console.log('w_obj comments[11]  :', JSON.stringify(w_obj.comments[11], null, '  '));

        console.log('compare date to date : ', w_obj.comments[11].date);
        console.log('                     : ', staticPost[0].comments.data[11].created_time);
        console.log('                     : ', new Date(staticPost[0].comments.data[11].created_time));
        */

        w_obj.comments[11].date.should.equal(new Date(staticPost[0].comments.data[11].created_time).getTime());
      });


      it(`${mtag}: A.3. Test comment requests.`, async function() {
        this.timeout(10000);
        logger.h3().info(this.test.title);
        // get a known post (save FB protocol version)
        let post = await fb.remotelistener.toFacebook.get(
          `/${staticPost[0].id}/`+ // 142118089752457_148634255767507/
          fb.remotelistener._getFeedFields(1, 2)+
          `&access_token=${channel.pluginData.pageToken}`);
        let w_obj = await FBPlugin.convertToW.convertPosts(fb.remotelistener, channel, [post]);
        logger.info('w_obj    : ', JSON.stringify(w_obj, null, '  '));
        w_obj.comments.length.should.equal(12);

        // Comment test
        w_obj.comments.length.should.equal(12);
        w_obj.comments[11].token.should.equal(channel.token);
        w_obj.comments[11].author.should.equal(staticPost[0].comments.data[11].from.name);
        w_obj.comments[11].authorUrl.should.equal(staticPost[0].comments.data[11].from.picture.data.url);
        w_obj.comments[11].postId.should.equal(staticPost[0].id);
        w_obj.comments[11].commentId.should.equal(staticPost[0].comments.data[11].id);
        w_obj.comments[11].message.should.equal(staticPost[0].comments.data[11].message);
        w_obj.comments[11].image.should.equal(staticPost[0].comments.data[11].attachment.media.image.src);
        w_obj.comments[11].video.length.should.equal(0);
        new Date(w_obj.comments[0].date).getTime().should.be.above( new Date('2017-10-01T19:36:26+0000').getTime() );
        w_obj.comments[11].date.should.equal(new Date(staticPost[0].comments.data[11].created_time).getTime());
      });
    });


    after( async function() {
      await fb.remotelistener.shutdown();
      await fb.wovonelistener.shutdown();
    });

  });


};


// Saved here for faster testing, but should live fetch as well
let staticPost = [
    {
      "id": "142118089752457_148634255767507",
      "created_time": "2017-11-30T19:36:26+0000",
      "message": "Look'n good.",
      "from": {
        "id": "142118089752457",
        "name": "Brads Local Retail",
        "picture": {
          "data": {
            "height": 50,
            "is_silhouette": false,
            "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
            "width": 50
          }
        }
      },
      "link": "https://www.facebook.com/142118089752457/photos/a.148634249100841.1073741827.142118089752457/148634205767512/?type=3",
      "full_picture": "https://scontent.xx.fbcdn.net/v/t1.0-9/24130022_148634205767512_6312941359864564182_n.png?oh=b019a84234643fcfbca1048bf5684a53&oe=5A8D840E",
      "comments": {
        "data": [
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148636229100643",
            "created_time": "2017-11-30T19:50:32+0000",
            "message": "testing pic"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638575767075",
            "created_time": "2017-11-30T20:04:28+0000",
            "message": "testing comment post 2"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638582433741",
            "created_time": "2017-11-30T20:04:30+0000",
            "message": "testing comment post 3"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638595767073",
            "created_time": "2017-11-30T20:04:32+0000",
            "message": "testing comment post 4"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638609100405",
            "created_time": "2017-11-30T20:04:34+0000",
            "message": "testing comment post 5"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638619100404",
            "created_time": "2017-11-30T20:04:36+0000",
            "message": "testing comment post 6"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638629100403",
            "created_time": "2017-11-30T20:04:38+0000",
            "message": "testing comment post 7"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638639100402",
            "created_time": "2017-11-30T20:04:40+0000",
            "message": "testing comment post 8"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638649100401",
            "created_time": "2017-11-30T20:04:41+0000",
            "message": "testing comment post 9"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638659100400",
            "created_time": "2017-11-30T20:04:43+0000",
            "message": "testing comment post 10"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148638665767066",
            "created_time": "2017-11-30T20:04:45+0000",
            "message": "testing comment post 11"
          },
          {
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148646779099588",
            "created_time": "2017-11-30T20:58:10+0000",
            "message": "image comment",
            "attachment": {
              "media": {
                "image": {
                  "height": 250,
                  "src": "https://scontent.xx.fbcdn.net/v/t1.0-9/24129825_148646732432926_3511199427840095175_n.png?oh=d817ade056d44b5baca79a3b814f00a6&oe=5A8CB365",
                  "width": 250
                }
              },
              "target": {
                "id": "148646732432926",
                "url": "https://www.facebook.com/142118089752457/photos/p.148646732432926/148646732432926/?type=3"
              },
              "type": "photo",
              "url": "https://www.facebook.com/142118089752457/photos/p.148646732432926/148646732432926/?type=3"
            },
            "likes": {
              "data": [
                {
                  "id": "142118089752457",
                  "name": "Brads Local Retail"
                }
              ],
              "paging": {
                "cursors": {
                  "before": "MTQyMTE4MDg5NzUyNDU3",
                  "after": "MTQyMTE4MDg5NzUyNDU3"
                }
              }
            }
          }
          /*
          ,
          {
            "from": {
              "id": "10155969434192628",
              "name": "Chadwick Wingrave",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148648125766120",
            "created_time": "2017-11-30T21:07:16+0000",
            "message": "Another comment."
          },
          { 
            "from": {
              "id": "142118089752457",
              "name": "Brads Local Retail",
              "picture": {
                "data": {
                  "height": 50,
                  "is_silhouette": false,
                  "url": "https://scontent.xx.fbcdn.net/v/t1.0-1/p50x50/24862185_150129955617937_4125970342179741459_n.png?oh=11a735eec18609b52fffc8049b594480&oe=5ACD2CE7",
                  "width": 50
                }
              }
            },
            "id": "148634255767507_148646779099588",
            "created_time": "2017-11-30T20:58:10+0000",
            "message": "image comment",
            "attachment": {
              "media": { 
                "image": {
                  "height": 250,
                  "src": "https://scontent.xx.fbcdn.net/v/t1.0-9/24129825_148646732432926_3511199427840095175_n.png?oh=d817ade056d44b5baca79a3b814f00a6&oe=5A8CB365",
                  "width": 250
                }
              },
              "target": {
                "id": "148646732432926",
                "url": "https://www.facebook.com/142118089752457/photos/p.148646732432926/148646732432926/?type=3"
              },
              "type": "photo",
              "url": "https://www.facebook.com/142118089752457/photos/p.148646732432926/148646732432926/?type=3"
            },
            "likes": {
              "data": [
              { 
                "id": "142118089752457",
                "name": "Brads Local Retail"
              }
              ],
              "paging": {
                "cursors": {
                  "before": "MTQyMTE4MDg5NzUyNDU3",
                  "after": "MTQyMTE4MDg5NzUyNDU3"
                }
              }
            }
          }
          */
        ],
        "paging": {
          "cursors": {
            "before": "WTI5dGJXVnVkRjlqZAFhKemIzSTZANVFE0TmpNMk1qSTVNVEF3TmpRek9qRTFNVEl3TnpFME16ST0ZD",
            "after": "WTI5dGJXVnVkRjlqZAFhKemIzSTZANVFE0TmpRNE1USTFOelkyTVRJd09qRTFNVEl3TnpZAd016WT0ZD"
          }
        }
      },
      "reactions": {
        "data": [
          {
            "id": "145901679508935",
            "name": "Brad Braisiey",
            "type": "HAHA"
          },
          {
            "id": "142118089752457",
            "name": "Brads Local Retail",
            "type": "HAHA"
          }
        ],
        "paging": {
          "cursors": {
            "before": "TVRBd01ESXlOamN6TVRJek9UYzNPakUxTVRJd056azFORGM2TnpnNE5qUTRNRE0zT1RFek16RXkZD",
            "after": "TVRReU1URTRNRGc1TnpVeU5EVTNPakUxTVRJd056a3pPRGM2TnpnNE5qUTRNRE0zT1RFek16RXkZD"
          }
        }
      }
    }
  ];
