

const should            = require('should');
const urlParse          = require('url-parse');
const opn               = require('opn');
const starwars          = require('starwars');


const config            = require('../service/config');
const Logger            = require('woveon-logger');

const FBPlugin          = require('../plugins/facebook');

const Helper            = require('./helper');


module.exports = () => {


  console.log('');
  console.log('************************************************');
  console.log('REMEMBER "ngrok http 50106"');
  console.log('REMEMBER to set Facebook webhook location to ngrok targret.');
  console.log('************************************************');
  console.log('');

  let mtag = require('path').basename(__filename).slice(0, -3);
  let logger = new Logger(mtag, {debug: true, showName: true});

  let wconfig = JSON.parse(JSON.stringify(config));
  let gconfig = JSON.parse(JSON.stringify(config));
//  console.log('wovonelistener config: ', JSON.stringify(gconfig));
//  console.log('remotelistener     config: ', JSON.stringify(wconfig));
  wconfig.service.port = process.env.WOV_ple_rl_port;
  gconfig.service.port = config.plugin.port;
//  console.log('wovonelistener config: ', JSON.stringify(gconfig));
//  console.log('remotelistener     config: ', JSON.stringify(wconfig));
  let fb = {
    wovonelistener: new FBPlugin.WoveonListener(gconfig,
        {logger: new Logger('fbG',
        {showName: true, debug: true, level: 'info'},
        {listener: false, requester: true})}),
    remotelistener: new FBPlugin.RemoteListener(wconfig,
        {logger: new Logger('fbW',
        {showName: true, debug: true, level: 'verbose'},
        {listener: true, requester: true})}),
  };
  fb.remotelistener.isrunning = null; // turn off runLoop
  let session    = null;
  let newchannel = null;
  let newpostid  = null;

  let messageOriginal = 'Star Wars:' + starwars();
  let messageNext     = 'Star Wars:' + starwars();


  describe(`${mtag}: Facebook plugin testing to live server.`, function() {
    logger.h3().info(this.title);

    // Create testing microservices for Facebook plugin
    before( async function() {
      await Helper.modelsClear();
      await fb.remotelistener.init();
      await fb.remotelistener.startup();
      await fb.wovonelistener.init();
      await fb.wovonelistener.startup();
    });


    it(`${mtag}: init session and fb Remote Auth`, function(done) {
      logger.h3().info(this.test.title);

      this.timeout(0); // permanent, until remotelistener restarts
      Helper.initAPISession()
        .catch((err) => {console.log(err); throw new Error('failed Helper.initAPISession');})
        .then((_session) => {
          session = _session;
          logger.info('session: ', JSON.stringify(session));
          return Helper.emulateBrowserChannelAuth(session);
        })
        .then((_remoteServerURL) => {
          logger.info('remoteServerURL: ', _remoteServerURL.data);
          let url = urlParse(_remoteServerURL.data, true);
          logger.info('url parsed: (need t)', url);
          newchannel = url.query.t; // grab token
          logger.info('newchannel: ', newchannel);

          fb.wovonelistener.onPostLaunch = ()=>{
            console.log('gatekeerps launch');
            setTimeout( ()=>{done();}, 1000); // add another delay
          };

          // open a browser window to authenticate with Facebook
          opn(_remoteServerURL.data);
        });
    });


    it(`${mtag}: create post`, async function() {
      logger.h3().info(this.test.title);

      let apiEvent = {type: 'createPost', token: newchannel, message: messageOriginal};

      let ret = await fb.wovonelistener.onAPIEvent(apiEvent)
            .catch((err) => {console.log(err); throw new Error('failed');});

      logger.info(`onAPIEvent (${JSON.stringify(apiEvent)}) returned: ${JSON.stringify(ret)}.`);
      ret.postId.should.not.be.equal(null);
      ret.authorUrl.should.not.be.equal('');
      newpostid = ret.postId;

      // ensure wovonelistener.Wov.Posts entry was created
      let r = await fb.wovonelistener.Wov.Posts.find({token: apiEvent.token, remoteId: newpostid});
      logger.info(` - Wov.Posts entry is ${JSON.stringify(r, null, '  ')}`);
      should.notEqual(r, null);
      r.length.should.equal(1);
      r[0].remoteId.should.equal( ret.postId );
      r[0].token.should.equal( apiEvent.token );
    });


    it(`${mtag}: update post`, async function() {
      logger.h3().info(this.test.title);

      let apiEvent = {
        type    : 'updatePost',
        token   : newchannel,
        postId  : newpostid,
        message : messageNext,
      };

      let ret = await fb.wovonelistener.onAPIEvent(apiEvent)
            .catch((err) => {console.log(err); throw new Error('failed');});

      console.log(`onAPIEvent (${JSON.stringify(apiEvent)}) returned: ${JSON.stringify(ret)}.`);
      should.notEqual(ret, null); // unclear what it souhld be
      ret.success.should.equal(true);
      console.log('update post returned ', ret);

    });


    it(`${mtag}: delete post`, async function() {
      logger.h3().info(this.test.title);

      this.timeout(10000);
      let apiEvent = {
        type   : 'deletePost',
        token  : newchannel,
        postId : newpostid,
      };

      let ret = await fb.wovonelistener.onAPIEvent(apiEvent)
            .catch((err) => {console.log(err); throw new Error('failed');});

      console.log(`onAPIEvent (${JSON.stringify(apiEvent)}) returned: ${JSON.stringify(ret)}.`);
      should.notEqual(ret, null); // unclear what it souhld be
      ret.success.should.equal(true);
    });


    it(`${mtag}: create comment`);
    it(`${mtag}: update comment`);
    it(`${mtag}: like post`);
    it(`${mtag}: unlike post`);
    it(`${mtag}: like comment`);
    it(`${mtag}: unlike comment`);
    it(`${mtag}: delete comment`);
    it(`${mtag}: delete post`);
    it(`${mtag}: disable webhook`);


    after( async function() {
      logger.info(`${mtag}: shutting down...`);
      await fb.remotelistener.shutdown();
      await fb.wovonelistener.shutdown();
      logger.info(`${mtag}: shut down complete.`);
    });

  });

};
