
const should            = require('should');
const opn               = require('opn');
const starwars          = require('starwars');
const urlParse          = require('url-parse');

const config            = require('../service/config');
const Logger            = require('woveon-logger');

const FBPlugin          = require('../plugins/facebook');

const Helper            = require('./helper');
const dbHelper          = require('./dbHelper');

module.exports = () => {

  let mtag   = require('path').basename(__filename).slice(0,-3);
  let logger = new Logger(mtag,{debug: true, showName: true});

  let wconfig = JSON.parse(JSON.stringify(config));
  let gconfig = JSON.parse(JSON.stringify(config));
  wconfig.service.port = process.env.WOV_ple_rl_port;
  gconfig.service.port = config.plugin.port;
  let fb = {
    wovonelistener: new FBPlugin.WoveonListener(gconfig,
        {logger: new Logger('fbG',
        {showName: true, debug: true, level: 'info'},
        {listener: false, requester: true})}),
    remotelistener: new FBPlugin.RemoteListener(wconfig,
        {logger: new Logger('fbW',
        {showName: true, debug: true, level: 'verbose'},
        {listener: true, requester: true, sipRSData: true})}),
  };
  fb.remotelistener.isrunning = null; // turn off runLoop, as we call manually
  let session    = null;
  let newchannel = null;
  let newpostid  = null;

  let message_original = starwars();
  let message_next     = starwars();

  describe(`${mtag}: Facebook plugin testing to live server.`, function() {
    logger.h1().info(this.title);

    // Create testing microservices for Facebook plugin
    before( async () => {
      await Helper.modelsClear();
      pg = await dbHelper.getWoveonDBConnection();
      await dbHelper.clearTables(pg); // posts, comments, likes, teamchannel
      await pg.end();

      await fb.remotelistener.init();
      await fb.remotelistener.startup();
      await fb.wovonelistener.init();
      await fb.wovonelistener.startup();
    });


    it(`${mtag}: init session and fb Remote Auth`, async function() {
      logger.h3().info(this.test.title);

      this.timeout(60000); // give time for me to click on browser

      session = await Helper.initAPISession()
        .catch((err) => {console.log(err); throw new Error('failed Helper.initAPISession');});
      logger.info('session: ', JSON.stringify(session));

      let authResponse = await Helper.emulateBrowserChannelAuth(session);
      let url = urlParse(authResponse.data, true);
      newchannel = url.query.t; // grab token
      logger.info('newchannel: ', newchannel);

      await new Promise( (r, j) => {
        fb.wovonelistener.onPostLaunch = ()=>{
          console.log('gatekeerps launch');
          setTimeout(() => {r(true);}, 1000); // add another delay
        };

        // open a browser window to authenticate with Facebook
        opn(authResponse.data);
      });

    });


    it(`${mtag}: runLoop to sip on data`, async function() {
      logger.h3().info(this.test.title);
      let delay = 10000;
      this.timeout(0);
      
      let channel = fb.remotelistener.myChannels[newchannel]; // a model

//      fb.remotelistener.isrunning = true; // start it back up
      fb.remotelistener.manualStartRunLoop();

      logger.with('color', 'blue').info('starting loop with channel : ', JSON.stringify(channel, null, '  '));
      await new Promise( (r, j) => {

        // error out after $delay ms
        setTimeout(() => {j(`Failed to finish in ${delay} ms`);}, delay);

        // check on status function
        let ch = function() {
          setTimeout( () => {
            logger.with('color', 'blue').info('channel continues: ', JSON.stringify(channel, null, '  '));
            if ( fb.remotelistener.activeLoop ) {
              ch(); // call self again
            } else { r(true); } // exit
          }, 1000);
        };

        ch();
      });
    });


  });

  after(`${mtag}: shutdown RemoteListener`, async function() {
    await fb.remotelistener.shutdown();
    await fb.wovonelistener.shutdown();
  });

};