
// ---------------------------------------------------------------------
// This is a series of functions for comparing data to the database.
// ---------------------------------------------------------------------

const Woveon            = require('../service/woveon');
const PG                = require('pg');

// --------------------------------------------------------------------- 
// PG - Make timestamps without timezone assume Z timezone, instead of local
let types = require('pg').types;
const TIMESTAMP_OID = 1114;
let parseFn = function(val) {
//  console.log(`parseFn date string ${val}`);
  return new Date(`${val}Z`);
};
types.setTypeParser(TIMESTAMP_OID, parseFn);
// --------------------------------------------------------------------- 


/**
 * Returns a connection to the Postgres database, using env variables.
 * @return {object}
 */
module.exports.getWoveonDBConnection = async function() {
  const pgconfig = {
    host     : process.env.WOV_postgres_host,
    port     : process.env.WOV_postgres_port_ext,
    user     : process.env.WOV_postgres_username,
    password : process.env.WOV_postgres_password,
    database : process.env.WOV_postgres_database,
  };
  let pg = new PG.Client(pgconfig);

  // Connect to the database
  await pg.connect((err) => {
    if ( err) {
      console.log(err);
      console.log(pgconfig);
      throw new Error('connection err');
    }
  });

  return pg;
};


/**
 * Clears all posts, comments, likes and teams.
 * @param {object} _pg - postgres connection
 */
module.exports.clearTables= async function(_pg) {
  let tables = ['teamchannel', 'post', 'comment', 'likes'];
  let q = null;

  // delete all in each table
  for (let i=0; i<tables.length; i++) {
    q = `delete from ${tables[i]};`;
    await _pg.query(q)
      .catch((err) => {console.log(err); _pg.end(); throw new Error('error');});
  }
};


/**
 * Grab the db representation of the post, call verifyPost on it
 * @param {object} _pg - postgres connection
 * @param {object} _woveonPost -
 * @param {integer}_dbid  -
 * @return {bool} - true on fail
 */
module.exports.comparePostToDB = async function(_pg, _woveonPost, _dbid) {
  let q = `select * from post where id='${_dbid}';`;
  let dbP = await _pg.query(q)
    .catch( (err) => {console.log(err); console.log(_dbid); _pg.end(); throw new Error('error');});
  // console.log(dbP.fields);
  // console.log('Database Rows:', dbP.rows);
  return module.exports.verifyPost(_woveonPost, dbP.rows[0]);
};


/**
 * Grab the db representation of the comment, call verifyComment on it
 * @param {object} _pg - postgres connection
 * @param {object} _woveonComment -
 * @param {integer}_dbid  -
 * @return {bool} - true on fail
 */
module.exports.compareCommentToDB = async function(_pg, _woveonComment, _dbid) {
  let q = `select comment.*, post."postId" FROM comment `+
          `INNER JOIN post on comment.postid::integer = post.id `+
          `WHERE comment.id = '${_dbid}';`;
  let dbC = await _pg.query(q)
    .catch( (err) => {console.log(err); console.log(_dbid); _pg.end(); throw new Error('error');});
  return module.exports.verifyComment(_woveonComment, dbC.rows[0]);
};


/**
 * Checks that what is passed in, is what comes out.
 * @param {object} _woveonPost 
 * @param {object} _dbPostRow 
 * @return {bool} - false... throws error if error
 */
module.exports.verifyPost = function(_woveonPost, _dbPostRow) {
  let keys = Object.keys(Woveon.genPost());
  for ( let i=0; i< keys.length; i++) {
    let k = keys[i];
    if ( _woveonPost.hasOwnProperty(k) && k != 'token' ) {
      let k = keys[i];
      let o = JSON.parse(JSON.stringify(_dbPostRow));
      if ( k == 'images' || k == 'videos' ) { // already stringified data
        o[k].should.equal(JSON.stringify(_woveonPost[k]));
      } else { JSON.stringify(o[k]).should.equal(JSON.stringify(_woveonPost[k])); }
    }
  }
  return false;
};


/**
 * Checks that what is passed in, is what comes out.
 * @param {object} _woveonComment -
 * @param {object} _dbCommentRow -
 * @return {bool} - false... throws error if error
 */
module.exports.verifyComment = function(_woveonComment, _dbCommentRow) {
//  console.log('verify Comment: '); console.log(_woveonComment); console.log(_dbCommentRow);
  let keys = Object.keys(Woveon.genComment());
  for ( let i=0; i< keys.length; i++) {
    let k = keys[i];
    if ( _woveonComment.hasOwnProperty(k) && k != 'token' ) {
      let k = keys[i];
      let o = JSON.parse(JSON.stringify(_dbCommentRow));
      console.log(`  - check key ${k}`);
      if ( k == 'postId' ) {
        o.postId.should.equal(_woveonComment.postId);
      } else {JSON.stringify(o[k]).should.equal(JSON.stringify(_woveonComment[k])); }
    }
  }
  return false;
};

