

// const _ChannelModel     = require('../models/channelConnector');
const WEService         = require('../../src/service');
const Logger            = require('woveon-logger');
const Models            = require('../../src/models');
const DB                = require('../../src/service').DB;


let logger = new Logger('channels'.toUpperCase(),
                  {showName : true, debug : true, level : 'verbose'},
                  {listener : true, requester : true});

let oAuthTokens = {};

/** */
class Channel {

  /**
   * A class to maintain information on each channel in the server.
   * @param {*} _data - information to init the channel. requires
   */
  constructor(_data) {

    if ( ! process.env.WOV_useDB ) {
      // required
      this.tokenId = _data.token; // sprintf("%08d",/*Service.generateRandomString(4),*/Channel._counter); // assume no conflicts
      Channel._counter++;
      this.teamId = _data.teamId;
      //    this.redirectUrl = _data.redirectUrl;
      this.title = _data.title;

      // loaded values
      this.isEnabled  = false; // _data.isEnabled || false;

      this.data = _data;

      // this would be stored on the RemoteServer, as a list of valid tokens
      // to access the data of this channel
      // this.authTokens = [];    // _data.authTokens || [];

      console.log('new channel', this);
      Channel.serverstate.channels[this.tokenId] = this;
    }
    else { console.log('new db channel', this); }
  }

  /**
   *
   */
  async saveToDB() {
    // save to db
    let cm = new Channel.ChannelModel(this.data);
    await cm.save();
  }

  /**
   */
  static async onInit() {
    await WEService.WEConfig.blockForInit();

    Channel.db = new DB(logger);
    WEService.WEConfig.displayMe();
    let fulldburl = `mongodb://${WEService.WEConfig.get('WOV_we_db_url')}/${WEService.WEConfig.get('WOV_we_db_collection')}`;
    await Channel.db.init(fulldburl);
    Channel.ChannelModel = await Models.channelConnector(Channel.db);
  }

  /**
   * 'enabled' means the RemoteService is sending information through.
  */
  enableChannel() { this.isEnabled = true; }
  /** */
  disableChannel() { this.isEnabled = false; }

  /**
   * 'activated' means the channel has been completely set up by a plugin with the RemoteService.
   */
  activateChannel() { this.isActivated = true; }
  /** */
  desactivateChannel() { this.isActivated = false; }


  /**
   * @param {string} _channelToken - token indexing the channel
   * @return {string} - pseudo auth token
  */
  static getOAuthToken(_channelToken) {
    logger.info('getOAuthToken: ', _channelToken);
    if ( oAuthTokens[_channelToken] == undefined ) {
      let oat = WEService.generateRandomString();
      oAuthTokens[_channelToken] = oat;
    }
    let retval = oAuthTokens[_channelToken];
    logger.info('  ... getOAuthToken: ', retval);
    return retval;
  }


  /**
   * Check if the oauth token is valid for the givein channel. TESTING only.
   *
   * @param {string} _channelToken - The channel's token identifier
   * @param {string} _oauthToken - The channel's oauthToken that has been generated for it for testing purposes only.
   * @return {boolean} - true on success, false on failure, when token does not exist in hash
   */
  static hasOAuthToken(_channelToken, _oauthToken) {
    logger.info('hadOAuthToekn : ', _channelToken, _oauthToken);
    logger.info('  : ', oAuthTokens);
    if ( oAuthTokens[_channelToken] == _oauthToken ) return true;
    return false;
  }

};


/**
 * @param {string} _tokenId - token of the channel to get
 * @return {object} - either the object, or Mongoose object of the channel
 */
Channel.getChannel = async (_tokenId ) => {
  let retval = null;
  if ( ! process.env.WOV_useDB ) { retval = Channel.serverstate.channels[_tokenId]; }
  else { retval = await Channel.ChannelModel.findOne({token : _tokenId}); }
  logger.info(`getChannel (${_tokenId}): `, retval);
  return retval;
};

Channel.serverstate = null;

module.exports = Channel;

