

const Channel   = require('./channel');
const WEService = require('../../src/service');

const WovReturn = WEService.WovReturn;
const WEConfig  = WEService.WEConfig;

// const uuidv4 = require('uuid/v4');


/**
 * The testserver emulates the Woveon API during plugin development as well as some fake
 * Remote Server calls such as fake OAUTH and such. Your plugin code should be running
 * on WL, RL and OV ports.
*/
module.exports = {

  logger : null,
  postId : 1,

  data : {posts : {}}, // indexed by xid

  // return to redirect with oauth key
  addRoutes(_server) {


    /** Create user session with Woveon.
    // should return a session object, with the email passed in
    //   id is constant, but xid and session change
     */
    _server.listener.onGet(`/api/${WEConfig.get('WOV_api_ver')}/user/auth`, async (_args) => {
      let retval = WovReturn.checkAttributes(_args, ['mail', 'password']);

      if (retval == null ) {
        let sess = {
          id        : 2, // 2nd is tester
          xid       : WEService.generateToken(),
          session   : WEService.generateRandomString(),
          mail      : _args.mail,
          name      : 'Tester Name',
          banned    : false,
          moderator : false,
          super     : false,
        };
        retval = WovReturn.retSuccess(sess);
      }

      return retval;
    }, __filename);


    // OLD - this was a mistaken route to create a channel, but it is really another channel
    _server.listener.onGet('/auth', async (args, res) => {
      let m = `DEPRECATED: using /api/${WEConfig.get('WOV_api_ver')}/user/protect/team/protect/channel ... as that is what the server actually does`;
      _server.logger.warn(m);
      return this.listener.retError(m);
    }, __filename);


    /**
     * Browser => api - create channel
     * - tells server that user X is going to create a channel for plugin Y
     *   so server returns a token and url to user to contact plugin with
     * - args:
     *    teamId   - identifier of the team that is being created
     *    session  - actually just the session token
     *    sourceId - plugin identifier supplied during plugin development (ex. WOV_plugin_id which is 7 for email)
     *    title    - name of the channel
     */
    _server.listener.onPost(`/api/${WEConfig.get('WOV_api_ver')}/user/protect/team/protect/channel`, async (args, res) => {
      let result = null;
      let retval = WovReturn.checkAttributes(args, ['teamId', 'session', 'sourceId', 'title']);

      // contact WL:/channel_auth
      if ( retval == null ) {
        // let authurl = WEConfig.wl.url+`/api/${WEConfig.get('WOV_api_ver')}/channel_auth`;
        _server.logger.info('CONFIG: ', WEConfig.toString());
        let authurl = `${WEConfig.get('WOV_we_wl_url')}:${WEConfig.get('WOV_we_wl_port')}/${WEConfig.get('WOV_api_ver')}/channel_auth`;

        _server.logger.info('...calling auth on WL: ', authurl);
        result = await _server.requester.get(authurl, {}); // no values passed? hmm...
        _server.logger.info('result: ', result);

        _server.logger.info('should return token, url and isActive', result);
        if ( result.success == false || result.data.url == null || result.data.token == null ) {
          retval = WovReturn.retError(result.data, 'Failed to auth channel: ');
        }
      }

      // create the channel and return a url and channel token
      if ( retval == null ) {
        let a = {
          title  : args.title,
          token  : result.data.token,
          teamId : args.teamId,
        };

        // let c = new Channel(a);
        new Channel(a);
        _server.logger.info(`saving channel (token : ${result.data.token})`);
        // await c.saveToDB();
        retval = WovReturn.retSuccess({url : result.data.url, token : result.data.token});
        _server.logger.info('returning: ', retval.data);
      }

      return retval;
    }, __filename);


    // Browser => WL./remote_auth
    /*
    _server.listener.onGet('/remote_auth', async (args, res) => {
      _server.logger.info('from: ', res.req.originalUrl);
      _server.logger.info('args:', args);
      if ( args.state == null || args.state == '' ) {
        _server.logger.throwError('stateful value needs to be given');
      }
      if ( args.accountId== null || args.accountId == '' ) {
        _server.logger.throwError('accountId needs to be given');
      }

      let c = await Channel.getChannel(args.accountId);
      _server.logger.info('channel ',c);
      if ( c == null ) {_server.logger.throwError('Can not find channel');}
      let r = {success : true};
      _server.logger.info('sending: ', r);
      res.send(r);


      // now handle redirect
      let oauthtoken = c.getOAuthToken();
      let rr = {success : true, token : oauthtoken, state : args.state};
      _server.logger.info('sending to redirect: ', rr, c.redirectUrl);
      let retval = await _server.requester.post(c.redirectUrl, null, rr);

      return retval;
    });
    */


    /**
     * This is RemoteService simulation, requiring :
     * @param {string} t     - channel token
     * @param {string} state - to verify back with plugin
     * @param {string} rsid  - remote service identifier of the plugin (no purpose, just demoinstration reasons)
     * @return {WovReturn}   - redirect to static webpage
     */
    _server.listener.onGet(`/api/${WEConfig.get('WOV_api_ver')}/remote_oauth`, async (_args, _res) => {
      _server.logger.aspect('oauth', 'remote_oauth hit hit');
      _server.logger.aspect('oauth', 'from: ', _res.req.originalUrl);
      _server.logger.aspect('oauth', 'args:', _args);

      let retval = WovReturn.checkAttributes(_args, ['t', 'state', 'rsid']);

      if ( retval == null ) {
        _server.logger.aspect('oauth', 'get channel for oauth');
        let channel = await Channel.getChannel(_args.t);
        _server.logger.info('  - channel ', channel);

        if ( channel == null ) { retval = WovReturn.retError(t, 'Could not find channel with token "t".'); }
      }

      if ( retval == null ) {
        _server.logger.aspect('oauth', 'add the access token to the channel');
        let oat = Channel.getOAuthToken(_args.t);
        _server.logger.aspect('oauth', '  - oauthtoken ', oat);

        // remote service would check the rsid, to see if it is valid with that service (i.e.
        // did Woveon auth with Facebook) and then ask the client if Woveon can have permissions it
        // is asking for. On success, can return redirect_url.
        // returns a token that allows access to this service, for woveon, on behalf of user
        // let url = `${WEConfig.redirectUrl}?oauthtoken=${oat}&state=${_args.state}&token=${_args.t}`;
        // retval = WovReturn.retRedirect(`/oauth.html?oauthtoken=${oat}&state=${_args.state}&token=${_args.t}`);


        // don't return a redirect, but tell where to go, as this was called by a server, not a browser
        // retval = WovReturn.retSuccess({'redirectUrl' : `${WEConfig.get('WOV_api_fullurl')}/oauth.html?oauthtoken=${oat}&state=${_args.state}&token=${_args.t}`});


        // simulated remote service should call back validate
        let url = `http://localhost:50105/${WEConfig.get('WOV_api_ver')}/channel_validatelaunch?oauthtoken=${oat}&state=${_args.state}&token=${_args.t}`;
        _server.logger.aspect('oauth', `validate channel url: POST: ${url}`);

        let result = await _server.requester.post(url);
        if ( result.success == true ) { retval = WovReturn.retSuccess(result.data); }
        else { retval = WovReturn.retError(result.data); }
        WovReturn.addMeta(retval, result.meta);

        // retval = await _server.requesterHtml.get(url);
        _server.logger.aspect('oauth', '  ... validate channel url returned: ', retval);
        _server.logger.aspect('  ... validate channel url returned: ', retval);
      }

      _server.logger.info('...returning from remote_oauth : ', retval);
      return retval;
    }, __filename);

    // RL => RemoteService./enable_channel
    // RL => RemoteService./connect_channel
    // _server.listener.onPost('/connect_channel', connectChannel, __filename);
    // _server.listener.onPost('/enable_channel', connectChannel, __filename);

    /**
     * @param {object} args -
     * @return {WovReturn} -
     */
    /*
    async function connectChannel(args) {
      _server.logger.info('args:', args);
      let retval = null;
      let c = null;

      if ( args.channel == null || args.channel == '' ) {
        retval = new Error('token needs to be given');
      }

      if ( retval == null ) {
        c = await Channel.getChannel(args.channel );
        if ( c == null ) retval = new Error('Can not find channel');
      }

      if ( retval == null ) {
        if ( args.oauth_token != null && Channel.hasOAuthToken(args.channel, args.oauth_token) ) {
          if ( c.enableChannel ) c.enableChannel();
          // NOTE: for database, won't have enableChannel, but don't save anything because the API database at woveon tracks activated channels with a boolean, not in particular states.
        } else {retval = new Error('Could not find OAuth token');}
      }

      if ( retval == null ) return WovReturn.retSuccess(true);
      else return WovReturn.retError(retval, 'Failed connectChannel');
    };
    */


    _server.listener.onPost(`/rs/${WEConfig.get('WOV_api_ver')}/hook_start`, channelStartHook, __filename);

    /**
     * Start the channel listening via this hook.
     *
     * NOTE: this is called by WL to test the credentials
     *
     */
    async function channelStartHook(_args) {
      let retval = WovReturn.checkAttributes(_args, ['oauthtoken', 'token']);
      let channel = null;

      if ( retval == null ) {
        _server.logger.aspect('hook.start', 'get channel');
        channel = await Channel.getChannel(_args.token);
        if ( channel == null ) { retval = WovReturn.retError(_args.token, 'Could not find channel with token'); }
      }

      if ( retval == null ) {
        _server.logger.aspect('hook.start', 'test state and oauthtoken');
        if ( Channel.hasOAuthToken(_args.token, _args.oauthtoken) == false ) { retval = WovReturn.retError(_args.oauthtoken, 'Could not match oauthtoken'); }
      }

      if ( retval == null ) { 
        _server.logger.aspect('hook.start', 'start hook: ', _args.token);
        if ( channel.enableChannel ) channel.enableChannel();
        retval = WovReturn.retSuccess(true); 
      }

      return retval;
    }


    // RL => api./activate_channel
    // args has applicationid, appsecret (of the app) and token (of the channel)
    _server.listener.onPut(`/api/${WEConfig.get('WOV_api_ver')}/application/protect`, async (args) => {
      let retval = null;

      _server.logger.info('args:', args);

      // check appsecret of applicationid matches... skipping

      // set channel active
      let c = await Channel.getChannel(args.token);
      if ( c == null ) retval = WovReturn.retError(args.token, 'Can not find channel with token');

      if ( retval == null ) {
        _server.logger.info('ok, set it active, save it back and return');
        _server.logger.info('channel : ', c);
        if ( c.activateChannel ) c.activateChannel();
        else {
          c.isActive = 'running';
          c.markModified('isActive');
          await c.save();
        }
        _server.logger.info('... saved', c);
      }

      if ( retval == null ) retval = WovReturn.retSuccess(true);

      return retval;
    }, __filename);


    /**
     * Searches by xid and token for a post. Does not check user/session/security.
     * @return {WovReturn} -
     */
    _server.listener.onGet(`/api/${WEConfig.get('WOV_api_ver')}/application/protect/posts`,    async (args, res) => {
      _server.logger.info('messages GET');
      let retval = WovReturn.checkAttributes(args, {token : true, xid : true, appsecret : true, applicationid : true});

      if ( retval == null ) {
        let result = this.data.posts[args.token];
        _server.logger.warn('bad token for get of post');
        if ( result == null ) { retval = WovReturn.retError(args, 'Could not find channel'); }
      }

      if ( retval == null ) {
        let result = this.data.posts[args.token][args.xid];
        if ( result == null ) { retval = WovReturn.retError(args, 'Could not find post'); }
        else { retval = WovReturn.retSuccess(result); }
      }
      return retval;
    },    __filename);

    // Res 'Post' ping test basically
    _server.listener.onPut(`/api/${WEConfig.get('WOV_api_ver')}/application/protect/posts`,    async (args, res) => {
      let post = null;

      _server.logger.info('messages PUT');

      let retval = WovReturn.checkAttributes(args, {

        // credentials - not checking in testserver
        'applicationid' : true,
        'appsecret'     : true,

        // identifiers, required for creating
        'xid'       : true,
        'xidTop'    : true,
        'xidParent' : false, // set to false since can be null, and some clients will remove null values from JSON.. I know right?
        'resInc'    : true,
        'token'     : true,

        // possible data of post
        'message'     : false,
        'date'        : false,
        'rsraw'       : false,
        'images'      : false,
        'videos'      : false,
        'subject'     : false,
        'link'        : false,
        'author'      : false,
        'authorUrl'   : false,
        'contentType' : false,

        // data created by server
        'recipient'   : false, // channel.title
      });

      if ( retval == null ) {
        post = this.data.posts[args.token][args.xid];
        if ( post == null ) { WovReturn.retError({'token' : args.token, 'xid' : args.xid}, 'Bad identifiers'); }
      }

      if ( retval == null ) {
        if ( args.message     ) post.message     = args.message;
        if ( args.date        ) post.date        = args.date;
        if ( args.rsraw       ) post.rsraw       = args.rsraw;
        if ( args.images      ) post.images      = args.images;
        if ( args.videos      ) post.videos      = args.videos;
        if ( args.subject     ) post.subject     = args.subject;
        if ( args.link        ) post.link        = args.link;
        if ( args.author      ) post.author      = args.author;
        if ( args.authorUrl   ) post.authorUrl   = args.authorUrl;
        if ( args.contentType ) post.contentType = args.contentType;

        this.data.posts[args.token][args.xid] = post;

        retval = WovReturn.retSuccess(post);
      }

      return retval;
    }, __filename);


    /**
     */
    _server.listener.onDelete(`/api/${WEConfig.get('WOV_api_ver')}/application/protect/posts`, async (args, res) => {
      _server.logger.info('messages DELETE');
      let retval = WovReturn.checkAttributes(args, {token : true, xid : true, appsecret : true, applicationid : true});

      if ( retval == null ) {
        let result = this.data.posts[args.token];
        _server.logger.warn('bad token');
        if ( result == null ) { retval = WovReturn.retError(args, 'Could not find channel'); }
      }

      if ( retval == null ) {
        let result = this.data.posts[args.token][args.xid];
        if ( result == null ) { retval = WovReturn.retError(args, 'Could not find post to delete'); }
      }

      if ( retval == null ) {
        delete this.data.posts[args.token][args.xid];
        retval = WovReturn.retSuccess(true);
      }

      return retval;
    }, __filename);

    /**
     * This calls apirest's core/models/Post.sj:createPostByApplication to create the post
     */
    _server.listener.onPost(`/api/${WEConfig.get('WOV_api_ver')}/application/protect/posts`,   async (args, res) => {
      _server.logger.info('messages POST');

      let retval = WovReturn.checkAttributes(args, {
        // authenticating data
        'applicationid' : true,
        'appsecret'     : true,

        // identifiers, required for creating
        'xid'       : true,
        'xidTop'    : true,
        'xidParent' : true,
        'token'     : true,

        // required data of post
        'message' : true,
        'date'    : true,
        'rsraw'   : true,

        // additional data of post
        'images'      : false,
        'videos'      : false,
        'subject'     : false,
        'link'        : false,
        'author'      : false,
        'authorUrl'   : false,
        'contentType' : false,

        // keys
        'resInc'      : true,
        'resParentId' : false,
        'postId'      : false, // hard to say what this is
        'commentId'   : false,
        'recipient'   : false, // channel.title
      });

      if ( retval == null ) {
        let channel = Channel.getChannel(args.token); // args.token is channel token
        _server.logger.info('channel ', channel);
        let result = {
          xid           : args.xid,
          teamchannelid : channel.teamId,
          images        : args.images || [],
          videos        : args.videos || [],
          isMedia       : ((args.images && args.images.length > 0) || (args.videos && args.videos.length > 0))?true:false,
          message       : args.message,
          subject       : args.subject,
          unRead        : true,
          postId        : this.postId++,
          recipient     : channel.title,
          link          : args.link,
          author        : args.author,
          authorUrl     : args.authorUrl,
          date          : args.date,
          contentType   : args.contentType,
          rsraw         : args.rsraw,
          priority      : .5,
          priority_u    : null,
        };

        // store post by xid
        this.data.posts[args.token] = this.data.posts[args.token] || {};
        this.data.posts[args.token][result.xid] = result;
        retval = WovReturn.retSuccess(result);
      }

      return retval;
    },  __filename);

  },
};
